@@ -28,7 +28,8 @@ typedef struct
   const gchar *padname;
   GstPad *target;
   GstElement *bin;
-} dyn_link;
+}
+dyn_link;
 
 static GstElement *
 gst_element_factory_make_or_warn (gchar * type, gchar * name)
@@ -572,7 +573,8 @@ typedef struct
 {
   const gchar *name;
   const GstFormat format;
-} seek_format;
+}
+seek_format;
 
 static seek_format seek_formats[] = {
   {"tim", GST_FORMAT_TIME},
@@ -808,7 +810,8 @@ typedef struct
 {
   gchar *name;
   GstElement *(*func) (const gchar * location);
-} Pipeline;
+}
+Pipeline;
 
 static Pipeline pipelines[] = {
   {"mp3", make_mp3_pipeline},
