@@ -31,6 +31,17 @@
 #include "gstalsaclock.h"
 #include "gstalsamixer.h"
 
+#define ALSA_DEBUG_FLUSH(this) G_STMT_START{ \
+  gchar *__str; \
+  ssize_t __size; \
+  __size = snd_output_buffer_string (this->out, &__str); \
+  if (__size > 0) { \
+    GST_DEBUG_OBJECT (this, "%*s", __size, __str); \
+    if (snd_output_flush (this->out) != 0) \
+      GST_ERROR_OBJECT (this, "error flushing output buffer"); \
+  } \
+}G_STMT_END
+
 /* GObject functions */
 static void gst_alsa_class_init (gpointer g_class, gpointer class_data);
 static void gst_alsa_init (GstAlsa * this);
@@ -312,6 +323,68 @@ gst_alsa_probe_get_properties (GstPropertyProbe * probe)
   return list;
 }
 
+static void
+device_list (snd_pcm_stream_t stream, GstAlsaClass * klass)
+{
+  snd_ctl_t *handle;
+  int card, err, dev;
+  snd_ctl_card_info_t *info;
+  snd_pcm_info_t *pcminfo;
+  gboolean mixer = (stream == -1);
+
+  if (stream == -1)
+    stream = 0;
+
+  snd_ctl_card_info_alloca (&info);
+  snd_pcm_info_alloca (&pcminfo);
+  card = -1;
+
+  if (snd_card_next (&card) < 0 || card < 0) {
+    /* no soundcard found */
+    return;
+  }
+  while (card >= 0) {
+    char name[32];
+
+    sprintf (name, "hw:%d", card);
+    if ((err = snd_ctl_open (&handle, name, 0)) < 0) {
+      goto next_card;
+    }
+    if ((err = snd_ctl_card_info (handle, info)) < 0) {
+      snd_ctl_close (handle);
+      goto next_card;
+    }
+
+    if (mixer) {
+      klass->devices = g_list_append (klass->devices, g_strdup (name));
+    } else {
+      dev = -1;
+      while (1) {
+        gchar *gst_device;
+
+        snd_ctl_pcm_next_device (handle, &dev);
+
+        if (dev < 0)
+          break;
+        snd_pcm_info_set_device (pcminfo, dev);
+        snd_pcm_info_set_subdevice (pcminfo, 0);
+        snd_pcm_info_set_stream (pcminfo, stream);
+        if ((err = snd_ctl_pcm_info (handle, pcminfo)) < 0) {
+          continue;
+        }
+
+        gst_device = g_strdup_printf ("hw:%d,%d", card, dev);
+        klass->devices = g_list_append (klass->devices, gst_device);
+      }
+    }
+    snd_ctl_close (handle);
+  next_card:
+    if (snd_card_next (&card) < 0) {
+      break;
+    }
+  }
+}
+
 static gboolean
 gst_alsa_class_probe_devices (GstAlsaClass * klass, gboolean check)
 {
@@ -322,25 +395,21 @@ gst_alsa_class_probe_devices (GstAlsaClass * klass, gboolean check)
    * do function-wise look-ups. */
 
   if (!init && !check) {
-#define MAX_DEVICES 16          /* random number */
-    gint num, res;
-    gchar *dev;
-    snd_pcm_t *pcm;
+    snd_pcm_stream_t mode = -1;
+    const GList *templates;
 
-    for (num = 0; num < MAX_DEVICES; num++) {
-      dev = g_strdup_printf ("hw:%d", num);
-
-      if (!(res = snd_pcm_open (&pcm, dev, 0, SND_PCM_NONBLOCK)) ||
-          res == -EBUSY) {
-        klass->devices = g_list_append (klass->devices, dev);
-
-        if (res != -EBUSY)
-          snd_pcm_close (pcm);
-      } else {
-        g_free (dev);
-      }
+    /* we assume one pad template at max [zero=mixer] */
+    templates =
+        gst_element_class_get_pad_template_list (GST_ELEMENT_CLASS (klass));
+    if (templates) {
+      if (GST_PAD_TEMPLATE_DIRECTION (templates->data) == GST_PAD_SRC)
+        mode = SND_PCM_STREAM_CAPTURE;
+      else
+        mode = SND_PCM_STREAM_PLAYBACK;
     }
 
+    device_list (mode, klass);
+
     init = TRUE;
   }
 
@@ -634,9 +703,20 @@ add_channels (GstStructure * structure, gint min_rate, gint max_rate,
     min_rate = GST_ALSA_MIN_RATE;
     max_rate = GST_ALSA_MAX_RATE;
   }
-  if (max_rate < 0) {
+  if (max_rate < 0 || min_rate == max_rate) {
     gst_structure_set (structure, "rate", G_TYPE_INT, min_rate, NULL);
   } else {
+    /* just to be sure */
+    if (min_rate > max_rate) {
+      gint temp;
+
+      GST_ERROR
+          ("minimum rate > maximum rate (%d > %d), please fix your soundcard drivers",
+          min_rate, max_rate);
+      temp = min_rate;
+      min_rate = max_rate;
+      max_rate = temp;
+    }
     gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, min_rate,
         max_rate, NULL);
   }
@@ -644,9 +724,20 @@ add_channels (GstStructure * structure, gint min_rate, gint max_rate,
     min_channels = 1;
     max_channels = GST_ALSA_MAX_CHANNELS;
   }
-  if (max_channels < 0) {
+  if (max_channels < 0 || min_channels == max_channels) {
     gst_structure_set (structure, "channels", G_TYPE_INT, min_channels, NULL);
   } else {
+    /* just to be sure */
+    if (min_channels > max_channels) {
+      gint temp;
+
+      GST_ERROR
+          ("minimum channels > maximum channels (%d > %d), please fix your soundcard drivers",
+          min_channels, max_channels);
+      temp = min_channels;
+      min_channels = max_channels;
+      max_channels = temp;
+    }
     gst_structure_set (structure, "channels", GST_TYPE_INT_RANGE,
         min_channels, max_channels, NULL);
   }
@@ -689,6 +780,7 @@ gst_alsa_caps (snd_pcm_format_t format, gint rate, gint channels)
     }
   }
 
+  gst_caps_do_simplify (ret_caps);
   return ret_caps;
 }
 
@@ -759,45 +851,128 @@ gst_alsa_get_caps (GstPad * pad)
     GST_WARNING_OBJECT (this, "no supported caps found, returning empty caps");
     return gst_caps_new_empty ();
   } else {
-    G_GNUC_UNUSED gchar *str = gst_caps_to_string (ret);
+    G_GNUC_UNUSED gchar *str;
 
+    gst_caps_do_simplify (ret);
+    str = gst_caps_to_string (ret);
     GST_LOG_OBJECT (this, "get_caps returns %s", str);
     g_free (str);
     return ret;
   }
 }
 
+static GstCaps *
+gst_alsa_fixate_to_mimetype (const GstCaps * caps, const gchar * mime)
+{
+  GstCaps *try, *result;
+
+  try = gst_caps_new_simple (mime, NULL);
+  result = gst_caps_intersect (try, caps);
+  gst_caps_free (try);
+  if (gst_caps_is_empty (result)) {
+    gst_caps_free (result);
+    return NULL;
+  }
+  if (gst_caps_is_subset (caps, result)) {
+    /* we didn't reduce caps */
+    gst_caps_free (result);
+    return NULL;
+  }
+  return result;
+}
+
+static GstCaps *
+gst_alsa_fixate_field_nearest_int (const GstCaps * caps,
+    const gchar * field_name, gint target)
+{
+  guint i;
+  GstCaps *result;
+  GstCaps *smaller = gst_caps_new_empty ();
+  GstCaps *equal = gst_caps_new_empty ();
+  GstCaps *bigger = gst_caps_new_empty ();
+
+  /* works like this: we fixate every structure and put them into one of those 
+   * caps depending on what we fixated to. We then return the best caps that is 
+   * not empty in the following order: equal, bigger, smaller 
+   * We also make sure the caps were really reduced.
+   */
+  for (i = 0; i < gst_caps_get_size (caps); i++) {
+    gint fixated_to;
+    GstStructure *copy = gst_structure_copy (gst_caps_get_structure (caps, i));
+
+    gst_caps_structure_fixate_field_nearest_int (copy, field_name, target);
+    if (gst_structure_get_int (copy, field_name, &fixated_to)) {
+      if (fixated_to == target) {
+        gst_caps_append_structure (equal, copy);
+      } else if (fixated_to > target) {
+        gst_caps_append_structure (bigger, copy);
+      } else {
+        gst_caps_append_structure (smaller, copy);
+      }
+    } else {
+      /* FIXME: what do we do here? Add to all or throw an error? */
+      g_return_val_if_reached (NULL);
+    }
+  }
+  if (!gst_caps_is_empty (equal)) {
+    gst_caps_free (bigger);
+    gst_caps_free (smaller);
+    result = equal;
+  } else {
+    gst_caps_free (equal);
+    if (!gst_caps_is_empty (bigger)) {
+      gst_caps_free (smaller);
+      result = bigger;
+    } else {
+      gst_caps_free (bigger);
+      if (gst_caps_is_empty (smaller)) {
+        gst_caps_free (smaller);
+        return NULL;
+      }
+      result = smaller;
+    }
+  }
+  if (gst_caps_is_subset (caps, result)) {
+    /* we didn't reduce caps */
+    gst_caps_free (result);
+    return NULL;
+  }
+  return result;
+}
+
 GstCaps *
 gst_alsa_fixate (GstPad * pad, const GstCaps * caps)
 {
-  GstCaps *newcaps;
-  GstStructure *structure;
+  GstCaps *result;
+  const gchar *mime;
 
-  newcaps =
-      gst_caps_new_full (gst_structure_copy (gst_caps_get_structure (caps, 0)),
-      NULL);
-  structure = gst_caps_get_structure (newcaps, 0);
+  if ((result = gst_alsa_fixate_to_mimetype (caps, "audio/x-raw-int")))
+    return result;
+  if ((result = gst_alsa_fixate_to_mimetype (caps, "audio/x-raw-float")))
+    return result;
+  if ((result = gst_alsa_fixate_to_mimetype (caps, "audio/x-alaw")))
+    return result;
+  if ((result = gst_alsa_fixate_to_mimetype (caps, "audio/x-mulaw")))
+    return result;
 
-  if (gst_caps_structure_fixate_field_nearest_int (structure, "rate", 44100)) {
-    return newcaps;
-  }
-  if (gst_caps_structure_fixate_field_nearest_int (structure, "channels", 2)) {
-    return newcaps;
-  }
-  if (strcmp (gst_structure_get_name (structure), "audio/x-raw-int") == 0) {
-    if (gst_caps_structure_fixate_field_nearest_int (structure, "depth", 16)) {
-      return newcaps;
-    }
-    if (gst_caps_structure_fixate_field_nearest_int (structure, "width", 16)) {
-      return newcaps;
-    }
-  } else {
-    if (gst_caps_structure_fixate_field_nearest_int (structure, "width", 32)) {
-      return newcaps;
-    }
-  }
+  /* now we know there's only one mimetype in the caps */
+  /* FIXME: I should check this to be really sure I didn't mess up somewhere */
 
-  gst_caps_free (newcaps);
+  if ((result = gst_alsa_fixate_field_nearest_int (caps, "rate", 44100)))
+    return result;
+  if ((result = gst_alsa_fixate_field_nearest_int (caps, "channels", 2)))
+    return result;
+
+  mime = gst_structure_get_name (gst_caps_get_structure (caps, 0));
+  if (g_str_equal (mime, "audio/x-raw-int")) {
+    if ((result = gst_alsa_fixate_field_nearest_int (caps, "width", 16)))
+      return result;
+    if ((result = gst_alsa_fixate_field_nearest_int (caps, "depth", 16)))
+      return result;
+  } else if (g_str_equal (mime, "audio/x-raw-float")) {
+    if ((result = gst_alsa_fixate_field_nearest_int (caps, "width", 32)))
+      return result;
+  }
 
   return NULL;
 }
@@ -1113,6 +1288,7 @@ static gboolean
 gst_alsa_open_audio (GstAlsa * this)
 {
   snd_pcm_info_t *info;
+  int ret;
 
   g_assert (this != NULL);
   g_assert (this->handle == NULL);
@@ -1124,17 +1300,37 @@ gst_alsa_open_audio (GstAlsa * this)
 
   GST_INFO ("Opening alsa device \"%s\"...", this->device);
 
-#if 0
-  /* enable this to get better debugging */
-  ERROR_CHECK (snd_output_stdio_attach (&this->out, stderr, 0),
+  ERROR_CHECK (snd_output_buffer_open (&this->out),
       "error opening log output: %s");
-#endif
 
-  if (snd_pcm_open (&this->handle, this->device,
-          GST_ALSA_GET_CLASS (this)->stream, SND_PCM_NONBLOCK) < 0) {
-    GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
-        (_("Alsa device \"%s\" is already in use by another program."),
-            this->device), (NULL));
+  if ((ret = snd_pcm_open (&this->handle, this->device,
+              GST_ALSA_GET_CLASS (this)->stream, SND_PCM_NONBLOCK)) < 0) {
+    /* ALSA inverts standard errno.h error codes */
+    switch (-ret) {
+      case EBUSY:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+            (_("ALSA device \"%s\" is already in use by another program."),
+                this->device), (NULL));
+        break;
+      case EACCES:
+      case ETXTBSY:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, OPEN_READ_WRITE,
+            (_("Could not access ALSA device \"%s\", check its permissions."),
+                this->device), GST_ERROR_SYSTEM);
+        break;
+
+      case ENXIO:
+      case ENODEV:
+      case ENOENT:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+            (_("ALSA device \"%s\" does not exist."), this->device), (NULL));
+        break;
+      default:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+            (_("ALSA device \"%s\" had an error."),
+                this->device), ("ALSA error %d: %s", ret, snd_strerror (ret)));
+        break;
+    }
     return FALSE;
   }
 
@@ -1147,14 +1343,28 @@ gst_alsa_open_audio (GstAlsa * this)
   return TRUE;
 }
 
+void
+gst_alsa_sw_params_dump (GstAlsa * this, snd_pcm_sw_params_t * sw_params)
+{
+  snd_pcm_sw_params_dump (sw_params, this->out);
+  ALSA_DEBUG_FLUSH (this);
+}
+
+void
+gst_alsa_hw_params_dump (GstAlsa * this, snd_pcm_hw_params_t * hw_params)
+{
+  snd_pcm_hw_params_dump (hw_params, this->out);
+  ALSA_DEBUG_FLUSH (this);
+}
+
 /* if someone finds an easy way to merge this with _set_hw_params, go ahead */
 static gboolean
 gst_alsa_probe_hw_params (GstAlsa * this, GstAlsaFormat * format)
 {
   snd_pcm_hw_params_t *hw_params;
-  snd_pcm_access_mask_t *mask;
   snd_pcm_uframes_t period_size;
   unsigned int period_count;
+  unsigned int rate;
 
   g_return_val_if_fail (this != NULL, FALSE);
   g_return_val_if_fail (format != NULL, FALSE);
@@ -1164,35 +1374,38 @@ gst_alsa_probe_hw_params (GstAlsa * this, GstAlsaFormat * format)
 
   snd_pcm_hw_params_alloca (&hw_params);
   SIMPLE_ERROR_CHECK (snd_pcm_hw_params_any (this->handle, hw_params));
-  SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_periods_integer (this->handle,
-          hw_params));
 
-#if 0
-  /* enable this for soundcard specific debugging */
-  snd_pcm_hw_params_dump (hw_params, this->out);
-#endif
+  gst_alsa_hw_params_dump (this, hw_params);
 
-  mask = alloca (snd_pcm_access_mask_sizeof ());
-  snd_pcm_access_mask_none (mask);
   if (GST_ELEMENT (this)->numpads == 1) {
-    snd_pcm_access_mask_set (mask,
-        this->
-        mmap ? SND_PCM_ACCESS_MMAP_INTERLEAVED : SND_PCM_ACCESS_RW_INTERLEAVED);
+    SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_access (this->handle,
+            hw_params, this->
+            mmap ? SND_PCM_ACCESS_MMAP_INTERLEAVED :
+            SND_PCM_ACCESS_RW_INTERLEAVED));
   } else {
-    snd_pcm_access_mask_set (mask,
-        this->
-        mmap ? SND_PCM_ACCESS_MMAP_NONINTERLEAVED :
-        SND_PCM_ACCESS_RW_NONINTERLEAVED);
+    SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_access (this->handle,
+            hw_params, this->
+            mmap ? SND_PCM_ACCESS_MMAP_NONINTERLEAVED :
+            SND_PCM_ACCESS_RW_NONINTERLEAVED));
   }
-  SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_access_mask (this->handle,
-          hw_params, mask));
 
   SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_format (this->handle, hw_params,
           format->format));
   SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_channels (this->handle, hw_params,
           format->channels));
-  SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_rate (this->handle, hw_params,
-          format->rate, 0));
+  /* FIXME: We should use snd_pcm_hw_params_set_rate instead of 
+   * snd_pcm_hw_params_set_rate_near here. Unfortunately alsa fails in that case
+   * far more often and seems to handle this quite well. (Example: ENS1371 
+   * driver on alsalib 1.0.5, kernel 2.6.6-mm5). If it sets far too wrong sample
+   * rates, we need to revert back to snd_pcm_hw_params_set_rate or check the
+   * rate that was set.
+   */
+  rate = format->rate;
+  SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_rate_near (this->handle, hw_params,
+          &rate, 0));
+  if (rate != format->rate)
+    GST_WARNING_OBJECT (this, "set rate (%u) differs from desired rate (%u)",
+        rate, format->rate);
 
   period_count = this->period_count;
   SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle,
@@ -1213,62 +1426,63 @@ static gboolean
 gst_alsa_set_hw_params (GstAlsa * this)
 {
   snd_pcm_hw_params_t *hw_params;
-  snd_pcm_access_mask_t *mask;
+  unsigned int rate;
 
   g_return_val_if_fail (this != NULL, FALSE);
   g_return_val_if_fail (this->handle != NULL, FALSE);
 
+  snd_pcm_hw_params_alloca (&hw_params);
+  ERROR_CHECK (snd_pcm_hw_params_any (this->handle, hw_params),
+      "Broken configuration for this PCM: %s");
+
   if (this->format) {
     GST_INFO ("Preparing format: %s %dHz, %d channels",
         snd_pcm_format_name (this->format->format), this->format->rate,
         this->format->channels);
-  } else {
-    GST_INFO ("Preparing format: (none)");
-  }
 
-  snd_pcm_hw_params_alloca (&hw_params);
-  ERROR_CHECK (snd_pcm_hw_params_any (this->handle, hw_params),
-      "Broken configuration for this PCM: %s");
-  ERROR_CHECK (snd_pcm_hw_params_set_periods_integer (this->handle, hw_params),
-      "cannot restrict period size to integral value: %s");
+    if (GST_ELEMENT (this)->numpads == 1) {
+      ERROR_CHECK (snd_pcm_hw_params_set_access (this->handle, hw_params, this->
+              mmap ? SND_PCM_ACCESS_MMAP_INTERLEAVED :
+              SND_PCM_ACCESS_RW_INTERLEAVED),
+          "This plugin does not support your harware: %s");
+    } else {
+      ERROR_CHECK (snd_pcm_hw_params_set_access (this->handle, hw_params, this->
+              mmap ? SND_PCM_ACCESS_MMAP_NONINTERLEAVED :
+              SND_PCM_ACCESS_RW_NONINTERLEAVED),
+          "This plugin does not support your harware: %s");
+    }
 
-  /* enable this for soundcard specific debugging */
-  /* snd_pcm_hw_params_dump (hw_params, this->out); */
-
-  mask = alloca (snd_pcm_access_mask_sizeof ());
-  snd_pcm_access_mask_none (mask);
-  if (GST_ELEMENT (this)->numpads == 1) {
-    snd_pcm_access_mask_set (mask,
-        this->
-        mmap ? SND_PCM_ACCESS_MMAP_INTERLEAVED : SND_PCM_ACCESS_RW_INTERLEAVED);
-  } else {
-    snd_pcm_access_mask_set (mask,
-        this->
-        mmap ? SND_PCM_ACCESS_MMAP_NONINTERLEAVED :
-        SND_PCM_ACCESS_RW_NONINTERLEAVED);
-  }
-  ERROR_CHECK (snd_pcm_hw_params_set_access_mask (this->handle, hw_params,
-          mask),
-      "The Gstreamer ALSA plugin does not support your hardware. Error: %s");
-
-  if (this->format) {
     ERROR_CHECK (snd_pcm_hw_params_set_format (this->handle, hw_params,
             this->format->format), "Sample format (%s) not available: %s",
         snd_pcm_format_name (this->format->format));
     ERROR_CHECK (snd_pcm_hw_params_set_channels (this->handle, hw_params,
             this->format->channels), "Channels count (%d) not available: %s",
         this->format->channels);
-    ERROR_CHECK (snd_pcm_hw_params_set_rate (this->handle, hw_params,
-            this->format->rate, 0), "error setting rate (%d): %s",
-        this->format->rate);
+    /* FIXME: We should use snd_pcm_hw_params_set_rate instead of 
+     * snd_pcm_hw_params_set_rate_near here. Unfortunately alsa fails in that case
+     * far more often and seems to handle this quite well. (Example: ENS1371 
+     * driver on alsalib 1.0.5, kernel 2.6.6-mm5). If it sets far too wrong sample
+     * rates, we need to revert back to snd_pcm_hw_params_set_rate or check the
+     * rate that was set.
+     */
+    rate = this->format->rate;
+    ERROR_CHECK (snd_pcm_hw_params_set_rate_near (this->handle, hw_params,
+            &rate, 0), "error setting rate (%d): %s", this->format->rate);
+    if (rate != this->format->rate)
+      GST_WARNING_OBJECT (this, "set rate (%u) differs from desired rate (%u)",
+          rate, this->format->rate);
+    ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle, hw_params,
+            &this->period_count, 0), "error setting period count to %u: %s",
+        (guint) this->period_count);
+    ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle,
+            hw_params, &this->period_size, 0),
+        "error setting period size to %u frames: %s",
+        (guint) this->period_size);
+  } else {
+    GST_INFO_OBJECT (this, "Preparing format: (none)");
   }
+  gst_alsa_hw_params_dump (this, hw_params);
 
-  ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle, hw_params,
-          &this->period_count, 0), "error setting buffer size to %u: %s",
-      (guint) this->period_count);
-  ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle, hw_params,
-          &this->period_size, 0), "error setting period size to %u frames: %s",
-      (guint) this->period_size);
 
   ERROR_CHECK (snd_pcm_hw_params (this->handle, hw_params),
       "Could not set hardware parameters: %s");
@@ -1295,10 +1509,17 @@ gst_alsa_set_sw_params (GstAlsa * this)
 {
   snd_pcm_sw_params_t *sw_params;
 
+  if (!this->format) {
+    GST_LOG_OBJECT (this, "not setting sw params, we're not negotiated yet");
+    return TRUE;
+  }
+
   snd_pcm_sw_params_alloca (&sw_params);
   ERROR_CHECK (snd_pcm_sw_params_current (this->handle, sw_params),
       "Could not get current software parameters: %s");
 
+  gst_alsa_sw_params_dump (this, sw_params);
+
   ERROR_CHECK (snd_pcm_sw_params_set_silence_size (this->handle, sw_params, 0),
       "could not set silence size: %s");
   ERROR_CHECK (snd_pcm_sw_params_set_silence_threshold (this->handle, sw_params,
@@ -1316,7 +1537,6 @@ gst_alsa_set_sw_params (GstAlsa * this)
       "Unable to set transfer align for playback: %s");
   ERROR_CHECK (snd_pcm_sw_params (this->handle, sw_params),
       "could not set sw_params: %s");
-
   return TRUE;
 }
 
@@ -1328,7 +1548,8 @@ gst_alsa_start_audio (GstAlsa * this)
   if (!gst_alsa_set_hw_params (this))
     return FALSE;
   if (!gst_alsa_set_sw_params (this))
-    return FALSE;
+    GST_WARNING_OBJECT (this,
+        "setting software parameters failed, we'll trust the defaults");
 
   GST_FLAG_SET (this, GST_ALSA_RUNNING);
   return TRUE;
@@ -1390,6 +1611,8 @@ gst_alsa_stop_audio (GstAlsa * this)
 static gboolean
 gst_alsa_close_audio (GstAlsa * this)
 {
+  gint err;
+
   /* if there's no pads, we never open. So we don't close either. */
   if (!gst_element_get_pad_list (GST_ELEMENT (this)))
     return TRUE;
@@ -1397,6 +1620,11 @@ gst_alsa_close_audio (GstAlsa * this)
   g_return_val_if_fail (this != NULL, FALSE);
   g_return_val_if_fail (this->handle != NULL, FALSE);
 
+  ALSA_DEBUG_FLUSH (this);
+  err = snd_output_close (this->out);
+  if (err != 0)
+    GST_ERROR_OBJECT (this, "failed to close debugging output: %s",
+        snd_strerror (err));
   ERROR_CHECK (snd_pcm_close (this->handle), "Error closing device: %s");
 
   this->handle = NULL;
@@ -1573,7 +1801,7 @@ gst_alsa_timestamp_to_samples (GstAlsa * this, GstClockTime time)
 inline GstClockTime
 gst_alsa_samples_to_timestamp (GstAlsa * this, snd_pcm_uframes_t samples)
 {
-  return (GstClockTime) (samples * GST_SECOND / this->format->rate);
+  return ((GstClockTime) samples) * GST_SECOND / this->format->rate;
 }
 
 inline snd_pcm_uframes_t
