@@ -95,8 +95,8 @@ gst_alsa_sink_get_type (void)
     };
 
     alsa_sink_type =
-        g_type_register_static (GST_TYPE_ALSA, "GstAlsaSink", &alsa_sink_info,
-        0);
+        g_type_register_static (GST_TYPE_ALSA_MIXER, "GstAlsaSink",
+        &alsa_sink_info, 0);
   }
   return alsa_sink_type;
 }
@@ -128,7 +128,7 @@ gst_alsa_sink_class_init (gpointer g_class, gpointer class_data)
   alsa_class = (GstAlsaClass *) klass;
 
   if (sink_parent_class == NULL)
-    sink_parent_class = g_type_class_ref (GST_TYPE_ALSA);
+    sink_parent_class = g_type_class_ref (GST_TYPE_ALSA_MIXER);
 
   alsa_class->stream = SND_PCM_STREAM_PLAYBACK;
   alsa_class->transmit_mmap = gst_alsa_sink_mmap;
@@ -210,28 +210,28 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
         break;
       case GST_EVENT_DISCONTINUOUS:
       {
-        GstClockTime value;
+        GstClockTime value, delay;
 
-        /* only the first pad my seek */
+        /* only the first pad may seek */
         if (pad_nr != 0) {
           break;
         }
+        delay = (this->format == NULL) ? 0 :
+            GST_SECOND * this->transmitted / this->format->rate -
+            gst_alsa_sink_get_time (this);
         if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
-          gst_element_set_time (GST_ELEMENT (this), value);
-        } else if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT,
-                &value)) {
+          gst_element_set_time_delay (GST_ELEMENT (this), MIN (value, delay), delay);
+        } else if (this->format &&
+            (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT, &value) ||
+                gst_event_discont_get_value (event, GST_FORMAT_BYTES,
+                    &value))) {
           value = gst_alsa_samples_to_timestamp (this, value);
-          gst_element_set_time (GST_ELEMENT (this), value);
-        } else if (gst_event_discont_get_value (event, GST_FORMAT_BYTES,
-                &value)) {
-          value = gst_alsa_bytes_to_timestamp (this, value);
-          gst_element_set_time (GST_ELEMENT (this), value);
+          gst_element_set_time_delay (GST_ELEMENT (this), MIN (value, delay), delay);
         } else {
-          GST_ERROR_OBJECT (this,
+          GST_WARNING_OBJECT (this,
               "couldn't extract time from discont event. Bad things might happen!");
         }
 
-
         break;
       }
       default:
@@ -254,7 +254,9 @@ gst_alsa_sink_mmap (GstAlsa * this, snd_pcm_sframes_t * avail)
   const snd_pcm_channel_area_t *dst;
   snd_pcm_channel_area_t *src;
   GstAlsaSink *sink = GST_ALSA_SINK (this);
-  int i, err, width = snd_pcm_format_physical_width (this->format->format);
+  int i;
+  int err = -1;
+  int width = snd_pcm_format_physical_width (this->format->format);
 
   /* areas points to the memory areas that belong to gstreamer. */
   src = g_malloc0 (this->format->channels * sizeof (snd_pcm_channel_area_t));
@@ -277,7 +279,7 @@ gst_alsa_sink_mmap (GstAlsa * this, snd_pcm_sframes_t * avail)
 
   if ((err = snd_pcm_mmap_begin (this->handle, &dst, &offset, avail)) < 0) {
     GST_ERROR_OBJECT (this, "mmap failed: %s", snd_strerror (err));
-    return -1;
+    goto out;
   }
 
   if ((err =
@@ -285,13 +287,15 @@ gst_alsa_sink_mmap (GstAlsa * this, snd_pcm_sframes_t * avail)
               *avail, this->format->format)) < 0) {
     snd_pcm_mmap_commit (this->handle, offset, 0);
     GST_ERROR_OBJECT (this, "data copy failed: %s", snd_strerror (err));
-    return -1;
+    goto out;
   }
   if ((err = snd_pcm_mmap_commit (this->handle, offset, *avail)) < 0) {
     GST_ERROR_OBJECT (this, "mmap commit failed: %s", snd_strerror (err));
-    return -1;
+    goto out;
   }
 
+out:
+  g_free (src);
   return err;
 }
 static int
@@ -325,7 +329,7 @@ static void
 gst_alsa_sink_loop (GstElement * element)
 {
   snd_pcm_sframes_t avail, avail2, copied, sample_diff, max_discont;
-  snd_pcm_uframes_t samplestamp, time_sample;
+  snd_pcm_uframes_t samplestamp, expected;
   gint i;
   guint bytes;                  /* per channel */
   GstAlsa *this = GST_ALSA (element);
@@ -369,12 +373,34 @@ sink_restart:
             gst_alsa_timestamp_to_samples (this,
             GST_BUFFER_TIMESTAMP (sink->buf[i]));
         max_discont = gst_alsa_timestamp_to_samples (this, this->max_discont);
-        time_sample =
-            gst_alsa_timestamp_to_samples (this,
-            gst_element_get_time (GST_ELEMENT (this)));
-        snd_pcm_delay (this->handle, &sample_diff);
-        /* actual diff = buffer samplestamp - played - to_play */
-        sample_diff = samplestamp - time_sample - sample_diff;
+        /* optimization: check if we're using our own clock
+         * This optimization is important because if we're using our own clock 
+         * gst_element_get_time calls snd_pcm_delay and the following code assumes
+         * that both calls return the same value. However they can be wildly 
+         * different, since snd_pcm_delay goes deep into the kernel.
+         */
+        if (gst_element_get_clock (element) == GST_CLOCK (this->clock)) {
+          /* FIXME: this is ugly because of the variables it uses but I don't know a 
+           * better way to get this info */
+          if (element->base_time > this->clock->start_time) {
+            expected =
+                this->transmitted - gst_alsa_timestamp_to_samples (this,
+                element->base_time - this->clock->start_time);
+          } else {
+            expected =
+                this->transmitted + gst_alsa_timestamp_to_samples (this,
+                this->clock->start_time - element->base_time);
+          }
+        } else {
+          if (snd_pcm_delay (this->handle, &sample_diff) != 0) {
+            sample_diff = 0;
+          }
+          expected =
+              gst_alsa_timestamp_to_samples (this,
+              gst_element_get_time (GST_ELEMENT (this))) + sample_diff;
+          /* actual diff = buffer samplestamp - played - to_play */
+        }
+        sample_diff = samplestamp - expected;
 
         if ((!GST_BUFFER_TIMESTAMP_IS_VALID (sink->buf[i])) ||
             (-max_discont <= sample_diff && sample_diff <= max_discont)) {
@@ -392,8 +418,8 @@ sink_restart:
               samples * snd_pcm_format_physical_width (this->format->format) /
               8;
           GST_INFO_OBJECT (this,
-              "Allocating %d bytes (%ld samples) now to resync: sample %ld expected, but got %ld",
-              size, MIN (bytes, sample_diff), time_sample, samplestamp);
+              "Allocating %d bytes (%ld samples) now to resync: sample %lu expected, but got %ld",
+              size, MIN (bytes, sample_diff), expected, samplestamp);
           sink->data[i] = g_try_malloc (size);
           if (!sink->data[i]) {
             GST_WARNING_OBJECT (this,
@@ -410,19 +436,19 @@ sink_restart:
         } else if (gst_alsa_samples_to_bytes (this,
                 -sample_diff) >= sink->buf[i]->size) {
           GST_INFO_OBJECT (this,
-              "Skipping %lu samples to resync (complete buffer): sample %ld expected, but got %ld",
-              gst_alsa_bytes_to_samples (this, sink->buf[i]->size), time_sample,
+              "Skipping %lu samples to resync (complete buffer): sample %lu expected, but got %ld",
+              gst_alsa_bytes_to_samples (this, sink->buf[i]->size), expected,
               samplestamp);
           /* this buffer is way behind */
           gst_buffer_unref (sink->buf[i]);
           sink->buf[i] = NULL;
           continue;
         } else if (sample_diff < 0) {
-          gint difference = gst_alsa_samples_to_bytes (this, -samplestamp);
+          gint difference = gst_alsa_samples_to_bytes (this, -sample_diff);
 
           GST_INFO_OBJECT (this,
-              "Skipping %lu samples to resync: sample %ld expected, but got %ld",
-              (gulong) - sample_diff, time_sample, samplestamp);
+              "Skipping %lu samples to resync: sample %lu expected, but got %ld",
+              (gulong) - sample_diff, expected, samplestamp);
           /* this buffer is only a bit behind */
           sink->size[i] = sink->buf[i]->size - difference;
           sink->data[i] = sink->buf[i]->data + difference;
@@ -512,10 +538,14 @@ gst_alsa_sink_get_time (GstAlsa * this)
 {
   snd_pcm_sframes_t delay;
 
-  if (snd_pcm_delay (this->handle, &delay) == 0 && this->format) {
-    return GST_SECOND * (GstClockTime) (this->transmitted >
-        delay ? this->transmitted - delay : 0) / this->format->rate;
-  } else {
+  if (!this->format)
+    return 0;
+  if (snd_pcm_delay (this->handle, &delay) != 0) {
+    return this->transmitted / this->format->rate;
+  }
+  if (this->transmitted <= delay) {
     return 0;
   }
+
+  return GST_SECOND * (this->transmitted - delay) / this->format->rate;
 }
