@@ -32,7 +32,7 @@
 /* elementfactory information */
 static GstElementDetails gst_fameenc_details = {
   "MPEG1 and MPEG4 video encoder using the libfame library",
-  "Codec/Video/Encoder",
+  "Codec/Encoder/Video",
   "Uses fameenc to encode MPEG video streams",
   "Thomas Vander Stichele <thomas@apestaart.org>"
 };
@@ -90,7 +90,8 @@ typedef struct
 {
   gint num;
   gint den;
-} frame_rate_entry;
+}
+frame_rate_entry;
 
 static const frame_rate_entry frame_rates[] = {
   {0, 0},
@@ -161,6 +162,7 @@ gst_fameenc_get_type (void)
       0,
       (GInstanceInitFunc) gst_fameenc_init,
     };
+
     fameenc_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstFameEnc", &fameenc_info, 0);
   }
