@@ -1,7 +1,7 @@
 /* GStreamer
- * Copyright (C) 2003 Benjamin Otte <in7y118@public.uni-hamburg.de>
+ * Copyright (C) 2003-2004 Benjamin Otte <otte@gnome.org>
  *
- * gstid3tagsetter.c: plugin for reading / modifying id3 tags
+ * gstid3tag.c: plugin for reading / modifying id3 tags
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,18 +23,20 @@
 #include "config.h"
 #endif
 #include "gstmad.h"
-#include <gst/gsttaginterface.h>
+#include <stdlib.h>
 #include <string.h>
+#include <gst/gsttaginterface.h>
 
 #define ID3_TYPE_FIND_SIZE 40960
 GST_DEBUG_CATEGORY_STATIC (gst_id3_tag_debug);
 #define GST_CAT_DEFAULT gst_id3_tag_debug
 
-#define GST_TYPE_ID3_TAG (gst_id3_tag_get_type())
+#define GST_TYPE_ID3_TAG (gst_id3_tag_get_type(GST_ID3_TAG_PARSE_BASE ))
 #define GST_ID3_TAG(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ID3_TAG, GstID3Tag))
-#define GST_ID3_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ID3_TAG, GstID3Tag))
+#define GST_ID3_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ID3_TAG, GstID3TagClass))
 #define GST_IS_ID3_TAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ID3_TAG))
 #define GST_IS_ID3_TAG_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ID3_TAG))
+#define GST_ID3_TAG_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_ID3_TAG, GstID3TagClass))
 
 typedef struct _GstID3Tag GstID3Tag;
 typedef struct _GstID3TagClass GstID3TagClass;
@@ -46,19 +48,24 @@ typedef enum
   GST_ID3_TAG_STATE_READING_V1_TAG,
   GST_ID3_TAG_STATE_SEEKING_TO_NORMAL,
   GST_ID3_TAG_STATE_NORMAL_START,
-  GST_ID3_TAG_STATE_NORMAL,
+  GST_ID3_TAG_STATE_NORMAL
 }
 GstID3TagState;
 
 typedef enum
 {
-  GST_ID3_TAG_PARSE_UNKNOWN,
-  GST_ID3_TAG_PARSE_TAG,
-  GST_ID3_TAG_PARSE_WRITE,
-  GST_ID3_TAG_PARSE_PARSE
+  GST_ID3_TAG_PARSE_BASE = 0,
+  GST_ID3_TAG_PARSE_DEMUX = 1,
+  GST_ID3_TAG_PARSE_MUX = 2,
+  GST_ID3_TAG_PARSE_ANY = 3
 }
 GstID3ParseMode;
 
+#define IS_DEMUXER(tag) ((tag)->parse_mode & GST_ID3_TAG_PARSE_DEMUX)
+#define IS_MUXER(tag) ((tag)->parse_mode & GST_ID3_TAG_PARSE_MUX)
+#define CAN_BE_DEMUXER(tag) (GST_ID3_TAG_GET_CLASS(tag)->type & GST_ID3_TAG_PARSE_DEMUX)
+#define CAN_BE_MUXER(tag) (GST_ID3_TAG_GET_CLASS(tag)->type & GST_ID3_TAG_PARSE_MUX)
+
 struct _GstID3Tag
 {
   GstElement element;
@@ -92,16 +99,10 @@ struct _GstID3Tag
 struct _GstID3TagClass
 {
   GstElementClass parent_class;
+
+  GstID3ParseMode type;
 };
 
-/* elementfactory information */
-static GstElementDetails gst_id3_tag_details =
-GST_ELEMENT_DETAILS ("id3 tag extractor",
-    "Tag",
-    "Extract tagging information from mp3s",
-    "Benjamin Otte <in7y118@public.uni-hamburg.de>");
-
-
 /* signals and args */
 enum
 {
@@ -120,15 +121,30 @@ enum
 
 GST_DEBUG_CATEGORY_EXTERN (mad_debug);
 
-static GstStaticPadTemplate id3_tag_src_template_factory =
+static GstStaticPadTemplate id3_tag_src_any_template_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
-  /* FIXME: for spider - GST_PAD_ALWAYS, */
+  /* FIXME: for spider - should be GST_PAD_ALWAYS, */
     GST_PAD_SOMETIMES,
     GST_STATIC_CAPS ("ANY")
     );
 
-static GstStaticPadTemplate id3_tag_sink_template_factory =
+static GstStaticPadTemplate id3_tag_src_id3_template_factory =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("application/x-id3")
+    );
+
+static GstStaticPadTemplate id3_tag_sink_any_template_factory =
+    GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    /* FIXME: find a way to extend this generically */
+    GST_STATIC_CAPS ("audio/mpeg, mpegversion=(int)1; audio/x-flac")
+    );
+
+static GstStaticPadTemplate id3_tag_sink_id3_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -136,9 +152,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     );
 
 
-static void gst_id3_tag_base_init (gpointer g_class);
-static void gst_id3_tag_class_init (GstID3TagClass * klass);
-static void gst_id3_tag_init (GstID3Tag * tag);
+static void gst_id3_tag_class_init (gpointer g_class, gpointer class_data);
+static void gst_id3_tag_init (GTypeInstance * instance, gpointer g_class);
 static void gst_id3_tag_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_id3_tag_get_property (GObject * object,
@@ -162,21 +177,26 @@ static GstElementClass *parent_class = NULL;
 /* static guint gst_id3_tag_signals[LAST_SIGNAL] = { 0 }; */
 
 GType
-gst_id3_tag_get_type (void)
+gst_id3_tag_get_type (guint type)
 {
-  static GType id3_tag_type = 0;
+  static GType id3_tag_type[4] = { 0, 0, 0, 0 };
+  static gchar *name[4] = { "GstID3TagBase", "GstID3Demux", "GstID3Mux",
+    "GstID3Tag"
+  };
 
-  if (!id3_tag_type) {
-    static const GTypeInfo id3_tag_info = {
+  g_assert (type < 4);
+
+  if (!id3_tag_type[type]) {
+    GTypeInfo id3_tag_info = {
       sizeof (GstID3TagClass),
-      gst_id3_tag_base_init,
-      NULL,
-      (GClassInitFunc) gst_id3_tag_class_init,
       NULL,
       NULL,
+      gst_id3_tag_class_init,
+      NULL,
+      GUINT_TO_POINTER (type),
       sizeof (GstID3Tag),
       0,
-      (GInstanceInitFunc) gst_id3_tag_init,
+      gst_id3_tag_init,
     };
     static const GInterfaceInfo tag_setter_info = {
       NULL,
@@ -184,55 +204,80 @@ gst_id3_tag_get_type (void)
       NULL
     };
 
-    id3_tag_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstID3Tag", &id3_tag_info,
-        0);
+    id3_tag_type[type] = g_type_register_static (
+        (type == GST_ID3_TAG_PARSE_BASE) ? GST_TYPE_ELEMENT :
+        GST_TYPE_ID3_TAG, name[type], &id3_tag_info, 0);
 
-    g_type_add_interface_static (id3_tag_type, GST_TYPE_TAG_SETTER,
-        &tag_setter_info);
-
-    GST_DEBUG_CATEGORY_INIT (gst_id3_tag_debug, "id3tag", 0,
-        "id3 tag reader / setter");
+    if (type & GST_ID3_TAG_PARSE_MUX) {
+      g_type_add_interface_static (id3_tag_type[type], GST_TYPE_TAG_SETTER,
+          &tag_setter_info);
+    }
   }
-  return id3_tag_type;
+  return id3_tag_type[type];
 }
+
+/* elementfactory information */
+GstElementDetails gst_id3_tag_details[3] = {
+  GST_ELEMENT_DETAILS ("id3 tag extractor",
+      "Codec/Demuxer/Audio",
+      "Extract ID3 tagging information",
+      "Benjamin Otte <otte@gnome.org>"),
+  GST_ELEMENT_DETAILS ("id3 muxer",
+      "Codec/Muxer/Audio",
+      "Add ID3 tagging information",
+      "Benjamin Otte <otte@gnome.org>"),
+  GST_ELEMENT_DETAILS ("id3 tag extractor",
+      "Tag",
+      "Extract tagging information from mp3s",
+      "Benjamin Otte <otte@gnome.org>")
+};
+
 static void
-gst_id3_tag_base_init (gpointer g_class)
+gst_id3_tag_class_init (gpointer g_class, gpointer class_data)
 {
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
+  GstID3TagClass *tag_class = GST_ID3_TAG_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_id3_tag_details);
+  tag_class->type = GPOINTER_TO_UINT (class_data);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&id3_tag_sink_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&id3_tag_src_template_factory));
-}
-static void
-gst_id3_tag_class_init (GstID3TagClass * klass)
-{
-  GstElementClass *gstelement_class;
-  GObjectClass *gobject_class;
+  if (tag_class->type == GST_ID3_TAG_PARSE_BASE) {
+    parent_class = g_type_class_peek_parent (g_class);
+    gstelement_class->change_state = gst_id3_tag_change_state;
+  } else {
+    gst_element_class_set_details (gstelement_class,
+        &gst_id3_tag_details[tag_class->type - 1]);
+  }
 
-  gstelement_class = (GstElementClass *) klass;
-  gobject_class = (GObjectClass *) klass;
+  if (tag_class->type & GST_ID3_TAG_PARSE_DEMUX) {
+    g_object_class_install_property (gobject_class, ARG_PREFER_V1,
+        g_param_spec_boolean ("prefer-v1", "prefer version 1 tag",
+            "Prefer tags from tag at end of file", FALSE,
+            G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+    gst_element_class_add_pad_template (gstelement_class,
+        gst_static_pad_template_get (&id3_tag_src_any_template_factory));
+  } else {
+    gst_element_class_add_pad_template (gstelement_class,
+        gst_static_pad_template_get (&id3_tag_src_id3_template_factory));
+  }
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
-
-  gstelement_class->change_state = gst_id3_tag_change_state;
-
-  g_object_class_install_property (gobject_class, ARG_V1_TAG,
-      g_param_spec_boolean ("v1-tag", "add version 1 tag",
-          "Add version 1 tag at end of file", FALSE,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-  g_object_class_install_property (gobject_class, ARG_V2_TAG,
-      g_param_spec_boolean ("v2-tag", "add version 2 tag",
-          "Add version 2 tag at start of file", TRUE,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-  g_object_class_install_property (gobject_class, ARG_PREFER_V1,
-      g_param_spec_boolean ("prefer-v1", "prefer version 1 tag",
-          "Prefer tags from tag at end of file", FALSE,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+  if (tag_class->type & GST_ID3_TAG_PARSE_MUX) {
+    g_object_class_install_property (gobject_class, ARG_V2_TAG,
+        g_param_spec_boolean ("v2-tag", "add version 2 tag",
+            "Add version 2 tag at start of file", TRUE,
+            G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+    g_object_class_install_property (gobject_class, ARG_V1_TAG,
+        g_param_spec_boolean ("v1-tag", "add version 1 tag",
+            "Add version 1 tag at end of file", FALSE,
+            G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+  }
+  if (tag_class->type == GST_ID3_TAG_PARSE_MUX) {
+    gst_element_class_add_pad_template (gstelement_class,
+        gst_static_pad_template_get (&id3_tag_sink_any_template_factory));
+  } else {
+    gst_element_class_add_pad_template (gstelement_class,
+        gst_static_pad_template_get (&id3_tag_sink_id3_template_factory));
+  }
 
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_id3_tag_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_id3_tag_get_property);
@@ -244,13 +289,15 @@ gst_id3_tag_get_caps (GstPad * pad)
   GstID3Tag *tag = GST_ID3_TAG (gst_pad_get_parent (pad));
 
   if (tag->found_caps) {
-    GstCaps *caps;
+    GstCaps *caps = gst_caps_copy (tag->found_caps);
 
-    caps = gst_caps_from_string ("application/x-gst-tags; application/x-id3");
-    gst_caps_append (caps, gst_caps_copy (tag->found_caps));
+    if (CAN_BE_MUXER (tag)) {
+      gst_caps_append (caps,
+          gst_caps_from_string ("application/x-gst-tags; application/x-id3"));
+    }
     return caps;
   } else {
-    return gst_caps_new_any ();
+    return gst_caps_copy (gst_pad_get_pad_template_caps (pad));
   }
 }
 
@@ -259,8 +306,8 @@ gst_id3_tag_add_src_pad (GstID3Tag * tag)
 {
   g_assert (tag->srcpad == NULL);
   tag->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&id3_tag_src_template_factory), "src");
+      gst_pad_new_from_template (gst_element_class_get_pad_template
+      (GST_ELEMENT_GET_CLASS (tag), "src"), "src");
   gst_pad_set_event_function (tag->srcpad,
       GST_DEBUG_FUNCPTR (gst_id3_tag_src_event));
   gst_pad_set_event_mask_function (tag->srcpad,
@@ -275,21 +322,27 @@ gst_id3_tag_add_src_pad (GstID3Tag * tag)
       GST_DEBUG_FUNCPTR (gst_id3_tag_src_link));
   gst_element_add_pad (GST_ELEMENT (tag), tag->srcpad);
 }
+
 static void
-gst_id3_tag_init (GstID3Tag * tag)
+gst_id3_tag_init (GTypeInstance * instance, gpointer g_class)
 {
-  /* create the sink and src pads */
-  tag->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&id3_tag_sink_template_factory), "sink");
-  gst_element_add_pad (GST_ELEMENT (tag), tag->sinkpad);
-  gst_pad_set_chain_function (tag->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_id3_tag_chain));
+  GstID3Tag *tag = GST_ID3_TAG (instance);
 
+  if (GST_ID3_TAG_GET_CLASS (tag)->type != GST_ID3_TAG_PARSE_BASE) {
+    /* create the sink and src pads */
+    tag->sinkpad =
+        gst_pad_new_from_template (gst_element_class_get_pad_template
+        (GST_ELEMENT_CLASS (g_class), "sink"), "sink");
+    gst_element_add_pad (GST_ELEMENT (tag), tag->sinkpad);
+    gst_pad_set_chain_function (tag->sinkpad,
+        GST_DEBUG_FUNCPTR (gst_id3_tag_chain));
+  }
+  if (GST_ID3_TAG_GET_CLASS (tag)->type == GST_ID3_TAG_PARSE_MUX) {
+    /* only the muxer class here, all other use sometimes pads */
+    gst_id3_tag_add_src_pad (tag);
+  }
   /* FIXME: for the alli^H^H^H^Hspider - gst_id3_tag_add_src_pad (tag); */
-
-  tag->state = GST_ID3_TAG_STATE_READING_V2_TAG;
-  tag->parse_mode = GST_ID3_TAG_PARSE_UNKNOWN;
+  tag->parse_mode = GST_ID3_TAG_PARSE_BASE;
   tag->buffer = NULL;
 
   GST_FLAG_SET (tag, GST_ELEMENT_EVENT_AWARE);
@@ -305,20 +358,24 @@ gst_id3_tag_set_property (GObject * object, guint prop_id, const GValue * value,
   switch (prop_id) {
     case ARG_V1_TAG:
       tag->v1tag_render = g_value_get_boolean (value);
-      g_object_notify (object, "v1-tag");
       break;
     case ARG_V2_TAG:
       tag->v2tag_render = g_value_get_boolean (value);
-      g_object_notify (object, "v2-tag");
       break;
     case ARG_PREFER_V1:
       tag->prefer_v1tag = g_value_get_boolean (value);
-      g_object_notify (object, "prefer-v1");
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
+
+  /* make sure we render at least one tag */
+  if (GST_ID3_TAG_GET_CLASS (tag)->type == GST_ID3_TAG_PARSE_MUX &&
+      !tag->v1tag_render && !tag->v2tag_render) {
+    g_object_set (object, prop_id == ARG_V1_TAG ? "v2-tag" : "v1-tag", TRUE,
+        NULL);
+  }
 }
 static void
 gst_id3_tag_get_property (GObject * object, guint prop_id, GValue * value,
@@ -672,7 +729,8 @@ gst_id3_tag_get_tag_to_render (GstID3Tag * tag)
   if (tag->event_tags)
     ret = gst_tag_list_copy (tag->event_tags);
   if (ret) {
-    gst_tag_list_insert (ret, tag->parsed_tags, GST_TAG_MERGE_KEEP);
+    if (tag->parsed_tags)
+      gst_tag_list_insert (ret, tag->parsed_tags, GST_TAG_MERGE_KEEP);
   } else if (tag->parsed_tags) {
     ret = gst_tag_list_copy (tag->parsed_tags);
   }
@@ -702,7 +760,10 @@ gst_id3_tag_handle_event (GstPad * pad, GstEvent * event)
                     GST_BUFFER_SIZE (tag->buffer)
                     : 0))
               GST_ELEMENT_ERROR (tag, CORE, EVENT, (NULL),
-                  ("Seek during ID3v2 tag reading"));
+                  ("Got seek to %" G_GUINT64_FORMAT " during ID3v2 tag reading"
+                      " (allowed was %" G_GUINT64_FORMAT ")", value,
+                      (guint64) (tag->buffer ? GST_BUFFER_OFFSET (tag->buffer)
+                          + GST_BUFFER_SIZE (tag->buffer) : 0)));
           }
           gst_data_unref (GST_DATA (event));
           break;
@@ -723,15 +784,26 @@ gst_id3_tag_handle_event (GstPad * pad, GstEvent * event)
           gst_data_unref (GST_DATA (event));
           break;
         case GST_ID3_TAG_STATE_NORMAL_START:
-          GST_ERROR_OBJECT (tag, "tag event not sent, FIXME");
-          gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_NORMAL);
-          /* fall through */
+          if (!CAN_BE_DEMUXER (tag)) {
+            /* initial discont, ignore */
+            gst_data_unref (GST_DATA (event));
+            break;
+          } else {
+            GST_ERROR_OBJECT (tag, "tag event not sent, FIXME");
+            gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_NORMAL);
+            /* fall through */
+          }
         case GST_ID3_TAG_STATE_NORMAL:{
           gint64 value;
           GstEvent *new;
 
           if (gst_event_discont_get_value (event, GST_FORMAT_BYTES, &value)) {
-            value += tag->v1tag_size;
+            if (value > tag->v2tag_size) {
+              value -= tag->v2tag_size;
+            } else {
+              /* FIXME: throw an error here? */
+              value = 0;
+            }
             new =
                 gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, value, 0);
             gst_data_unref (GST_DATA (event));
@@ -755,7 +827,7 @@ gst_id3_tag_handle_event (GstPad * pad, GstEvent * event)
       gst_data_unref (GST_DATA (event));
       break;
     case GST_EVENT_EOS:
-      if (tag->v1tag_render && !tag->parse_mode) {
+      if (tag->v1tag_render && IS_MUXER (tag)) {
         GstTagList *merged;
         struct id3_tag *id3;
 
@@ -850,7 +922,7 @@ gst_id3_tag_do_typefind (GstID3Tag * tag, GstBuffer * buffer)
 static gboolean
 gst_id3_tag_do_caps_nego (GstID3Tag * tag, GstBuffer * buffer)
 {
-  if (buffer != NULL) {
+  if (buffer != NULL && CAN_BE_DEMUXER (tag)) {
     tag->found_caps = gst_id3_tag_do_typefind (tag, buffer);
     if (!tag->found_caps) {
       return FALSE;
@@ -859,9 +931,11 @@ gst_id3_tag_do_caps_nego (GstID3Tag * tag, GstBuffer * buffer)
   if (!tag->srcpad)
     gst_id3_tag_add_src_pad (tag);
   if (!gst_pad_is_linked (tag->srcpad)) {
-    tag->parse_mode = GST_ID3_TAG_PARSE_TAG;
+    GST_DEBUG_OBJECT (tag, "srcpad not linked, not proceeding");
+    tag->parse_mode = GST_ID3_TAG_GET_CLASS (tag)->type;
     return TRUE;
   } else {
+    GST_DEBUG_OBJECT (tag, "renegotiating");
     return gst_pad_renegotiate (tag->srcpad) != GST_PAD_LINK_REFUSED;
   }
 }
@@ -874,19 +948,23 @@ gst_id3_tag_src_link (GstPad * pad, const GstCaps * caps)
 
   tag = GST_ID3_TAG (gst_pad_get_parent (pad));
 
-  if (!tag->found_caps)
+  if (!tag->found_caps && CAN_BE_DEMUXER (tag))
     return GST_PAD_LINK_DELAYED;
+  if (!CAN_BE_MUXER (tag) || !CAN_BE_DEMUXER (tag)) {
+    tag->parse_mode = GST_ID3_TAG_GET_CLASS (tag)->type;
+    return GST_PAD_LINK_OK;
+  }
 
   mimetype = gst_structure_get_name (gst_caps_get_structure (caps, 0));
 
   if (strcmp (mimetype, "application/x-id3") == 0) {
-    tag->parse_mode = GST_ID3_TAG_PARSE_WRITE;
+    tag->parse_mode = GST_ID3_TAG_PARSE_MUX;
     GST_LOG_OBJECT (tag, "normal operation, using application/x-id3 output");
   } else if (strcmp (mimetype, "application/x-gst-tags") == 0) {
-    tag->parse_mode = GST_ID3_TAG_PARSE_TAG;
+    tag->parse_mode = GST_ID3_TAG_PARSE_ANY;
     GST_LOG_OBJECT (tag, "fast operation, just outputting tags");
   } else {
-    tag->parse_mode = GST_ID3_TAG_PARSE_PARSE;
+    tag->parse_mode = GST_ID3_TAG_PARSE_DEMUX;
     GST_LOG_OBJECT (tag, "parsing operation, extracting tags");
   }
 
@@ -899,8 +977,13 @@ gst_id3_tag_send_tag_event (GstID3Tag * tag)
   GstTagList *merged = gst_tag_list_merge (tag->event_tags, tag->parsed_tags,
       GST_TAG_MERGE_KEEP);
 
+  if (tag->parsed_tags)
+    gst_element_found_tags (GST_ELEMENT (tag), tag->parsed_tags);
   if (merged) {
-    gst_element_found_tags_for_pad (GST_ELEMENT (tag), tag->srcpad, 0, merged);
+    GstEvent *event = gst_event_new_tag (merged);
+
+    GST_EVENT_TIMESTAMP (event) = 0;
+    gst_pad_push (tag->srcpad, GST_DATA (event));
   }
 }
 static void
@@ -966,7 +1049,7 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
       }
       gst_data_unref (GST_DATA (tag->buffer));
       tag->buffer = NULL;
-      if (tag->parse_mode != GST_ID3_TAG_PARSE_TAG) {
+      if (tag->parse_mode != GST_ID3_TAG_PARSE_ANY) {
         /* seek to beginning */
         GST_LOG_OBJECT (tag, "seeking back to beginning");
         if (gst_pad_send_event (GST_PAD_PEER (tag->sinkpad),
@@ -1033,6 +1116,13 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
       buffer =
           gst_buffer_create_sub (tag->buffer, tag->v2tag_size,
           GST_BUFFER_SIZE (tag->buffer) - tag->v2tag_size);
+      /* the offsets will be corrected further down, we just copy them */
+      if (GST_BUFFER_OFFSET_IS_VALID (tag->buffer))
+        GST_BUFFER_OFFSET (buffer) =
+            GST_BUFFER_OFFSET (tag->buffer) + tag->v2tag_size;
+      if (GST_BUFFER_OFFSET_END_IS_VALID (tag->buffer))
+        GST_BUFFER_OFFSET_END (buffer) =
+            GST_BUFFER_OFFSET_END (tag->buffer) + tag->v2tag_size;
       gst_data_unref (GST_DATA (tag->buffer));
       tag->buffer = NULL;
       if (tag->found_caps == NULL)
@@ -1052,7 +1142,7 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
       g_assert (tag->buffer == NULL);
       gst_id3_tag_send_tag_event (tag);
 
-      if (tag->parse_mode == GST_ID3_TAG_PARSE_WRITE && tag->v2tag_render) {
+      if (IS_MUXER (tag) && tag->v2tag_render) {
         struct id3_tag *id3;
         GstTagList *merged;
         GstBuffer *tag_buffer;
@@ -1079,23 +1169,40 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
       }
       gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_NORMAL);
       tag->v1tag_size_new = (tag->v1tag_render &&
-          tag->parse_mode == GST_ID3_TAG_PARSE_WRITE &&
+          IS_MUXER (tag) &&
           (tag->parsed_tags != NULL ||
               gst_tag_setter_get_list (GST_TAG_SETTER (tag)) !=
               NULL)) ? 128 : 0;
       /* fall through */
     case GST_ID3_TAG_STATE_NORMAL:
-      if (tag->parse_mode == GST_ID3_TAG_PARSE_TAG) {
+      if (tag->parse_mode == GST_ID3_TAG_PARSE_ANY) {
+        gst_data_unref (GST_DATA (buffer));
         gst_element_set_eos (GST_ELEMENT (tag));
         gst_pad_push (tag->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
       } else {
-        if (buffer->offset >= tag->v1tag_offset) {
-          gst_data_unref (GST_DATA (buffer));
-          return;
-        } else if (buffer->offset + buffer->size > tag->v1tag_offset) {
-          GstBuffer *sub = gst_buffer_create_sub (buffer, 0,
-              buffer->size - 128);
+        if (GST_BUFFER_OFFSET_IS_VALID (buffer)) {
+          if (buffer->offset >= tag->v1tag_offset) {
+            gst_data_unref (GST_DATA (buffer));
+            return;
+          } else if (buffer->offset + buffer->size > tag->v1tag_offset) {
+            GstBuffer *sub = gst_buffer_create_sub (buffer, 0,
+                buffer->size - 128);
 
+            gst_data_unref (GST_DATA (buffer));
+            buffer = sub;
+          }
+        }
+        if (tag->v2tag_size) {
+          GstBuffer *sub =
+              gst_buffer_create_sub (buffer, 0, GST_BUFFER_SIZE (buffer));
+          if (GST_BUFFER_OFFSET_IS_VALID (buffer))
+            GST_BUFFER_OFFSET (sub) =
+                GST_BUFFER_OFFSET (buffer) - tag->v2tag_size +
+                tag->v2tag_size_new;
+          if (GST_BUFFER_OFFSET_END_IS_VALID (buffer))
+            GST_BUFFER_OFFSET_END (sub) =
+                GST_BUFFER_OFFSET_END (buffer) - tag->v2tag_size +
+                tag->v2tag_size_new;
           gst_data_unref (GST_DATA (buffer));
           buffer = sub;
         }
@@ -1122,9 +1229,13 @@ gst_id3_tag_change_state (GstElement * element)
       tag->v1tag_size = 0;
       tag->v1tag_offset = G_MAXUINT64;
       tag->v2tag_size = 0;
+      if (CAN_BE_DEMUXER (tag)) {
+        gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_READING_V2_TAG);
+      } else {
+        gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_NORMAL_START);
+      }
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
-      /* do something to get out of the chain function faster */
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
@@ -1145,7 +1256,7 @@ gst_id3_tag_change_state (GstElement * element)
         gst_caps_free (tag->found_caps);
         tag->found_caps = NULL;
       }
-      tag->parse_mode = GST_ID3_TAG_PARSE_UNKNOWN;
+      tag->parse_mode = GST_ID3_TAG_PARSE_BASE;
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -1162,14 +1273,20 @@ plugin_init (GstPlugin * plugin)
   if (!gst_library_load ("gsttags"))
     return FALSE;
 
-  /* We register id3tag with a rank one less than mad so that mad always
-     gets picked before id3tag if possible.  This avoids multiple plugging
-     of id3tag in the autoplugger for now. */
   if (!gst_element_register (plugin, "mad", GST_RANK_PRIMARY,
           gst_mad_get_type ())
-      || !gst_element_register (plugin, "id3tag", GST_RANK_PRIMARY - 1,
-          gst_id3_tag_get_type ()))
+      || !gst_element_register (plugin, "id3demux", GST_RANK_PRIMARY,
+          gst_id3_tag_get_type (GST_ID3_TAG_PARSE_DEMUX))
+      || !gst_element_register (plugin, "id3mux", GST_RANK_NONE,        /* removed for spider */
+          gst_id3_tag_get_type (GST_ID3_TAG_PARSE_MUX))
+      /* FIXME 0.9: remove this element */
+      || !gst_element_register (plugin, "id3tag", GST_RANK_NONE,
+          gst_id3_tag_get_type (GST_ID3_TAG_PARSE_ANY))) {
     return FALSE;
+  }
+
+  GST_DEBUG_CATEGORY_INIT (gst_id3_tag_debug, "id3tag", 0,
+      "id3 tag reader / setter");
 
   return TRUE;
 }
