@@ -126,15 +126,20 @@ GST_STATIC_PAD_TEMPLATE ("src",
         "signed = (boolean) true, "
         "width = (int) 16, "
         "depth = (int) 16, "
-        "rate = (int) [ 11025, 48000 ], " "channels = (int) [ 1, 2 ]")
+        "rate = (int) { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000 }, "
+        "channels = (int) [ 1, 2 ]")
     );
 
+/* FIXME: make three caps, for mpegversion 1, 2 and 2.5 */
 static GstStaticPadTemplate mad_sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/mpeg, "
-        "mpegversion = (int) 1, " "layer = (int) [ 1, 3 ]")
+        "mpegversion = (int) 1, "
+        "layer = (int) [ 1, 3 ], "
+        "rate = (int) { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000 }, "
+        "channels = (int) [ 1, 2 ]")
     );
 
 static void gst_mad_base_init (gpointer g_class);
@@ -294,7 +299,7 @@ gst_mad_class_init (GstMadClass * klass)
           FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_IGNORE_CRC,
       g_param_spec_boolean ("ignore_crc", "Ignore CRC", "Ignore CRC errors",
-          FALSE, G_PARAM_READWRITE));
+          TRUE, G_PARAM_READWRITE));
 
   /* register tags */
 #define GST_TAG_LAYER    "layer"
@@ -357,7 +362,7 @@ gst_mad_init (GstMad * mad)
   mad->tags = NULL;
 
   mad->half = FALSE;
-  mad->ignore_crc = FALSE;
+  mad->ignore_crc = TRUE;
   mad->check_for_xing = TRUE;
   GST_FLAG_SET (mad, GST_ELEMENT_EVENT_AWARE);
 }
@@ -908,31 +913,40 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
           GstEvent *discont;
 
           /* see how long the input bytes take */
-          format = GST_FORMAT_TIME;
-          if (!gst_pad_convert (pad,
-                  GST_EVENT_DISCONT_OFFSET (event, i).format,
-                  value, &format, &time)) {
-            time = 0;
+          if (GST_EVENT_DISCONT_OFFSET (event, i).format != GST_FORMAT_TIME) {
+            format = GST_FORMAT_TIME;
+            if (!gst_pad_convert (pad,
+                    GST_EVENT_DISCONT_OFFSET (event, i).format,
+                    value, &format, &time)) {
+              continue;
+            }
+          } else {
+            time = value;
           }
 
           /* for now, this is the best we can do to get the total number
-           * of samples */
+           * of samples. This is suboptimal because the incoming event
+           * might contain this information already (although I believe
+           * that this doesn't happen anywhere so far). */
           format = GST_FORMAT_DEFAULT;
           if (!gst_pad_convert (mad->srcpad,
                   GST_FORMAT_TIME, time, &format, &mad->total_samples)) {
-            mad->total_samples = 0;
+            continue;
           }
 
-          gst_event_unref (event);
-
           if (GST_PAD_IS_USABLE (mad->srcpad)) {
             discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
                 time, NULL);
             gst_pad_push (mad->srcpad, GST_DATA (discont));
           }
-          break;
+          gst_event_unref (event);
+          goto done;
         }
       }
+      GST_WARNING ("Failed to retrieve sample position");
+      /* this isn't really correct? */
+      gst_pad_event_default (pad, event);
+    done:
       mad->tempsize = 0;
       /* we don't need to restart when we get here */
       mad->restart = FALSE;
@@ -946,7 +960,6 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
       gst_pad_event_default (pad, event);
       break;
   }
-  return;
 }
 
 static gboolean
@@ -1368,9 +1381,8 @@ gst_mad_chain (GstPad * pad, GstData * _data)
               &mad->total_samples);
           mad->last_ts = GST_CLOCK_TIME_NONE;
         }
-        time_offset = mad->total_samples * GST_SECOND
-            / mad->frame.header.samplerate;
-        time_duration = (nsamples * GST_SECOND / mad->frame.header.samplerate);
+        time_offset = mad->total_samples * GST_SECOND / mad->rate;
+        time_duration = (nsamples * GST_SECOND / mad->rate);
       }
 
       if (mad->index) {
