@@ -38,6 +38,7 @@ unsigned int audio_format = 100;
 enum
 {
   /* FILL ME */
+  SIGNAL_CONNECTION_PROBLEM,
   LAST_SIGNAL
 };
 
@@ -55,36 +56,23 @@ enum
   ARG_PROTOCOL,                 /* Protocol to connect with */
 
   ARG_MOUNT,                    /* mountpoint of stream (icecast only) */
-  ARG_URL,                      /* Url of stream (I'm guessing) */
+  ARG_URL                       /* Url of stream (I'm guessing) */
 };
 
-static GstPadTemplate *
-sink_template_factory (void)
-{
-  static GstPadTemplate *template = NULL;
-
-  if (!template) {
-    template = gst_pad_template_new ("sink",
-        GST_PAD_SINK,
-        GST_PAD_ALWAYS,
-        gst_caps_new ("shout2send_sink",
-            "application/ogg",
-            NULL),
-        gst_caps_new ("shout2send_sink",
-            "audio/mpeg",
-            gst_props_new ("mpegversion", GST_PROPS_INT (1),
-                "layer", GST_PROPS_INT_RANGE (1, 3), NULL)), NULL);
-  }
-
-  return template;
-}
+static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("application/ogg; "
+        "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]")
+    );
 
 static void gst_shout2send_class_init (GstShout2sendClass * klass);
 static void gst_shout2send_base_init (GstShout2sendClass * klass);
 static void gst_shout2send_init (GstShout2send * shout2send);
 
 static void gst_shout2send_chain (GstPad * pad, GstData * _data);
-static GstPadLinkReturn gst_shout2send_connect (GstPad * pad, GstCaps * caps);
+static GstPadLinkReturn gst_shout2send_connect (GstPad * pad,
+    const GstCaps * caps);
 
 static void gst_shout2send_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -95,7 +83,7 @@ static GstElementStateReturn gst_shout2send_change_state (GstElement * element);
 
 static GstElementClass *parent_class = NULL;
 
-/*static guint gst_shout2send_signals[LAST_SIGNAL] = { 0 }; */
+static guint gst_shout2send_signals[LAST_SIGNAL] = { 0 };
 
 #define GST_TYPE_SHOUT_PROTOCOL (gst_shout2send_protocol_get_type())
 static GType
@@ -103,13 +91,13 @@ gst_shout2send_protocol_get_type (void)
 {
   static GType shout2send_protocol_type = 0;
   static GEnumValue shout2send_protocol[] = {
-    {SHOUT2SEND_PROTOCOL_ICE, "1", "Ice Protocol"},
-    {SHOUT2SEND_PROTOCOL_XAUDIOCAST, "2",
+    {SHOUT2SEND_PROTOCOL_XAUDIOCAST, "1",
         "Xaudiocast Protocol (icecast 1.3.x)"},
-    {SHOUT2SEND_PROTOCOL_ICY, "3", "Icy Protocol (ShoutCast)"},
-    {SHOUT2SEND_PROTOCOL_HTTP, "4", "Http Protocol (icecast 2.x)"},
+    {SHOUT2SEND_PROTOCOL_ICY, "2", "Icy Protocol (ShoutCast)"},
+    {SHOUT2SEND_PROTOCOL_HTTP, "3", "Http Protocol (icecast 2.x)"},
     {0, NULL, NULL},
   };
+
   if (!shout2send_protocol_type) {
     shout2send_protocol_type =
         g_enum_register_static ("GstShout2SendProtocol", shout2send_protocol);
@@ -134,6 +122,7 @@ gst_shout2send_get_type (void)
       0,
       (GInstanceInitFunc) gst_shout2send_init,
     };
+
     shout2send_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstShout2send",
         &shout2send_info, 0);
@@ -146,7 +135,8 @@ gst_shout2send_base_init (GstShout2sendClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class, sink_template_factory ());
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
   gst_element_class_set_details (element_class, &shout2send_details);
 }
 
@@ -185,7 +175,12 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_URL, g_param_spec_string ("url", "url", "url", NULL, G_PARAM_READWRITE));        /* CHECKME */
 
 
-
+  /* signals */
+  gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM] =
+      g_signal_new ("connection-problem", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_CLEANUP, G_STRUCT_OFFSET (GstShout2sendClass,
+          connection_problem), NULL, NULL, g_cclosure_marshal_VOID__INT,
+      G_TYPE_NONE, 1, G_TYPE_INT);
   gobject_class->set_property = gst_shout2send_set_property;
   gobject_class->get_property = gst_shout2send_get_property;
 
@@ -196,7 +191,8 @@ static void
 gst_shout2send_init (GstShout2send * shout2send)
 {
   shout2send->sinkpad =
-      gst_pad_new_from_template (sink_template_factory (), "sink");
+      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
+      "sink");
   gst_element_add_pad (GST_ELEMENT (shout2send), shout2send->sinkpad);
   gst_pad_set_chain_function (shout2send->sinkpad, gst_shout2send_chain);
 
@@ -232,7 +228,10 @@ gst_shout2send_chain (GstPad * pad, GstData * _data)
   ret = shout_send (shout2send->conn, GST_BUFFER_DATA (buf),
       GST_BUFFER_SIZE (buf));
   if (ret != SHOUTERR_SUCCESS) {
-    g_warning ("send error: %s...\n", shout_get_error (shout2send->conn));
+    GST_WARNING ("send error: %s...\n", shout_get_error (shout2send->conn));
+    g_signal_emit (G_OBJECT (shout2send),
+        gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM], 0,
+        shout_get_errno (shout2send->conn));
   }
 
   shout_sync (shout2send->conn);
@@ -361,15 +360,17 @@ gst_shout2send_get_property (GObject * object, guint prop_id, GValue * value,
 }
 
 static GstPadLinkReturn
-gst_shout2send_connect (GstPad * pad, GstCaps * caps)
+gst_shout2send_connect (GstPad * pad, const GstCaps * caps)
 {
+  const gchar *mimetype;
 
-  if (!strcmp (gst_caps_get_mime (caps), "audio/mpeg")) {
+  mimetype = gst_structure_get_name (gst_caps_get_structure (caps, 0));
+  if (!strcmp (mimetype, "audio/mpeg")) {
     audio_format = SHOUT_FORMAT_MP3;
     return GST_PAD_LINK_OK;
   }
 
-  if (!strcmp (gst_caps_get_mime (caps), "application/ogg")) {
+  if (!strcmp (mimetype, "application/ogg")) {
     audio_format = SHOUT_FORMAT_VORBIS;
     return GST_PAD_LINK_OK;
   } else {
@@ -400,9 +401,6 @@ gst_shout2send_change_state (GstElement * element)
       shout2send->conn = shout_new ();
 
       switch (shout2send->protocol) {
-        case SHOUT2SEND_PROTOCOL_ICE:
-          proto = SHOUT_PROTOCOL_ICE;
-          break;
         case SHOUT2SEND_PROTOCOL_XAUDIOCAST:
           proto = SHOUT_PROTOCOL_XAUDIOCAST;
           break;
