@@ -22,9 +22,12 @@
 #endif
 
 #include "vorbisenc.h"
+#include "oggvorbisenc.h"
 #include "vorbisdec.h"
+#include "vorbisparse.h"
 
 GST_DEBUG_CATEGORY (vorbisdec_debug);
+GST_DEBUG_CATEGORY (vorbisparse_debug);
 
 static gboolean
 plugin_init (GstPlugin * plugin)
@@ -36,6 +39,10 @@ plugin_init (GstPlugin * plugin)
     return FALSE;
 
   if (!gst_element_register (plugin, "vorbisenc", GST_RANK_NONE,
+          GST_TYPE_OGGVORBISENC))
+    return FALSE;
+
+  if (!gst_element_register (plugin, "rawvorbisenc", GST_RANK_NONE,
           GST_TYPE_VORBISENC))
     return FALSE;
 
@@ -43,8 +50,14 @@ plugin_init (GstPlugin * plugin)
           gst_vorbis_dec_get_type ()))
     return FALSE;
 
+  if (!gst_element_register (plugin, "vorbisparse", GST_RANK_NONE,
+          gst_vorbis_parse_get_type ()))
+    return FALSE;
+
   GST_DEBUG_CATEGORY_INIT (vorbisdec_debug, "vorbisdec", 0,
       "vorbis decoding element");
+  GST_DEBUG_CATEGORY_INIT (vorbisparse_debug, "vorbisparse", 0,
+      "vorbis parsing element");
   return TRUE;
 }
 
