@@ -63,8 +63,8 @@ enum
 enum
 {
   ARG_0,
-  ARG_NUM_PADS,
-  /* FILL ME */
+  ARG_NUM_PADS
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate gst_adder_src_template =
@@ -112,6 +112,7 @@ gst_adder_get_type (void)
       sizeof (GstAdder), 0,
       (GInstanceInitFunc) gst_adder_init,
     };
+
     adder_type = g_type_register_static (GST_TYPE_ELEMENT, "GstAdder",
         &adder_info, 0);
     GST_DEBUG_CATEGORY_INIT (gst_adder_debug, "adder", 0,
@@ -453,15 +454,18 @@ gst_adder_loop (GstElement * element)
       GST_LOG ("done copying data");
     }
   }
-  if (adder->width == 0)
-    GST_ELEMENT_ERROR (adder, CORE, NEGOTIATION, (NULL),
-              ("width is 0"));
-  if (adder->channels == 0)
-    GST_ELEMENT_ERROR (adder, CORE, NEGOTIATION, (NULL),
-              ("channels is 0"));
-  if (adder->rate == 0)
-    GST_ELEMENT_ERROR (adder, CORE, NEGOTIATION, (NULL),
-              ("rate is 0"));
+  if (adder->width == 0) {
+    GST_ELEMENT_ERROR (adder, CORE, NEGOTIATION, (NULL), ("width is 0"));
+    return;
+  }
+  if (adder->channels == 0) {
+    GST_ELEMENT_ERROR (adder, CORE, NEGOTIATION, (NULL), ("channels is 0"));
+    return;
+  }
+  if (adder->rate == 0) {
+    GST_ELEMENT_ERROR (adder, CORE, NEGOTIATION, (NULL), ("rate is 0"));
+    return;
+  }
 
   GST_BUFFER_TIMESTAMP (buf_out) = adder->timestamp;
   if (adder->format == GST_ADDER_FORMAT_FLOAT)
