@@ -46,6 +46,7 @@
 /* for audio codec IDs */
 #include <gst/riff/riff-ids.h>
 
+#include "asfheaders.h"
 #include "gstasfmux.h"
 
 /* elementfactory information */
@@ -65,7 +66,7 @@ enum
 
 enum
 {
-  ARG_0,
+  ARG_0
 };
 
 static GstStaticPadTemplate gst_asfmux_src_template =
@@ -174,6 +175,7 @@ gst_asfmux_get_type (void)
       0,
       (GInstanceInitFunc) gst_asfmux_init,
     };
+
     asfmux_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstAsfMux", &asfmux_info, 0);
   }
@@ -370,7 +372,7 @@ gst_asfmux_vidsink_link (GstPad * pad, const GstCaps * caps)
 
     goto done;
   }
-  return GST_PAD_LINK_REFUSED;
+/*  return GST_PAD_LINK_REFUSED; */
 
 done:
   stream->bitrate = 1024 * 1024;
@@ -476,7 +478,7 @@ gst_asfmux_audsink_link (GstPad * pad, const GstCaps * caps)
 
     goto done;
   }
-  return GST_PAD_LINK_REFUSED;
+/*  return GST_PAD_LINK_REFUSED; */
 
 done:
   stream->bitrate = stream->header.audio.byte_rate * 8;
@@ -1054,7 +1056,7 @@ gst_asfmux_file_stop (GstAsfMux * asfmux)
     gst_pad_push (asfmux->srcpad, GST_DATA (footer));
   } else if (gst_asfmux_can_seek (asfmux)) {
     /* rewrite an updated header */
-    guint64 filesize;
+    gint64 filesize;
     GstFormat fmt = GST_FORMAT_BYTES;
     GstEvent *event;
 
