@@ -48,22 +48,32 @@ enum
 {
   ARG_0,
   ARG_FILTERLEN,
-  ARG_METHOD,
-  /* FILL ME */
+  ARG_METHOD
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate gst_audioscale_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS ("audio/x-raw-int, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) { 8, 16, 24, 32 }, "
+        "depth = (int) [ 1, 32 ], " "signed = (boolean) { true, false }")
     );
 
 static GstStaticPadTemplate gst_audioscale_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS ("audio/x-raw-int, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, MAX ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) { 8, 16, 24, 32 }, "
+        "depth = (int) [ 1, 32 ], " "signed = (boolean) { true, false }")
     );
 
 #define GST_TYPE_AUDIOSCALE_METHOD (gst_audioscale_method_get_type())
@@ -88,6 +98,7 @@ gst_audioscale_method_get_type (void)
 static void gst_audioscale_base_init (gpointer g_class);
 static void gst_audioscale_class_init (AudioscaleClass * klass);
 static void gst_audioscale_init (Audioscale * audioscale);
+static void gst_audioscale_dispose (GObject * object);
 
 static void gst_audioscale_chain (GstPad * pad, GstData * _data);
 
@@ -149,6 +160,7 @@ gst_audioscale_class_init (AudioscaleClass * klass)
 
   gobject_class->set_property = gst_audioscale_set_property;
   gobject_class->get_property = gst_audioscale_get_property;
+  gobject_class->dispose = gst_audioscale_dispose;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FILTERLEN,
       g_param_spec_int ("filter_length", "filter_length", "filter_length",
@@ -383,6 +395,18 @@ gst_audioscale_init (Audioscale * audioscale)
   /* we will be reinitialized when the G_PARAM_CONSTRUCTs hit */
 }
 
+static void
+gst_audioscale_dispose (GObject * object)
+{
+  Audioscale *audioscale = GST_AUDIOSCALE (object);
+
+  if (audioscale->gst_resample)
+    g_free (audioscale->gst_resample);
+  audioscale->gst_resample = NULL;
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 gst_audioscale_chain (GstPad * pad, GstData * _data)
 {
