@@ -26,25 +26,19 @@
 #include "plugin.h"
 
 static gboolean
-plugin_init (GstPlugin *plugin)
+plugin_init (GstPlugin * plugin)
 {
   if (!gst_element_register (plugin, "interleave",
-			     GST_RANK_NONE, gstplugin_interleave_get_type ()) ||
+          GST_RANK_NONE, gstplugin_interleave_get_type ()) ||
       !gst_element_register (plugin, "deinterleave",
-			     GST_RANK_NONE, gstplugin_deinterleave_get_type ()))
+          GST_RANK_NONE, gstplugin_deinterleave_get_type ()))
     return FALSE;
 
   return TRUE;
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "interleave",
-  "Audio interleaver/deinterleaver",
-  plugin_init,
-  VERSION,
-  GST_LICENSE,
-  GST_PACKAGE,
-  GST_ORIGIN
-)
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "interleave",
+    "Audio interleaver/deinterleaver",
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
