@@ -165,8 +165,6 @@ gst_level_link (GstPad * pad, const GstCaps * caps)
         filter->MS[i] = filter->RMS_dB[i] = 0.0;
   }
 
-  filter->inited = TRUE;
-
   return GST_PAD_LINK_OK;
 }
 
@@ -196,6 +194,13 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
   g_return_if_fail (filter != NULL);
   g_return_if_fail (GST_IS_LEVEL (filter));
 
+  if (!gst_pad_is_negotiated (pad)) {
+    GST_ELEMENT_ERROR (filter, CORE, NEGOTIATION, (NULL),
+        ("sinkpad not negotiated"));
+    gst_data_unref (_data);
+    return;
+  }
+
   for (i = 0; i < filter->channels; ++i)
     filter->CS[i] = filter->peak[i] = filter->MS[i] = filter->RMS_dB[i] = 0.0;
 
@@ -288,12 +293,8 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
 static GstElementStateReturn
 gst_level_change_state (GstElement * element)
 {
-  GstLevel *filter = GST_LEVEL (element);
-
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_PAUSED_TO_PLAYING:
-      if (!filter->inited)
-        return GST_STATE_FAILURE;
       break;
     default:
       break;
@@ -405,7 +406,7 @@ gst_level_class_init (GstLevelClass * klass)
   gst_filter_signals[SIGNAL_LEVEL] =
       g_signal_new ("level", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstLevelClass, level), NULL, NULL,
-      gstlevel_cclosure_marshal_VOID__DOUBLE_INT_DOUBLE_DOUBLE_DOUBLE,
+      gst_level_marshal_VOID__DOUBLE_INT_DOUBLE_DOUBLE_DOUBLE,
       G_TYPE_NONE, 5,
       G_TYPE_DOUBLE, G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
 }
@@ -418,15 +419,14 @@ gst_level_init (GstLevel * filter)
       (&sink_template_factory), "sink");
   gst_pad_set_link_function (filter->sinkpad, gst_level_link);
   gst_pad_set_getcaps_function (filter->sinkpad, gst_pad_proxy_getcaps);
+  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
+  gst_pad_set_chain_function (filter->sinkpad, gst_level_chain);
+
   filter->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&src_template_factory), "src");
   gst_pad_set_link_function (filter->srcpad, gst_level_link);
   gst_pad_set_getcaps_function (filter->srcpad, gst_pad_proxy_getcaps);
-
-  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
-  gst_pad_set_chain_function (filter->sinkpad, gst_level_chain);
-  filter->srcpad = gst_pad_new ("src", GST_PAD_SRC);
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
   filter->CS = NULL;
