@@ -43,8 +43,8 @@ enum
 {
   ARG_0,
   ARG_METADATA,
-  ARG_STREAMINFO,
-  /* FILL ME */
+  ARG_STREAMINFO
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -79,10 +79,6 @@ gst_matroska_demux_change_state (GstElement * element);
 static void gst_matroska_demux_set_clock (GstElement * element,
     GstClock * clock);
 
-/* gobject bla bla */
-static void gst_matroska_demux_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec);
-
 /* caps functions */
 static GstCaps *gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext
     * videocontext,
@@ -154,23 +150,12 @@ gst_matroska_demux_base_init (GstMatroskaDemuxClass * klass)
 static void
 gst_matroska_demux_class_init (GstMatroskaDemuxClass * klass)
 {
-  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  g_object_class_install_property (gobject_class, ARG_METADATA,
-      g_param_spec_boxed ("metadata", "Metadata", "Metadata",
-          GST_TYPE_STRUCTURE, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
-      g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
-          GST_TYPE_STRUCTURE, G_PARAM_READABLE));
-
   parent_class = g_type_class_ref (GST_TYPE_EBML_READ);
 
-  gobject_class->get_property = gst_matroska_demux_get_property;
-
   gstelement_class->change_state = gst_matroska_demux_change_state;
   gstelement_class->send_event = gst_matroska_demux_send_event;
   gstelement_class->set_clock = gst_matroska_demux_set_clock;
@@ -196,7 +181,6 @@ gst_matroska_demux_init (GstMatroskaDemux * demux)
   for (i = 0; i < GST_MATROSKA_DEMUX_MAX_STREAMS; i++) {
     demux->src[i] = NULL;
   }
-  demux->streaminfo = demux->metadata = NULL;
   demux->writing_app = demux->muxing_app = NULL;
   demux->index = NULL;
 
@@ -234,10 +218,6 @@ gst_matroska_demux_reset (GstElement * element)
   demux->num_v_streams = 0;
 
   /* reset media info */
-
-  gst_caps_replace (&demux->metadata, NULL);
-  gst_caps_replace (&demux->streaminfo, NULL);
-
   g_free (demux->writing_app);
   demux->writing_app = NULL;
   g_free (demux->muxing_app);
@@ -1491,6 +1471,37 @@ gst_matroska_demux_parse_metadata (GstMatroskaDemux * demux,
   gboolean res = TRUE;
   guint32 id;
   guint64 length = 0;
+  struct
+  {
+    gchar *matroska_tagname;
+    gchar *gstreamer_tagname;
+  }
+  tag_conv[] =
+  {
+    {
+    GST_MATROSKA_TAG_ID_TITLE, GST_TAG_TITLE}
+    , {
+    GST_MATROSKA_TAG_ID_AUTHOR, GST_TAG_ARTIST}
+    , {
+    GST_MATROSKA_TAG_ID_ALBUM, GST_TAG_ALBUM}
+    , {
+    GST_MATROSKA_TAG_ID_COMMENTS, GST_TAG_COMMENT}
+    , {
+    GST_MATROSKA_TAG_ID_BITSPS, GST_TAG_BITRATE}
+    , {
+    GST_MATROSKA_TAG_ID_ENCODER, GST_TAG_ENCODER}
+    , {
+    GST_MATROSKA_TAG_ID_DATE, GST_TAG_DATE}
+    , {
+    GST_MATROSKA_TAG_ID_ISRC, GST_TAG_ISRC}
+    , {
+    GST_MATROSKA_TAG_ID_COPYRIGHT, GST_TAG_COPYRIGHT}
+    , {
+    NULL, NULL}
+  };
+  gint i;
+  gboolean have_tags = FALSE;
+  GstTagList *taglist = gst_tag_list_new ();
 
   if (prevent_eos) {
     length = gst_bytestream_length (ebml->bs);
@@ -1512,8 +1523,122 @@ gst_matroska_demux_parse_metadata (GstMatroskaDemux * demux,
     }
 
     switch (id) {
+      case GST_MATROSKA_ID_TAG:
+        if (!gst_ebml_read_master (ebml, &id)) {
+          res = FALSE;
+          break;
+        }
+
+        while (res) {
+          /* read all sub-entries */
+          if (prevent_eos && length == gst_bytestream_tell (ebml->bs)) {
+            res = FALSE;
+            break;
+          } else if (!(id = gst_ebml_peek_id (ebml, &demux->level_up))) {
+            res = FALSE;
+            break;
+          } else if (demux->level_up) {
+            demux->level_up--;
+            break;
+          }
+
+          switch (id) {
+            case GST_MATROSKA_ID_SIMPLETAG:{
+              gchar *tag = NULL, *value = NULL;
+
+              if (!gst_ebml_read_master (ebml, &id)) {
+                res = FALSE;
+                break;
+              }
+
+              while (res) {
+                /* read all sub-entries */
+                if (prevent_eos && length == gst_bytestream_tell (ebml->bs)) {
+                  res = FALSE;
+                  break;
+                } else if (!(id = gst_ebml_peek_id (ebml, &demux->level_up))) {
+                  res = FALSE;
+                  break;
+                } else if (demux->level_up) {
+                  demux->level_up--;
+                  break;
+                }
+
+                switch (id) {
+                  case GST_MATROSKA_ID_TAGNAME:
+                    g_free (tag);
+                    res = gst_ebml_read_ascii (ebml, &id, &tag);
+                    break;
+
+                  case GST_MATROSKA_ID_TAGSTRING:
+                    g_free (value);
+                    res = gst_ebml_read_utf8 (ebml, &id, &value);
+                    break;
+
+                  default:
+                    GST_WARNING ("Unknown entry 0x%x in metadata collection",
+                        id);
+                    /* fall-through */
+
+                  case GST_EBML_ID_VOID:
+                    if (!gst_ebml_read_skip (ebml))
+                      res = FALSE;
+                    break;
+                }
+
+                if (demux->level_up) {
+                  demux->level_up--;
+                  break;
+                }
+              }
+
+              if (tag && value) {
+                for (i = 0; tag_conv[i].matroska_tagname != NULL; i++) {
+                  if (!strcmp (tag_conv[i].matroska_tagname, tag)) {
+                    GValue src = { 0 }
+                    , dest =
+                    {
+                    0};
+                    const gchar *type = tag_conv[i].gstreamer_tagname;
+                    GType dest_type = gst_tag_get_type (type);
+
+                    g_value_init (&src, G_TYPE_STRING);
+                    g_value_set_string (&src, value);
+                    g_value_init (&dest, dest_type);
+                    g_value_transform (&src, &dest);
+                    g_value_unset (&src);
+                    gst_tag_list_add_values (taglist, GST_TAG_MERGE_APPEND,
+                        type, &dest, NULL);
+                    g_value_unset (&dest);
+                    have_tags = TRUE;
+                    break;
+                  }
+                }
+              }
+              g_free (tag);
+              g_free (value);
+              break;
+            }
+
+            default:
+              GST_WARNING ("Unknown entry 0x%x in metadata collection", id);
+              /* fall-through */
+
+            case GST_EBML_ID_VOID:
+              if (!gst_ebml_read_skip (ebml))
+                res = FALSE;
+              break;
+          }
+
+          if (demux->level_up) {
+            demux->level_up--;
+            break;
+          }
+        }
+        break;
+
       default:
-        GST_WARNING ("metadata unimplemented");
+        GST_WARNING ("Unknown entry 0x%x in metadata header", id);
         /* fall-through */
 
       case GST_EBML_ID_VOID:
@@ -1528,6 +1653,22 @@ gst_matroska_demux_parse_metadata (GstMatroskaDemux * demux,
     }
   }
 
+  if (have_tags) {
+    const GList *padlist;
+
+    /* let the world know about this wonderful thing */
+    for (padlist = gst_element_get_pad_list (GST_ELEMENT (ebml));
+        padlist != NULL; padlist = padlist->next) {
+      if (GST_PAD_IS_SRC (padlist->data) && GST_PAD_IS_USABLE (padlist->data)) {
+        gst_pad_push (GST_PAD (padlist->data),
+            GST_DATA (gst_event_new_tag (taglist)));
+      }
+    }
+    gst_element_found_tags (GST_ELEMENT (ebml), taglist);
+  } else {
+    gst_tag_list_free (taglist);
+  }
+
   return res;
 }
 
@@ -2395,7 +2536,8 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
     } else {
       gst_caps_set_simple (caps, "width", GST_TYPE_INT_RANGE, 32, 64, NULL);
     }
-  } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_AC3) ||
+  } else if (!strncmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_AC3,
+          strlen (GST_MATROSKA_CODEC_ID_AUDIO_AC3)) ||
       !strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_DTS)) {
     caps = gst_caps_new_simple ("audio/x-ac3", NULL);
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_VORBIS)) {
@@ -2447,6 +2589,7 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
         "mpegversion", G_TYPE_INT, mpegversion, NULL);
   } else {
     GST_WARNING ("Unknown codec '%s', cannot build Caps", codec_id);
+    g_print ("Codec=%s\n", codec_id);
     return NULL;
   }
 
@@ -2514,28 +2657,6 @@ gst_matroska_demux_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
-static void
-gst_matroska_demux_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec)
-{
-  GstMatroskaDemux *demux;
-
-  g_return_if_fail (GST_IS_MATROSKA_DEMUX (object));
-  demux = GST_MATROSKA_DEMUX (object);
-
-  switch (prop_id) {
-    case ARG_STREAMINFO:
-      g_value_set_boxed (value, demux->streaminfo);
-      break;
-    case ARG_METADATA:
-      g_value_set_boxed (value, demux->metadata);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 gboolean
 gst_matroska_demux_plugin_init (GstPlugin * plugin)
 {
