@@ -23,6 +23,7 @@
 
 #include <gst/gst.h>
 #include <gst/video/video.h>
+#include <gst/audio/audio.h>
 #include "monoscope.h"
 
 #define GST_TYPE_MONOSCOPE (gst_monoscope_get_type())
@@ -34,32 +35,33 @@
 typedef struct _GstMonoscope GstMonoscope;
 typedef struct _GstMonoscopeClass GstMonoscopeClass;
 
-struct _GstMonoscope {
+struct _GstMonoscope
+{
   GstElement element;
 
   /* pads */
-  GstPad *sinkpad,*srcpad;
-  GstBufferPool *peerpool;
+  GstPad *sinkpad, *srcpad;
 
   /* the timestamp of the next frame */
   guint64 next_time;
   gint16 datain[512];
 
   /* video state */
-  gfloat fps;
+  gdouble fps;
   gint width;
   gint height;
   gboolean first_buffer;
 
   /* visualisation state */
-  struct monoscope_state * visstate;
+  struct monoscope_state *visstate;
 };
 
-struct _GstMonoscopeClass {
+struct _GstMonoscopeClass
+{
   GstElementClass parent_class;
 };
 
-GType gst_monoscope_get_type(void);
+GType gst_monoscope_get_type (void);
 
 
 /* elementfactory information */
@@ -71,63 +73,63 @@ static GstElementDetails gst_monoscope_details = {
 };
 
 /* signals and args */
-enum {
+enum
+{
   /* FILL ME */
   LAST_SIGNAL
 };
 
-enum {
-  ARG_0,
-  /* FILL ME */
+enum
+{
+  ARG_0
+      /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (src_template,
-  "src",
-  GST_PAD_SRC,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "monoscopesrc",
-    "video/x-raw-rgb",
-      "bpp",		GST_PROPS_INT (32),
-      "depth",		GST_PROPS_INT (32),
-      "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
-      "red_mask",   	GST_PROPS_INT (R_MASK_32),
-      "green_mask", 	GST_PROPS_INT (G_MASK_32),
-      "blue_mask",  	GST_PROPS_INT (B_MASK_32),
-      "width",		GST_PROPS_INT_RANGE (16, 4096),
-      "height",		GST_PROPS_INT_RANGE (16, 4096),
-      "framerate",	GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT)
-  )
-)
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("video/x-raw-rgb, "
+        "bpp = (int) 32, "
+        "depth = (int) 24, "
+        "endianness = (int) BIG_ENDIAN, "
+        "red_mask = (int) " GST_VIDEO_BYTE2_MASK_32 ", "
+        "green_mask = (int) " GST_VIDEO_BYTE3_MASK_32 ", "
+        "blue_mask = (int) " GST_VIDEO_BYTE4_MASK_32 ", "
+        "width = (int)256, "
+        "height = (int)128, " "framerate = " GST_VIDEO_FPS_RANGE)
+    );
+#else
+static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("video/x-raw-rgb, "
+        "bpp = (int) 32, "
+        "depth = (int) 24, "
+        "endianness = (int) BIG_ENDIAN, "
+        "red_mask = (int) " GST_VIDEO_BYTE3_MASK_32 ", "
+        "green_mask = (int) " GST_VIDEO_BYTE2_MASK_32 ", "
+        "blue_mask = (int) " GST_VIDEO_BYTE1_MASK_32 ", "
+        "width = (int)256, "
+        "height = (int)128, " "framerate = " GST_VIDEO_FPS_RANGE)
+    );
+#endif
 
-GST_PAD_TEMPLATE_FACTORY (sink_template,
-  "sink",					/* the name of the pads */
-  GST_PAD_SINK,				/* type of the pad */
-  GST_PAD_ALWAYS,				/* ALWAYS/SOMETIMES */
-  GST_CAPS_NEW (
-    "monoscopesink",				/* the name of the caps */
-    "audio/x-raw-int",				/* the mime type of the caps */
-       /* Properties follow: */
-      "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-      "signed",     GST_PROPS_BOOLEAN (TRUE),
-      "width",      GST_PROPS_INT (16),
-      "depth",      GST_PROPS_INT (16),
-      "rate",       GST_PROPS_INT_RANGE (8000, 96000),
-      "channels",   GST_PROPS_INT (1)
-  )
-)
+static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (GST_AUDIO_INT_STANDARD_PAD_TEMPLATE_CAPS)
+    );
 
 
-static void	gst_monoscope_class_init	(GstMonoscopeClass *klass);
-static void	gst_monoscope_base_init		(GstMonoscopeClass *klass);
-static void	gst_monoscope_init		(GstMonoscope *monoscope);
+static void gst_monoscope_class_init (GstMonoscopeClass * klass);
+static void gst_monoscope_base_init (GstMonoscopeClass * klass);
+static void gst_monoscope_init (GstMonoscope * monoscope);
 
-static void	gst_monoscope_chain		(GstPad *pad, GstData *_data);
+static void gst_monoscope_chain (GstPad * pad, GstData * _data);
 
-static GstPadLinkReturn 
-		gst_monoscope_sinkconnect 	(GstPad *pad, GstCaps *caps);
 static GstPadLinkReturn
-		gst_monoscope_srcconnect 	(GstPad *pad, GstCaps *caps);
+gst_monoscope_srcconnect (GstPad * pad, const GstCaps * caps);
 
 static GstElementClass *parent_class = NULL;
 
@@ -138,9 +140,9 @@ gst_monoscope_get_type (void)
 
   if (!type) {
     static const GTypeInfo info = {
-      sizeof (GstMonoscopeClass),      
-      (GBaseInitFunc) gst_monoscope_base_init,      
-      NULL,      
+      sizeof (GstMonoscopeClass),
+      (GBaseInitFunc) gst_monoscope_base_init,
+      NULL,
       (GClassInitFunc) gst_monoscope_class_init,
       NULL,
       NULL,
@@ -148,119 +150,78 @@ gst_monoscope_get_type (void)
       0,
       (GInstanceInitFunc) gst_monoscope_init,
     };
+
     type = g_type_register_static (GST_TYPE_ELEMENT, "GstMonoscope", &info, 0);
   }
   return type;
 }
 
 static void
-gst_monoscope_base_init (GstMonoscopeClass *klass)
+gst_monoscope_base_init (GstMonoscopeClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   gst_element_class_add_pad_template (element_class,
-		GST_PAD_TEMPLATE_GET (src_template));
+      gst_static_pad_template_get (&src_template));
   gst_element_class_add_pad_template (element_class,
-		GST_PAD_TEMPLATE_GET (sink_template));
+      gst_static_pad_template_get (&sink_template));
   gst_element_class_set_details (element_class, &gst_monoscope_details);
 }
 
 static void
-gst_monoscope_class_init(GstMonoscopeClass *klass)
+gst_monoscope_class_init (GstMonoscopeClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass*) klass;
-  gstelement_class = (GstElementClass*) klass;
+  gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 }
 
 static void
-gst_monoscope_init (GstMonoscope *monoscope)
+gst_monoscope_init (GstMonoscope * monoscope)
 {
   /* create the sink and src pads */
-  monoscope->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sink_template ), "sink");
-  monoscope->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (src_template ), "src");
+  monoscope->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
+      "sink");
+  monoscope->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&src_template),
+      "src");
   gst_element_add_pad (GST_ELEMENT (monoscope), monoscope->sinkpad);
   gst_element_add_pad (GST_ELEMENT (monoscope), monoscope->srcpad);
 
   gst_pad_set_chain_function (monoscope->sinkpad, gst_monoscope_chain);
-  gst_pad_set_link_function (monoscope->sinkpad, gst_monoscope_sinkconnect);
   gst_pad_set_link_function (monoscope->srcpad, gst_monoscope_srcconnect);
 
   monoscope->next_time = 0;
-  monoscope->peerpool = NULL;
 
   /* reset the initial video state */
   monoscope->first_buffer = TRUE;
   monoscope->width = 256;
   monoscope->height = 128;
-  monoscope->fps = 25.; /* desired frame rate */
+  monoscope->fps = 25.;         /* desired frame rate */
 }
 
 static GstPadLinkReturn
-gst_monoscope_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_monoscope_srcconnect (GstPad * pad, const GstCaps * caps)
 {
-  GstMonoscope *monoscope;
-  monoscope = GST_MONOSCOPE (gst_pad_get_parent (pad));
+  GstMonoscope *monoscope = GST_MONOSCOPE (gst_pad_get_parent (pad));
+  GstStructure *structure;
 
-  if (!GST_CAPS_IS_FIXED (caps)) {
-    return GST_PAD_LINK_DELAYED;
-  }
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int (structure, "width", &monoscope->width);
+  gst_structure_get_int (structure, "height", &monoscope->height);
+  gst_structure_get_double (structure, "framerate", &monoscope->fps);
 
   return GST_PAD_LINK_OK;
 }
 
-static GstPadLinkReturn
-gst_monoscope_negotiate (GstMonoscope *monoscope)
-{
-  GstCaps *caps;
-
-  caps = GST_CAPS_NEW (
-		     "monoscopesrc",
-		     "video/x-raw-rgb",
-		       "bpp", 		GST_PROPS_INT (32), 
-		       "depth", 	GST_PROPS_INT (32), 
-		       "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN), 
-		       "red_mask", 	GST_PROPS_INT (R_MASK_32), 
-		       "green_mask", 	GST_PROPS_INT (G_MASK_32), 
-		       "blue_mask", 	GST_PROPS_INT (B_MASK_32), 
-		       "width", 	GST_PROPS_INT (monoscope->width), 
-		       "height", 	GST_PROPS_INT (monoscope->height),
-               "framerate",     GST_PROPS_FLOAT (monoscope->fps)
-		   );
-
-  return gst_pad_try_set_caps (monoscope->srcpad, caps);
-}
-
-static GstPadLinkReturn
-gst_monoscope_srcconnect (GstPad *pad, GstCaps *caps)
-{
-  GstPadLinkReturn ret;
-  GstMonoscope *monoscope = GST_MONOSCOPE (gst_pad_get_parent (pad));
-
-  if (gst_caps_has_property_typed (caps, "width", GST_PROPS_INT_TYPE)) {
-    gst_caps_get_int (caps, "width", &monoscope->width);
-  }
-  if (gst_caps_has_property_typed (caps, "height", GST_PROPS_INT_TYPE)) {
-    gst_caps_get_int (caps, "height", &monoscope->height);
-  }
-  if (gst_caps_has_property_typed (caps, "framerate", GST_PROPS_FLOAT_TYPE)) {
-    gst_caps_get_float (caps, "framerate", &monoscope->fps);
-  }
-
-  if ((ret = gst_monoscope_negotiate (monoscope)) <= 0)
-    return ret;
-
-  return GST_PAD_LINK_DONE;
-}
-
 static void
-gst_monoscope_chain (GstPad *pad, GstData *_data)
+gst_monoscope_chain (GstPad * pad, GstData * _data)
 {
   GstBuffer *bufin = GST_BUFFER (_data);
   GstMonoscope *monoscope;
@@ -279,24 +240,25 @@ gst_monoscope_chain (GstPad *pad, GstData *_data)
 
   /* FIXME: should really select the first 1024 samples after the timestamp. */
   if (GST_BUFFER_TIMESTAMP (bufin) < monoscope->next_time || samples_in < 1024) {
-    GST_DEBUG ("timestamp is %" G_GUINT64_FORMAT ": want >= %" G_GUINT64_FORMAT, GST_BUFFER_TIMESTAMP (bufin), monoscope->next_time);
+    GST_DEBUG ("timestamp is %" G_GUINT64_FORMAT ": want >= %" G_GUINT64_FORMAT,
+        GST_BUFFER_TIMESTAMP (bufin), monoscope->next_time);
     gst_buffer_unref (bufin);
     return;
   }
 
   data = (gint16 *) GST_BUFFER_DATA (bufin);
   /* FIXME: Select samples in a better way. */
-  for (i=0; i < 512; i++) {
+  for (i = 0; i < 512; i++) {
     monoscope->datain[i] = *data++;
   }
 
   if (monoscope->first_buffer) {
     monoscope->visstate = monoscope_init (monoscope->width, monoscope->height);
-    g_assert(monoscope->visstate != 0);
+    g_assert (monoscope->visstate != 0);
     GST_DEBUG ("making new pad");
-    if (!GST_PAD_CAPS (monoscope->srcpad)) {
-      if (gst_monoscope_negotiate (monoscope) <= 0) {
-        gst_element_error (GST_ELEMENT (monoscope), "could not set caps");
+    if (!gst_pad_is_negotiated (monoscope->srcpad)) {
+      if (gst_pad_renegotiate (monoscope->srcpad) <= 0) {
+        GST_ELEMENT_ERROR (monoscope, CORE, NEGOTIATION, (NULL), (NULL));
         return;
       }
     }
@@ -305,7 +267,8 @@ gst_monoscope_chain (GstPad *pad, GstData *_data)
 
   bufout = gst_buffer_new ();
   GST_BUFFER_SIZE (bufout) = monoscope->width * monoscope->height * 4;
-  GST_BUFFER_DATA (bufout) = (guchar *) monoscope_update (monoscope->visstate, monoscope->datain);
+  GST_BUFFER_DATA (bufout) =
+      (guchar *) monoscope_update (monoscope->visstate, monoscope->datain);
   GST_BUFFER_TIMESTAMP (bufout) = monoscope->next_time;
   GST_BUFFER_FLAG_SET (bufout, GST_BUFFER_DONTFREE);
 
@@ -320,20 +283,14 @@ gst_monoscope_chain (GstPad *pad, GstData *_data)
 }
 
 static gboolean
-plugin_init (GstPlugin *plugin)
+plugin_init (GstPlugin * plugin)
 {
-  return gst_element_register(plugin, "monoscope",
-			      GST_RANK_NONE, GST_TYPE_MONOSCOPE);
+  return gst_element_register (plugin, "monoscope",
+      GST_RANK_NONE, GST_TYPE_MONOSCOPE);
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "monoscope",
-  "Monoscope visualization",
-  plugin_init,
-  VERSION,
-  "LGPL",
-  GST_PACKAGE,
-  GST_ORIGIN
-)
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "monoscope",
+    "Monoscope visualization",
+    plugin_init, VERSION, "LGPL", GST_PACKAGE, GST_ORIGIN)
