@@ -23,18 +23,24 @@
 #endif
 #include "qtdemux.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <zlib.h>
 
 GST_DEBUG_CATEGORY_EXTERN (qtdemux_debug);
 #define GST_CAT_DEFAULT qtdemux_debug
 
-#define QTDEMUX_GUINT32_GET(a) GUINT32_FROM_BE(*(guint32 *)(a))
-#define QTDEMUX_GUINT16_GET(a) GUINT16_FROM_BE(*(guint16 *)(a))
+/* temporary hack */
+#define g_print(...)            /* */
+#define gst_util_dump_mem(a,b)  /* */
+
+#define QTDEMUX_GUINT32_GET(a)	(GST_READ_UINT32_BE(a))
+#define QTDEMUX_GUINT24_GET(a)	(GST_READ_UINT32_BE(a) >> 8)
+#define QTDEMUX_GUINT16_GET(a)	(GST_READ_UINT16_BE(a))
 #define QTDEMUX_GUINT8_GET(a) (*(guint8 *)(a))
-#define QTDEMUX_FP32_GET(a) (GUINT32_FROM_BE(*(guint16 *)(a))/65536.0)
-#define QTDEMUX_FP16_GET(a) (GUINT16_FROM_BE(*(guint16 *)(a))/256.0)
-#define QTDEMUX_FOURCC_GET(a) GUINT32_FROM_LE(*(guint32 *)(a))
+#define QTDEMUX_FP32_GET(a)	((GST_READ_UINT32_BE(a))/65536.0)
+#define QTDEMUX_FP16_GET(a)	((GST_READ_UINT16_BE(a))/256.0)
+#define QTDEMUX_FOURCC_GET(a)	(GST_READ_UINT32_LE(a))
 
 #define QTDEMUX_GUINT64_GET(a) ((((guint64)QTDEMUX_GUINT32_GET(a))<<32)|QTDEMUX_GUINT32_GET(((void *)a)+4))
 
@@ -99,7 +105,7 @@ enum QtDemuxState
   QTDEMUX_STATE_SEEKING,
   QTDEMUX_STATE_MOVIE,
   QTDEMUX_STATE_SEEKING_EOS,
-  QTDEMUX_STATE_EOS,
+  QTDEMUX_STATE_EOS
 };
 
 static GNode *qtdemux_tree_get_child_by_type (GNode * node, guint32 fourcc);
@@ -123,10 +129,10 @@ enum
 };
 
 static GstStaticPadTemplate gst_qtdemux_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
-    GST_PAD_SOMETIMES,
-    GST_STATIC_CAPS ("video/quicktime")
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("video/quicktime; audio/x-m4a")
     );
 
 static GstStaticPadTemplate gst_qtdemux_videosrc_template =
@@ -156,10 +162,15 @@ static void qtdemux_parse (GstQTDemux * qtdemux, GNode * node, void *buffer,
 static QtNodeType *qtdemux_type_get (guint32 fourcc);
 static void qtdemux_node_dump (GstQTDemux * qtdemux, GNode * node);
 static void qtdemux_parse_tree (GstQTDemux * qtdemux);
+static void qtdemux_parse_udta (GstQTDemux * qtdemux, GNode * udta);
+static void qtdemux_tag_add (GstQTDemux * qtdemux, const char *tag,
+    GNode * node);
+static void gst_qtdemux_handle_esds (GstQTDemux * qtdemux,
+    QtDemuxStream * stream, GNode * esds);
 static GstCaps *qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
     const guint8 * stsd_data);
 static GstCaps *qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc,
-    const guint8 * data);
+    const guint8 * data, int len);
 
 static GType
 gst_qtdemux_get_type (void)
@@ -220,6 +231,10 @@ gst_qtdemux_init (GstQTDemux * qtdemux)
   gst_element_set_loop_function (GST_ELEMENT (qtdemux),
       gst_qtdemux_loop_header);
   gst_element_add_pad (GST_ELEMENT (qtdemux), qtdemux->sinkpad);
+
+  qtdemux->last_ts = GST_CLOCK_TIME_NONE;
+  qtdemux->need_discont = FALSE;
+  qtdemux->need_flush = FALSE;
 }
 
 static const GstFormat *
@@ -321,7 +336,8 @@ static gboolean
 gst_qtdemux_handle_src_query (GstPad * pad, GstQueryType type,
     GstFormat * format, gint64 * value)
 {
-  gboolean res = TRUE;
+  gboolean res = FALSE;
+  GstQTDemux *qtdemux = GST_QTDEMUX (gst_pad_get_parent (pad));
 
   //QtDemuxStream *stream = gst_pad_get_element_private(pad);
 
@@ -329,7 +345,11 @@ gst_qtdemux_handle_src_query (GstPad * pad, GstQueryType type,
     case GST_QUERY_TOTAL:
       switch (*format) {
         case GST_FORMAT_TIME:
-          *value = 0;           /* FIXME */
+          if (qtdemux->duration != 0 && qtdemux->timescale != 0) {
+            *value =
+                (guint64) qtdemux->duration * GST_SECOND / qtdemux->timescale;
+            res = TRUE;
+          }
           break;
         case GST_FORMAT_BYTES:
           *value = 0;           /* FIXME */
@@ -345,7 +365,10 @@ gst_qtdemux_handle_src_query (GstPad * pad, GstQueryType type,
     case GST_QUERY_POSITION:
       switch (*format) {
         case GST_FORMAT_TIME:
-          *value = 0;           /* FIXME */
+          if (GST_CLOCK_TIME_IS_VALID (qtdemux->last_ts)) {
+            *value = qtdemux->last_ts;
+            res = TRUE;
+          }
           break;
         case GST_FORMAT_BYTES:
           *value = 0;           /* FIXME */
@@ -370,8 +393,7 @@ static gboolean
 gst_qtdemux_handle_src_event (GstPad * pad, GstEvent * event)
 {
   gboolean res = TRUE;
-
-  //QtDemuxStream *stream = gst_pad_get_element_private(pad);
+  GstQTDemux *qtdemux = GST_QTDEMUX (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
@@ -380,13 +402,35 @@ gst_qtdemux_handle_src_event (GstPad * pad, GstEvent * event)
       switch (GST_EVENT_SEEK_FORMAT (event)) {
         case GST_FORMAT_BYTES:
         case GST_FORMAT_DEFAULT:
+          res = FALSE;
+          break;
         case GST_FORMAT_TIME:
         {
           gint64 desired_offset = GST_EVENT_SEEK_OFFSET (event);
+          gint i = 0, n;
+          QtDemuxStream *stream = gst_pad_get_element_private (pad);
 
           GST_DEBUG ("seeking to %" G_GINT64_FORMAT, desired_offset);
 
-          res = FALSE;
+          if (!stream->n_samples) {
+            res = FALSE;
+            break;
+          }
+
+          /* resync to new time */
+          for (n = 0; n < qtdemux->n_streams; n++) {
+            QtDemuxStream *str = qtdemux->streams[n];
+
+            for (i = 0; i < str->n_samples; i++) {
+              if (str->samples[i].timestamp > desired_offset)
+                break;
+            }
+            str->sample_index = i;
+          }
+          qtdemux->need_discont = TRUE;
+          if (GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH)
+            qtdemux->need_flush = TRUE;
+          break;
         }
         default:
           res = FALSE;
@@ -445,7 +489,6 @@ gst_qtdemux_handle_sink_event (GstQTDemux * qtdemux)
       gst_pad_event_default (qtdemux->sinkpad, event);
       return FALSE;
     case GST_EVENT_FLUSH:
-      //g_warning("flush event");
       break;
     case GST_EVENT_DISCONTINUOUS:
       GST_DEBUG ("discontinuous event");
@@ -467,20 +510,24 @@ gst_qtdemux_change_state (GstElement * element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
-      break;
-    case GST_STATE_READY_TO_PAUSED:
       qtdemux->bs = gst_bytestream_new (qtdemux->sinkpad);
       qtdemux->state = QTDEMUX_STATE_HEADER;
+      GST_DEBUG ("new bytestream");
       /* FIXME */
       break;
+    case GST_STATE_READY_TO_PAUSED:
+      break;
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
-      gst_bytestream_destroy (qtdemux->bs);
+      qtdemux->last_ts = GST_CLOCK_TIME_NONE;
+      qtdemux->need_discont = FALSE;
+      qtdemux->need_flush = FALSE;
       break;
     case GST_STATE_READY_TO_NULL:
+      gst_bytestream_destroy (qtdemux->bs);
       break;
     default:
       break;
@@ -512,6 +559,7 @@ gst_qtdemux_loop_header (GstElement * element)
     case QTDEMUX_STATE_HEADER:
     {
       do {
+        /* FIXME: we peek for 16 bytes, but what if the atom is smaller ? */
         ret = gst_bytestream_peek_bytes (qtdemux->bs, &data, 16);
         if (ret < 16) {
           if (!gst_qtdemux_handle_sink_event (qtdemux)) {
@@ -522,22 +570,25 @@ gst_qtdemux_loop_header (GstElement * element)
         }
       } while (1);
 
-      length = GUINT32_FROM_BE (*(guint32 *) data);
+      length = GST_READ_UINT32_BE (data);
       GST_DEBUG ("length %08x", length);
-      fourcc = GUINT32_FROM_LE (*(guint32 *) (data + 4));
-      GST_DEBUG ("fourcc " GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
+      fourcc = GST_READ_UINT32_LE (data + 4);
+      GST_DEBUG ("atom type " GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
 
       if (length == 0) {
         length = gst_bytestream_length (qtdemux->bs) - cur_offset;
       }
       if (length == 1) {
+        /* this means we have an extended size, which is the 64 bit value of
+         * the next 8 bytes */
         guint32 length1, length2;
 
-        length1 = GUINT32_FROM_BE (*(guint32 *) (data + 8));
+        length1 = GST_READ_UINT32_BE (data + 8);
         GST_DEBUG ("length1 %08x", length1);
-        length2 = GUINT32_FROM_BE (*(guint32 *) (data + 12));
+        length2 = GST_READ_UINT32_BE (data + 12);
         GST_DEBUG ("length2 %08x", length2);
 
+        /* FIXME: I guess someone didn't want to make 64 bit size work :) */
         length = length2;
       }
 
@@ -553,6 +604,7 @@ gst_qtdemux_loop_header (GstElement * element)
           GstBuffer *moov;
 
           do {
+            /* read in the complete data from the moov atom */
             ret = gst_bytestream_read (qtdemux->bs, &moov, length);
             if (ret < length) {
               GST_DEBUG ("read failed (%d < %d)", ret, length);
@@ -563,10 +615,12 @@ gst_qtdemux_loop_header (GstElement * element)
               break;
             }
           } while (1);
+          qtdemux->offset += length;
 
           qtdemux_parse_moov (qtdemux, GST_BUFFER_DATA (moov), length);
-          if (1)
+          if (1) {
             qtdemux_node_dump (qtdemux, qtdemux->moov_node);
+          }
           qtdemux_parse_tree (qtdemux);
           qtdemux->state = QTDEMUX_STATE_MOVIE;
           break;
@@ -580,8 +634,18 @@ gst_qtdemux_loop_header (GstElement * element)
       }
       ret = gst_bytestream_seek (qtdemux->bs, cur_offset + length,
           GST_SEEK_METHOD_SET);
-      qtdemux->offset = cur_offset + length;
       GST_DEBUG ("seek returned %d", ret);
+      if (ret == FALSE) {
+        length = cur_offset + length;
+        cur_offset = qtdemux->offset;
+        length -= cur_offset;
+        if (gst_bytestream_flush (qtdemux->bs, length) == FALSE) {
+          if (!gst_qtdemux_handle_sink_event (qtdemux)) {
+            return;
+          }
+        }
+      }
+      qtdemux->offset = cur_offset + length;
       break;
     }
     case QTDEMUX_STATE_SEEKING_EOS:
@@ -631,6 +695,9 @@ gst_qtdemux_loop_header (GstElement * element)
         }
         ret = gst_bytestream_seek (qtdemux->bs, 0, GST_SEEK_METHOD_END);
         GST_DEBUG ("seek returned %d", ret);
+        if (ret == FALSE) {
+          gst_bytestream_flush (qtdemux->bs, 0xffffffff);
+        }
 
         qtdemux->state = QTDEMUX_STATE_SEEKING_EOS;
         return;
@@ -646,12 +713,22 @@ gst_qtdemux_loop_header (GstElement * element)
           index, stream->sample_index, offset, size,
           stream->samples[stream->sample_index].timestamp);
 
-      cur_offset = gst_bytestream_tell (qtdemux->bs);
+      // cur_offset = gst_bytestream_tell (qtdemux->bs);
+      cur_offset = qtdemux->offset;
       if (offset != cur_offset) {
         GST_DEBUG ("seeking to offset %d", offset);
         GST_LOG ("seeking to offset %d", offset);
         ret = gst_bytestream_seek (qtdemux->bs, offset, GST_SEEK_METHOD_SET);
         GST_DEBUG ("seek returned %d", ret);
+        if (ret == FALSE && offset > cur_offset) {
+          if (gst_bytestream_flush (qtdemux->bs, offset - cur_offset) == FALSE) {
+            if (!gst_qtdemux_handle_sink_event (qtdemux)) {
+              return;
+            }
+          }
+        } else if (ret == FALSE && offset < cur_offset)
+          GST_ERROR ("cannot flush backwards");
+        qtdemux->offset = offset;
         return;
       }
 
@@ -668,6 +745,7 @@ gst_qtdemux_loop_header (GstElement * element)
           break;
         }
       } while (TRUE);
+      qtdemux->offset += size;
 
       if (buf) {
         /* hum... */
@@ -684,8 +762,29 @@ gst_qtdemux_loop_header (GstElement * element)
 
         GST_BUFFER_TIMESTAMP (buf) =
             stream->samples[stream->sample_index].timestamp;
+        qtdemux->last_ts = GST_BUFFER_TIMESTAMP (buf);
         GST_BUFFER_DURATION (buf) =
             stream->samples[stream->sample_index].duration;
+        if (qtdemux->need_flush) {
+          gst_pad_event_default (qtdemux->sinkpad,
+              gst_event_new (GST_EVENT_FLUSH));
+          qtdemux->need_flush = FALSE;
+        }
+        if (qtdemux->need_discont) {
+          GstEvent *event = gst_event_new_discontinuous (FALSE,
+              GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (buf),
+              GST_FORMAT_UNDEFINED);
+          gint n;
+
+          qtdemux->need_discont = FALSE;
+          for (n = 0; n < qtdemux->n_streams; n++) {
+            gst_event_ref (event);
+            gst_pad_push (qtdemux->streams[n]->pad, GST_DATA (event));
+          }
+          gst_event_unref (event);
+        }
+        g_print ("Pushing buf with time=%" GST_TIME_FORMAT "\n",
+            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
         gst_pad_push (stream->pad, GST_DATA (buf));
 
         GST_INFO ("pushing buffer on %" GST_PTR_FORMAT, stream->pad);
@@ -748,6 +847,7 @@ gst_qtdemux_add_stream (GstQTDemux * qtdemux, QtDemuxStream * stream)
   gst_pad_set_formats_function (stream->pad, gst_qtdemux_get_src_formats);
   gst_pad_set_convert_function (stream->pad, gst_qtdemux_src_convert);
 
+  g_print ("setting caps %s\n", gst_caps_to_string (stream->caps));
   gst_pad_set_explicit_caps (stream->pad, stream->caps);
 
   GST_DEBUG ("adding pad %p to qtdemux %p", stream->pad, qtdemux);
@@ -797,6 +897,28 @@ gst_qtdemux_add_stream (GstQTDemux * qtdemux, QtDemuxStream * stream)
 #define FOURCC_cmov	GST_MAKE_FOURCC('c','m','o','v')
 #define FOURCC_dcom	GST_MAKE_FOURCC('d','c','o','m')
 #define FOURCC_cmvd	GST_MAKE_FOURCC('c','m','v','d')
+#define FOURCC_hint	GST_MAKE_FOURCC('h','i','n','t')
+#define FOURCC_mp4a	GST_MAKE_FOURCC('m','p','4','a')
+#define FOURCC_mp4v	GST_MAKE_FOURCC('m','p','4','v')
+#define FOURCC_wave	GST_MAKE_FOURCC('w','a','v','e')
+#define FOURCC_appl	GST_MAKE_FOURCC('a','p','p','l')
+#define FOURCC_esds	GST_MAKE_FOURCC('e','s','d','s')
+#define FOURCC_hnti	GST_MAKE_FOURCC('h','n','t','i')
+#define FOURCC_rtp_	GST_MAKE_FOURCC('r','t','p',' ')
+#define FOURCC_sdp_	GST_MAKE_FOURCC('s','d','p',' ')
+#define FOURCC_meta	GST_MAKE_FOURCC('m','e','t','a')
+#define FOURCC_ilst	GST_MAKE_FOURCC('i','l','s','t')
+#define FOURCC__nam	GST_MAKE_FOURCC(0xa9,'n','a','m')
+#define FOURCC__ART	GST_MAKE_FOURCC(0xa9,'A','R','T')
+#define FOURCC__alb	GST_MAKE_FOURCC(0xa9,'a','l','b')
+#define FOURCC_gnre	GST_MAKE_FOURCC('g','n','r','e')
+#define FOURCC_trkn	GST_MAKE_FOURCC('t','r','k','n')
+#define FOURCC_cpil	GST_MAKE_FOURCC('c','p','i','l')
+#define FOURCC_tmpo	GST_MAKE_FOURCC('t','m','p','o')
+#define FOURCC__too	GST_MAKE_FOURCC(0xa9,'t','o','o')
+#define FOURCC_____	GST_MAKE_FOURCC('-','-','-','-')
+#define FOURCC_free	GST_MAKE_FOURCC('f','r','e','e')
+#define FOURCC_data	GST_MAKE_FOURCC('d','a','t','a')
 
 
 static void qtdemux_dump_mvhd (GstQTDemux * qtdemux, void *buffer, int depth);
@@ -815,6 +937,8 @@ static void qtdemux_dump_stco (GstQTDemux * qtdemux, void *buffer, int depth);
 static void qtdemux_dump_co64 (GstQTDemux * qtdemux, void *buffer, int depth);
 static void qtdemux_dump_dcom (GstQTDemux * qtdemux, void *buffer, int depth);
 static void qtdemux_dump_cmvd (GstQTDemux * qtdemux, void *buffer, int depth);
+static void qtdemux_dump_unknown (GstQTDemux * qtdemux, void *buffer,
+    int depth);
 
 QtNodeType qt_node_types[] = {
   {FOURCC_moov, "movie", QT_CONTAINER,},
@@ -822,7 +946,7 @@ QtNodeType qt_node_types[] = {
       qtdemux_dump_mvhd},
   {FOURCC_clip, "clipping", QT_CONTAINER,},
   {FOURCC_trak, "track", QT_CONTAINER,},
-  {FOURCC_udta, "user data", 0,},       /* special container */
+  {FOURCC_udta, "user data", QT_CONTAINER,},    /* special container */
   {FOURCC_ctab, "color table", 0,},
   {FOURCC_tkhd, "track header", 0,
       qtdemux_dump_tkhd},
@@ -868,10 +992,29 @@ QtNodeType qt_node_types[] = {
       qtdemux_dump_co64},
   {FOURCC_vide, "video media", 0},
   {FOURCC_cmov, "compressed movie", QT_CONTAINER},
-  {FOURCC_dcom, "compressed data", 0,
-      qtdemux_dump_dcom},
-  {FOURCC_cmvd, "compressed movie data", 0,
-      qtdemux_dump_cmvd},
+  {FOURCC_dcom, "compressed data", 0, qtdemux_dump_dcom},
+  {FOURCC_cmvd, "compressed movie data", 0, qtdemux_dump_cmvd},
+  {FOURCC_hint, "hint", 0,},
+  {FOURCC_mp4a, "mp4a", 0,},
+  {FOURCC_mp4v, "mp4v", 0,},
+  {FOURCC_wave, "wave", QT_CONTAINER},
+  {FOURCC_appl, "appl", QT_CONTAINER},
+  {FOURCC_hnti, "hnti", QT_CONTAINER},
+  {FOURCC_rtp_, "rtp ", 0, qtdemux_dump_unknown},
+  {FOURCC_sdp_, "sdp ", 0, qtdemux_dump_unknown},
+  {FOURCC_meta, "meta", 0, qtdemux_dump_unknown},
+  {FOURCC_ilst, "ilst", QT_CONTAINER,},
+  {FOURCC__nam, "Name", QT_CONTAINER,},
+  {FOURCC__ART, "Artist", QT_CONTAINER,},
+  {FOURCC__alb, "Album", QT_CONTAINER,},
+  {FOURCC_gnre, "Genre", QT_CONTAINER,},
+  {FOURCC_trkn, "Track Number", QT_CONTAINER,},
+  {FOURCC_cpil, "cpil", QT_CONTAINER,},
+  {FOURCC_tmpo, "Tempo", QT_CONTAINER,},
+  {FOURCC__too, "too", QT_CONTAINER,},
+  {FOURCC_____, "----", QT_CONTAINER,},
+  {FOURCC_data, "data", 0, qtdemux_dump_unknown},
+  {FOURCC_free, "free", 0,},
   {0, "unknown", 0},
 };
 static int n_qt_node_types = sizeof (qt_node_types) / sizeof (qt_node_types[0]);
@@ -932,6 +1075,7 @@ qtdemux_parse_moov (GstQTDemux * qtdemux, void *buffer, int length)
 
   qtdemux->moov_node = g_node_new (buffer);
 
+  GST_DEBUG_OBJECT (qtdemux, "parsing 'moov' atom");
   qtdemux_parse (qtdemux, qtdemux->moov_node, buffer, length);
 
   cmov = qtdemux_tree_get_child_by_type (qtdemux->moov_node, FOURCC_cmov);
@@ -973,13 +1117,16 @@ qtdemux_parse (GstQTDemux * qtdemux, GNode * node, void *buffer, int length)
   QtNodeType *type;
   void *end;
 
-  GST_LOG ("qtdemux_parse %p %d", buffer, length);
+  GST_LOG ("qtdemux_parse buffer %p length %d", buffer, length);
 
   node_length = QTDEMUX_GUINT32_GET (buffer);
   fourcc = QTDEMUX_FOURCC_GET (buffer + 4);
 
   type = qtdemux_type_get (fourcc);
 
+  if (fourcc == 0 || node_length == 8)
+    return;
+
   GST_LOG ("parsing '" GST_FOURCC_FORMAT "', length=%d",
       GST_FOURCC_ARGS (fourcc), node_length);
 
@@ -997,6 +1144,10 @@ qtdemux_parse (GstQTDemux * qtdemux, GNode * node, void *buffer, int length)
         GST_LOG ("buffer overrun");
       }
       len = QTDEMUX_GUINT32_GET (buf);
+      if (len < 8) {
+        GST_LOG ("bad length");
+        break;
+      }
 
       child = g_node_new (buf);
       g_node_append (node, child);
@@ -1005,6 +1156,121 @@ qtdemux_parse (GstQTDemux * qtdemux, GNode * node, void *buffer, int length)
       buf += len;
     }
   } else {
+    if (fourcc == FOURCC_stsd) {
+      void *buf;
+      guint32 len;
+
+      gst_util_dump_mem (buffer, length);
+      GST_DEBUG_OBJECT (qtdemux,
+          "parsing stsd (sample table, sample description) atom");
+      buf = buffer + 16;
+      end = buffer + length;
+      while (buf < end) {
+        GNode *child;
+
+        if (buf + 8 >= end) {
+          /* FIXME: get annoyed */
+          GST_LOG ("buffer overrun");
+        }
+        len = QTDEMUX_GUINT32_GET (buf);
+
+        child = g_node_new (buf);
+        g_node_append (node, child);
+        qtdemux_parse (qtdemux, child, buf, len);
+
+        buf += len;
+      }
+    } else if (fourcc == FOURCC_mp4a) {
+      void *buf;
+      guint32 len;
+      guint32 version;
+
+      version = QTDEMUX_GUINT32_GET (buffer + 16);
+      if (version == 0x00010000) {
+        buf = buffer + 0x34;
+        end = buffer + length;
+        while (buf < end) {
+          GNode *child;
+
+          if (buf + 8 >= end) {
+            /* FIXME: get annoyed */
+            GST_LOG ("buffer overrun");
+          }
+          len = QTDEMUX_GUINT32_GET (buf);
+
+          child = g_node_new (buf);
+          g_node_append (node, child);
+          qtdemux_parse (qtdemux, child, buf, len);
+
+          buf += len;
+        }
+      }
+    } else if (fourcc == FOURCC_mp4v) {
+      void *buf;
+      guint32 len;
+      guint32 version;
+      int tlen;
+
+      g_print ("parsing in mp4v\n");
+      version = QTDEMUX_GUINT32_GET (buffer + 16);
+      g_print ("version %08x\n", version);
+      if (1 || version == 0x00000000) {
+
+        buf = buffer + 0x32;
+        end = buffer + length;
+
+        tlen = QTDEMUX_GUINT8_GET (buf);
+        g_print ("tlen = %d\n", tlen);
+        buf++;
+        g_print ("string = %.*s\n", tlen, (char *) buf);
+        buf += tlen;
+        buf += 23;
+
+        gst_util_dump_mem (buf, end - buf);
+        while (buf < end) {
+          GNode *child;
+
+          if (buf + 8 >= end) {
+            /* FIXME: get annoyed */
+            GST_LOG ("buffer overrun");
+          }
+          len = QTDEMUX_GUINT32_GET (buf);
+          if (len == 0)
+            break;
+
+          child = g_node_new (buf);
+          g_node_append (node, child);
+          qtdemux_parse (qtdemux, child, buf, len);
+
+          buf += len;
+        }
+      }
+    } else if (fourcc == FOURCC_meta) {
+      void *buf;
+      guint32 len;
+
+      buf = buffer + 12;
+      end = buffer + length;
+      while (buf < end) {
+        GNode *child;
+
+        if (buf + 8 >= end) {
+          /* FIXME: get annoyed */
+          GST_LOG ("buffer overrun");
+        }
+        len = QTDEMUX_GUINT32_GET (buf);
+        if (len < 8) {
+          GST_LOG ("bad length");
+          break;
+        }
+
+        child = g_node_new (buf);
+        g_node_append (node, child);
+        qtdemux_parse (qtdemux, child, buf, len);
+
+        buf += len;
+      }
+    }
 #if 0
     if (fourcc == FOURCC_cmvd) {
       int uncompressed_length;
@@ -1047,6 +1313,9 @@ qtdemux_type_get (guint32 fourcc)
     if (qt_node_types[i].fourcc == fourcc)
       return qt_node_types + i;
   }
+
+  GST_ERROR ("unknown QuickTime node type " GST_FOURCC_FORMAT,
+      GST_FOURCC_ARGS (fourcc));
   return qt_node_types + n_qt_node_types - 1;
 }
 
@@ -1059,8 +1328,8 @@ qtdemux_node_dump_foreach (GNode * node, gpointer data)
   QtNodeType *type;
   int depth;
 
-  node_length = GUINT32_FROM_BE (*(guint32 *) buffer);
-  fourcc = GUINT32_FROM_LE (*(guint32 *) (buffer + 4));
+  node_length = GST_READ_UINT32_BE (buffer);
+  fourcc = GST_READ_UINT32_LE (buffer + 4);
 
   type = qtdemux_type_get (fourcc);
 
@@ -1094,6 +1363,8 @@ qtdemux_dump_mvhd (GstQTDemux * qtdemux, void *buffer, int depth)
       QTDEMUX_GUINT32_GET (buffer + 20));
   GST_LOG ("%*s  duration:      %u", depth, "",
       QTDEMUX_GUINT32_GET (buffer + 24));
+  qtdemux->duration = QTDEMUX_GUINT32_GET (buffer + 24);
+  qtdemux->timescale = QTDEMUX_GUINT32_GET (buffer + 20);
   GST_LOG ("%*s  pref. rate:    %g", depth, "", QTDEMUX_FP32_GET (buffer + 28));
   GST_LOG ("%*s  pref. volume:  %g", depth, "", QTDEMUX_FP16_GET (buffer + 32));
   GST_LOG ("%*s  preview time:  %u", depth, "",
@@ -1354,7 +1625,7 @@ qtdemux_dump_stsc (GstQTDemux * qtdemux, void *buffer, int depth)
 static void
 qtdemux_dump_stsz (GstQTDemux * qtdemux, void *buffer, int depth)
 {
-  int i;
+  //int i;
   int n;
   int offset;
   int sample_size;
@@ -1369,19 +1640,21 @@ qtdemux_dump_stsz (GstQTDemux * qtdemux, void *buffer, int depth)
         QTDEMUX_GUINT32_GET (buffer + 16));
     n = QTDEMUX_GUINT32_GET (buffer + 16);
     offset = 20;
+#if 0
     for (i = 0; i < n; i++) {
       GST_LOG ("%*s    sample size:   %u", depth, "",
           QTDEMUX_GUINT32_GET (buffer + offset));
 
       offset += 4;
     }
+#endif
   }
 }
 
 static void
 qtdemux_dump_stco (GstQTDemux * qtdemux, void *buffer, int depth)
 {
-  int i;
+  //int i;
   int n;
   int offset;
 
@@ -1391,18 +1664,20 @@ qtdemux_dump_stco (GstQTDemux * qtdemux, void *buffer, int depth)
       QTDEMUX_GUINT32_GET (buffer + 12));
   n = QTDEMUX_GUINT32_GET (buffer + 12);
   offset = 16;
+#if 0
   for (i = 0; i < n; i++) {
     GST_LOG ("%*s    chunk offset:  %08x", depth, "",
         QTDEMUX_GUINT32_GET (buffer + offset));
 
     offset += 4;
   }
+#endif
 }
 
 static void
 qtdemux_dump_co64 (GstQTDemux * qtdemux, void *buffer, int depth)
 {
-  int i;
+  //int i;
   int n;
   int offset;
 
@@ -1412,12 +1687,14 @@ qtdemux_dump_co64 (GstQTDemux * qtdemux, void *buffer, int depth)
       QTDEMUX_GUINT32_GET (buffer + 12));
   n = QTDEMUX_GUINT32_GET (buffer + 12);
   offset = 16;
+#if 0
   for (i = 0; i < n; i++) {
     GST_LOG ("%*s    chunk offset:  %" G_GUINT64_FORMAT, depth, "",
         QTDEMUX_GUINT64_GET (buffer + offset));
 
     offset += 8;
   }
+#endif
 }
 
 static void
@@ -1433,6 +1710,18 @@ qtdemux_dump_cmvd (GstQTDemux * qtdemux, void *buffer, int depth)
   GST_LOG ("%*s  length: %d", depth, "", QTDEMUX_GUINT32_GET (buffer + 8));
 }
 
+static void
+qtdemux_dump_unknown (GstQTDemux * qtdemux, void *buffer, int depth)
+{
+  int len;
+
+  GST_LOG ("%*s  length: %d", depth, "", QTDEMUX_GUINT32_GET (buffer + 0));
+
+  len = QTDEMUX_GUINT32_GET (buffer + 0);
+  gst_util_dump_mem (buffer, len);
+
+}
+
 
 static GNode *
 qtdemux_tree_get_child_by_type (GNode * node, guint32 fourcc)
@@ -1445,7 +1734,7 @@ qtdemux_tree_get_child_by_type (GNode * node, guint32 fourcc)
       child = g_node_next_sibling (child)) {
     buffer = child->data;
 
-    child_fourcc = GUINT32_FROM_LE (*(guint32 *) (buffer + 4));
+    child_fourcc = GST_READ_UINT32_LE (buffer + 4);
 
     if (child_fourcc == fourcc) {
       return child;
@@ -1465,7 +1754,7 @@ qtdemux_tree_get_sibling_by_type (GNode * node, guint32 fourcc)
       child = g_node_next_sibling (child)) {
     buffer = child->data;
 
-    child_fourcc = GUINT32_FROM_LE (*(guint32 *) (buffer + 4));
+    child_fourcc = GST_READ_UINT32_LE (buffer + 4);
 
     if (child_fourcc == fourcc) {
       return child;
@@ -1481,6 +1770,20 @@ qtdemux_parse_tree (GstQTDemux * qtdemux)
 {
   GNode *mvhd;
   GNode *trak;
+  GNode *udta;
+
+  udta = qtdemux_tree_get_child_by_type (qtdemux->moov_node, FOURCC_udta);
+  if (udta) {
+    qtdemux_parse_udta (qtdemux, udta);
+
+    if (qtdemux->tag_list) {
+      g_print ("calling gst_element_found_tags with %s\n",
+          gst_structure_to_string (qtdemux->tag_list));
+      gst_element_found_tags (GST_ELEMENT (qtdemux), qtdemux->tag_list);
+    }
+  } else {
+    GST_LOG ("No udta node found.");
+  }
 
   mvhd = qtdemux_tree_get_child_by_type (qtdemux->moov_node, FOURCC_mvhd);
   if (mvhd == NULL) {
@@ -1502,6 +1805,8 @@ qtdemux_parse_tree (GstQTDemux * qtdemux)
 
   while ((trak = qtdemux_tree_get_sibling_by_type (trak, FOURCC_trak)) != NULL)
     qtdemux_parse_trak (qtdemux, trak);
+
+  gst_element_no_more_pads (GST_ELEMENT (qtdemux));
 }
 
 static void
@@ -1520,6 +1825,10 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
   GNode *stco;
   GNode *co64;
   GNode *stts;
+  GNode *mp4a;
+  GNode *mp4v;
+  GNode *wave;
+  GNode *esds;
   int n_samples;
   QtDemuxSample *samples;
   int n_samples_per_chunk;
@@ -1582,13 +1891,24 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
 
     fourcc = QTDEMUX_FOURCC_GET (stsd->data + offset + 4);
     stream->caps = qtdemux_video_caps (qtdemux, fourcc, stsd->data);
+
+    mp4v = qtdemux_tree_get_child_by_type (stsd, FOURCC_mp4v);
+    esds = NULL;
+    if (mp4v)
+      esds = qtdemux_tree_get_child_by_type (mp4v, FOURCC_esds);
+    if (esds) {
+      gst_qtdemux_handle_esds (qtdemux, stream, esds);
+    }
+
     GST_INFO ("type " GST_FOURCC_FORMAT " caps %" GST_PTR_FORMAT,
         GST_FOURCC_ARGS (QTDEMUX_FOURCC_GET (stsd->data + offset + 4)),
         stream->caps);
   } else if (stream->subtype == FOURCC_soun) {
     int version, samplesize;
     guint32 fourcc;
+    int len;
 
+    len = QTDEMUX_GUINT32_GET (stsd->data + 16);
     GST_LOG ("st type:          " GST_FOURCC_FORMAT,
         GST_FOURCC_ARGS (QTDEMUX_FOURCC_GET (stsd->data + 16 + 4)));
 
@@ -1628,7 +1948,7 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
           QTDEMUX_GUINT32_GET (stsd->data + offset + 12));
       stream->compression = 1;
       offset = 68;
-    } else {
+    } else if (version == 0x00000000) {
       stream->bytes_per_frame = stream->n_channels * samplesize / 8;
       stream->samples_per_packet = 1;
       stream->compression = 1;
@@ -1640,11 +1960,32 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
         stream->compression = 3;
       if (fourcc == GST_MAKE_FOURCC ('i', 'm', 'a', '4'))
         stream->compression = 4;
+    } else {
+      GST_ERROR ("unknown version %08x", version);
     }
 
-    stream->caps = qtdemux_audio_caps (qtdemux, fourcc,
-        (QTDEMUX_GUINT32_GET (stsd->data) >
-            offset) ? stsd->data + offset : NULL);
+    stream->caps = qtdemux_audio_caps (qtdemux, fourcc, NULL, 0);
+
+    mp4a = qtdemux_tree_get_child_by_type (stsd, FOURCC_mp4a);
+    wave = NULL;
+    if (mp4a)
+      wave = qtdemux_tree_get_child_by_type (mp4a, FOURCC_wave);
+    esds = NULL;
+    if (wave)
+      esds = qtdemux_tree_get_child_by_type (wave, FOURCC_esds);
+    if (esds) {
+      gst_qtdemux_handle_esds (qtdemux, stream, esds);
+#if 0
+      GstBuffer *buffer;
+      int len = QTDEMUX_GUINT32_GET (esds->data);
+
+      buffer = gst_buffer_new_and_alloc (len - 8);
+      memcpy (GST_BUFFER_DATA (buffer), esds->data + 8, len - 8);
+
+      gst_caps_set_simple (stream->caps, "codec_data",
+          GST_TYPE_BUFFER, buffer, NULL);
+#endif
+    }
     GST_INFO ("type " GST_FOURCC_FORMAT " caps %" GST_PTR_FORMAT,
         GST_FOURCC_ARGS (QTDEMUX_FOURCC_GET (stsd->data + 16 + 4)),
         stream->caps);
@@ -1794,30 +2135,31 @@ qtdemux_parse_trak (GstQTDemux * qtdemux, GNode * trak)
         sample_index += samples_per_chunk;
       }
     }
-/*
-done2:
-    n_sample_times = QTDEMUX_GUINT32_GET(stts->data + 12);
-    GST_LOG("n_sample_times = %d",n_sample_times);
+#if 0
+  done2:
+    n_sample_times = QTDEMUX_GUINT32_GET (stts->data + 12);
+    GST_LOG ("n_sample_times = %d", n_sample_times);
     timestamp = 0;
     index = 0;
     sample_index = 0;
-    for(i=0;i<n_sample_times;i++){
+    for (i = 0; i < n_sample_times; i++) {
       int duration;
       guint64 time;
-  
-      sample_index += QTDEMUX_GUINT32_GET(stts->data + 16 + 8*i);
-      duration = QTDEMUX_GUINT32_GET(stts->data + 16 + 8*i + 4);
-      for(;index < n_samples && samples[index].sample_index < sample_index;index++){
-	int size;
+
+      sample_index += QTDEMUX_GUINT32_GET (stts->data + 16 + 8 * i);
+      duration = QTDEMUX_GUINT32_GET (stts->data + 16 + 8 * i + 4);
+      for (; index < n_samples && samples[index].sample_index < sample_index;
+          index++) {
+        int size;
 
         samples[index].timestamp = timestamp;
-	size = samples[index+1].sample_index - samples[index].sample_index;
-	time = GST_SECOND / stream->rate; //(GST_SECOND * duration * samples[index].size)/stream->timescale ;
+        size = samples[index + 1].sample_index - samples[index].sample_index;
+        time = GST_SECOND / stream->rate;       //(GST_SECOND * duration * samples[index].size)/stream->timescale ;
         timestamp += time;
         samples[index].duration = time;
       }
     }
-*/
+#endif
   }
 done2:
 #if 0
@@ -1833,6 +2175,150 @@ done2:
   gst_qtdemux_add_stream (qtdemux, stream);
 }
 
+static void
+qtdemux_parse_udta (GstQTDemux * qtdemux, GNode * udta)
+{
+  GNode *meta;
+  GNode *ilst;
+  GNode *node;
+
+  meta = qtdemux_tree_get_child_by_type (udta, FOURCC_meta);
+  if (meta == NULL) {
+    GST_LOG ("no meta");
+    return;
+  }
+
+  ilst = qtdemux_tree_get_child_by_type (meta, FOURCC_ilst);
+  if (ilst == NULL) {
+    GST_LOG ("no ilst");
+    return;
+  }
+
+  g_print ("new tag list\n");
+  qtdemux->tag_list = gst_tag_list_new ();
+
+  node = qtdemux_tree_get_child_by_type (ilst, FOURCC__nam);
+  if (node) {
+    qtdemux_tag_add (qtdemux, GST_TAG_TITLE, node);
+  }
+
+  node = qtdemux_tree_get_child_by_type (ilst, FOURCC__ART);
+  if (node) {
+    qtdemux_tag_add (qtdemux, GST_TAG_ARTIST, node);
+  }
+
+  node = qtdemux_tree_get_child_by_type (ilst, FOURCC__alb);
+  if (node) {
+    qtdemux_tag_add (qtdemux, GST_TAG_ALBUM, node);
+  }
+
+
+}
+
+static void
+qtdemux_tag_add (GstQTDemux * qtdemux, const char *tag, GNode * node)
+{
+  GNode *data;
+  char *s;
+  int len;
+  int type;
+
+  data = qtdemux_tree_get_child_by_type (node, FOURCC_data);
+  if (data) {
+    len = QTDEMUX_GUINT32_GET (data->data);
+    type = QTDEMUX_GUINT32_GET (data->data + 8);
+    if (type == 0x00000001) {
+      s = g_strndup ((char *) data->data + 16, len - 16);
+      g_print ("adding tag %s\n", s);
+      gst_tag_list_add (qtdemux->tag_list, GST_TAG_MERGE_REPLACE, tag, s, NULL);
+      g_free (s);
+    }
+  }
+}
+
+/* taken from ffmpeg */
+static unsigned int
+get_size (guint8 * ptr, guint8 ** end)
+{
+  int count = 4;
+  int len = 0;
+
+  while (count--) {
+    int c = *ptr;
+
+    ptr++;
+    len = (len << 7) | (c & 0x7f);
+    if (!(c & 0x80))
+      break;
+  }
+  if (end)
+    *end = ptr;
+  return len;
+}
+
+static void
+gst_qtdemux_handle_esds (GstQTDemux * qtdemux, QtDemuxStream * stream,
+    GNode * esds)
+{
+  int len = QTDEMUX_GUINT32_GET (esds->data);
+  guint8 *ptr = esds->data;
+  guint8 *end = ptr + len;
+  int tag;
+  guint8 *data_ptr = NULL;
+  int data_len = 0;
+
+  gst_util_dump_mem (ptr, len);
+  ptr += 8;
+  g_print ("version/flags = %08x\n", QTDEMUX_GUINT32_GET (ptr));
+  ptr += 4;
+  while (ptr < end) {
+    tag = QTDEMUX_GUINT8_GET (ptr);
+    g_print ("tag = %02x\n", tag);
+    ptr++;
+    len = get_size (ptr, &ptr);
+    g_print ("len = %d\n", len);
+
+    switch (tag) {
+      case 0x03:
+        g_print ("ID %04x\n", QTDEMUX_GUINT16_GET (ptr));
+        g_print ("priority %04x\n", QTDEMUX_GUINT8_GET (ptr + 2));
+        ptr += 3;
+        break;
+      case 0x04:
+        g_print ("object_type_id %02x\n", QTDEMUX_GUINT8_GET (ptr));
+        g_print ("stream_type %02x\n", QTDEMUX_GUINT8_GET (ptr + 1));
+        g_print ("buffer_size_db %02x\n", QTDEMUX_GUINT24_GET (ptr + 2));
+        g_print ("max bitrate %d\n", QTDEMUX_GUINT32_GET (ptr + 5));
+        g_print ("avg bitrate %d\n", QTDEMUX_GUINT32_GET (ptr + 9));
+        ptr += 13;
+        break;
+      case 0x05:
+        g_print ("data:\n");
+        gst_util_dump_mem (ptr, len);
+        data_ptr = ptr;
+        data_len = len;
+        ptr += len;
+        break;
+      case 0x06:
+        g_print ("data %02x\n", QTDEMUX_GUINT8_GET (ptr));
+        ptr += 1;
+        break;
+      default:
+        g_print ("parse error\n");
+    }
+  }
+
+  if (data_ptr) {
+    GstBuffer *buffer;
+
+    buffer = gst_buffer_new_and_alloc (data_len);
+    memcpy (GST_BUFFER_DATA (buffer), data_ptr, data_len);
+    gst_util_dump_mem (GST_BUFFER_DATA (buffer), data_len);
+
+    gst_caps_set_simple (stream->caps, "codec_data", GST_TYPE_BUFFER,
+        buffer, NULL);
+  }
+}
 
 static GstCaps *
 qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
@@ -1935,23 +2421,37 @@ qtdemux_video_caps (GstQTDemux * qtdemux, guint32 fourcc,
       /* Cinepak */
       return gst_caps_from_string ("video/x-cinepak");
     case GST_MAKE_FOURCC ('r', 'p', 'z', 'a'):
+      /* Apple Video */
+      return gst_caps_from_string ("video/x-apple-video");
     case GST_MAKE_FOURCC ('r', 'l', 'e', ' '):
       /* Run-length encoding */
     case GST_MAKE_FOURCC ('s', 'm', 'c', ' '):
     case GST_MAKE_FOURCC ('k', 'p', 'c', 'd'):
     default:
+#if 0
       g_critical ("Don't know how to convert fourcc '" GST_FOURCC_FORMAT
           "' to caps\n", GST_FOURCC_ARGS (fourcc));
       return NULL;
+#endif
+      {
+        char *s;
+
+        s = g_strdup_printf ("video/x-gst-fourcc-" GST_FOURCC_FORMAT,
+            GST_FOURCC_ARGS (fourcc));
+        return gst_caps_new_simple (s, NULL);
+      }
   }
 }
 
 static GstCaps *
-qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data)
+qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data,
+    int len)
 {
   switch (fourcc) {
+#if 0
     case GST_MAKE_FOURCC ('N', 'O', 'N', 'E'):
       return NULL;              /*gst_caps_from_string ("audio/raw"); */
+#endif
     case GST_MAKE_FOURCC ('r', 'a', 'w', ' '):
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
@@ -1961,7 +2461,7 @@ qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data)
       return gst_caps_from_string ("audio/x-raw-int, "
           "width = (int) 16, "
           "depth = (int) 16, "
-          "endianness = (int) G_BIG_ENDIAN, " "signed = (boolean) true");
+          "endianness = (int) BIG_ENDIAN, " "signed = (boolean) true");
     case GST_MAKE_FOURCC ('s', 'o', 'w', 't'):
       /* FIXME */
       return gst_caps_from_string ("audio/x-raw-int, "
@@ -2021,16 +2521,17 @@ qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data)
       return gst_caps_from_string ("audio/x-dv");
     case GST_MAKE_FOURCC ('m', 'p', '4', 'a'):
       /* MPEG-4 AAC */
-      return gst_caps_from_string ("audio/mpeg, " "mpegversion = (int) 4");
+      return gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 4, NULL);
     case GST_MAKE_FOURCC ('Q', 'D', 'M', '2'):
       /* FIXME: QDesign music version 2 (no constant) */
-      if (QTDEMUX_GUINT32_GET (data) <= 100) {
-        gst_util_dump_mem ((guint8 *) data, 100);
-        return gst_caps_new_simple ("audio/x-qdm2",
-            "framesize", G_TYPE_INT, QTDEMUX_GUINT32_GET (data + 52),
-            "bitrate", G_TYPE_INT, QTDEMUX_GUINT32_GET (data + 40),
-            "blocksize", G_TYPE_INT, QTDEMUX_GUINT32_GET (data + 44), NULL);
-      }
+      return gst_caps_new_simple ("audio/x-qdm2",
+          "framesize", G_TYPE_INT, QTDEMUX_GUINT32_GET (data + 52),
+          "bitrate", G_TYPE_INT, QTDEMUX_GUINT32_GET (data + 40),
+          "blocksize", G_TYPE_INT, QTDEMUX_GUINT32_GET (data + 44), NULL);
+    case GST_MAKE_FOURCC ('a', 'g', 's', 'm'):
+      /* GSM */
+      return gst_caps_new_simple ("audio/x-gsm", NULL);
     case GST_MAKE_FOURCC ('q', 't', 'v', 'r'):
       /* ? */
     case GST_MAKE_FOURCC ('Q', 'D', 'M', 'C'):
@@ -2039,11 +2540,18 @@ qtdemux_audio_caps (GstQTDemux * qtdemux, guint32 fourcc, const guint8 * data)
       /* IMA 4:1 */
     case GST_MAKE_FOURCC ('Q', 'c', 'l', 'p'):
       /* QUALCOMM PureVoice */
-    case GST_MAKE_FOURCC ('a', 'g', 's', 'm'):
-      /* ? */
     default:
+#if 0
       g_critical ("Don't know how to convert fourcc '" GST_FOURCC_FORMAT
           "' to caps\n", GST_FOURCC_ARGS (fourcc));
       return NULL;
+#endif
+      {
+        char *s;
+
+        s = g_strdup_printf ("audio/x-gst-fourcc-" GST_FOURCC_FORMAT,
+            GST_FOURCC_ARGS (fourcc));
+        return gst_caps_new_simple (s, NULL);
+      }
   }
 }
