@@ -29,6 +29,7 @@
 #include <gst/gstelement.h>
 #include <gst/gstversion.h>
 #include <gst/gstinfo.h>
+#include <gst/gstutils.h>
 
 #include <string.h>
 #include <ctype.h>
@@ -326,10 +327,13 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
 
   while (skipped < GST_MP3_TYPEFIND_TRY_SYNC) {
     if (size <= 0) {
-      data = gst_type_find_peek (tf, skipped, GST_MP3_TYPEFIND_SYNC_SIZE);
+      size = GST_MP3_TYPEFIND_SYNC_SIZE * 2;
+      do {
+        size /= 2;
+        data = gst_type_find_peek (tf, skipped, size);
+      } while (size > 10 && !data);
       if (!data)
         break;
-      size = GST_MP3_TYPEFIND_SYNC_SIZE;
     }
     if (*data == 0xFF) {
       guint8 *head_data = NULL;
@@ -350,7 +354,7 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
         }
         if (!head_data)
           break;
-        head = GUINT32_FROM_BE (*((guint32 *) head_data));
+        head = GST_READ_UINT32_BE (head_data);
         if (!(length = mp3_type_frame_length_from_header (head, &layer,
                     &channels, &bitrate, &samplerate))) {
           GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
@@ -498,7 +502,7 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
         GST_LOG ("couldn't get MPEG pack header bytes");
         return 1;
       }
-      size = GUINT16_FROM_BE (*(guint16 *) data) + 6;
+      size = GST_READ_UINT16_BE (data) + 6;
       offset += 2;
       data = gst_type_find_peek (tf, offset, size - 6);
       if (!data) {
@@ -524,7 +528,7 @@ mpeg1_parse_header (GstTypeFind * tf, guint64 offset)
         GST_LOG ("couldn't get MPEG pack header bytes");
         return 1;
       }
-      size = GUINT16_FROM_BE (*(guint16 *) data) + 6;
+      size = GST_READ_UINT16_BE (data) + 6;
       /* FIXME: we could check PTS/DTS marker bits here... (bit overkill) */
       break;
   }
@@ -647,7 +651,7 @@ mpeg_video_stream_type_find (GstTypeFind * tf, gpointer unused)
     if (skipped > GST_MPEGVID_TYPEFIND_TRY_SYNC)
       break;
 
-    if (size < 4) {
+    if (size < 5) {
       data = gst_type_find_peek (tf, skipped, GST_MPEGVID_TYPEFIND_SYNC_SIZE);
       if (!data)
         break;
@@ -729,7 +733,7 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
       tip = GST_TYPE_FIND_MAXIMUM;
       break;
     }
-    offset += GUINT32_FROM_BE (*((guint32 *) data));
+    offset += GST_READ_UINT32_BE (data);
   }
   if (tip > 0) {
     gst_type_find_suggest (tf, tip, QT_CAPS);
@@ -772,6 +776,21 @@ shn_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** application/x-ape *********************************************/
+
+static GstStaticCaps ape_caps = GST_STATIC_CAPS ("application/x-ape");
+
+#define APE_CAPS gst_static_caps_get(&ape_caps)
+static void
+ape_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data && memcmp (data, "MAC ", 4) == 0) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APE_CAPS);
+  }
+}
+
 /*** audio/x-m4a *********************************************/
 
 static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/x-m4a");
@@ -1031,7 +1050,7 @@ matroska_type_find (GstTypeFind * tf, gpointer ununsed)
    * we don't parse the whole header but simply check for the
    * availability of that array of characters inside the header.
    * Not fully fool-proof, but good enough. */
-  for (n = 4 + size; n < 4 + size + total - sizeof (probe_data); n++)
+  for (n = 4 + size; n <= 4 + size + total - sizeof (probe_data); n++)
     if (!memcmp (&data[n], probe_data, sizeof (probe_data))) {
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, MATROSKA_CAPS);
       break;
@@ -1130,8 +1149,9 @@ theora_type_find (GstTypeFind * tf, gpointer private)
     if (memcmp (&data[1], "theora", 6) != 0)
       return;
     /* FIXME: make this more reliable when specs are out */
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, THEORA_CAPS);
   }
-  gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, THEORA_CAPS);
 }
 
 /*** audio/x-speex ***********************************************************/
@@ -1152,17 +1172,17 @@ speex_type_find (GstTypeFind * tf, gpointer private)
     data += 32;
 
     /* 4 byte header size >= 80 */
-    if (GINT32_FROM_LE (*((gint32 *) data)) < 80)
+    if (GST_READ_UINT32_LE (data) < 80)
       return;
     data += 4;
 
     /* 4 byte sample rate <= 48000 */
-    if (GINT32_FROM_LE (*((gint32 *) data)) > 48000)
+    if (GST_READ_UINT32_LE (data) > 48000)
       return;
     data += 4;
 
     /* currently there are only 3 speex modes. */
-    if (GINT32_FROM_LE (*((gint32 *) data)) > 3)
+    if (GST_READ_UINT32_LE (data) > 3)
       return;
     data += 12;
 
@@ -1178,7 +1198,8 @@ typedef struct
   guint size;
   guint probability;
   GstCaps *caps;
-} GstTypeFindData;
+}
+GstTypeFindData;
 static void
 start_with_type_find (GstTypeFind * tf, gpointer private)
 {
@@ -1266,6 +1287,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *wav_exts[] = { "wav", NULL };
   static gchar *aiff_exts[] = { "aiff", "aif", "aifc", NULL };
   static gchar *shn_exts[] = { "shn", NULL };
+  static gchar *ape_exts[] = { "ape", NULL };
   static gchar *uri_exts[] = { "ram", NULL };
   static gchar *jpeg_exts[] = { "jpg", "jpe", "jpeg", NULL };
   static gchar *gif_exts[] = { "gif", NULL };
@@ -1336,6 +1358,8 @@ plugin_init (GstPlugin * plugin)
       aiff_type_find, aiff_exts, AIFF_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-shorten", GST_RANK_SECONDARY,
       shn_type_find, shn_exts, SHN_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-ape", GST_RANK_SECONDARY,
+      ape_type_find, ape_exts, APE_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "image/jpeg", GST_RANK_PRIMARY, jpeg_type_find,
       jpeg_exts, JPEG_CAPS, NULL);
   TYPE_FIND_REGISTER_START_WITH (plugin, "image/gif", GST_RANK_PRIMARY,
