@@ -38,8 +38,8 @@ enum
 enum
 {
   ARG_0,
-  ARG_METHOD,
-  /* FILL ME */
+  ARG_METHOD
+      /* FILL ME */
 };
 
 static void gst_videofilter_base_init (gpointer g_class);
@@ -74,6 +74,7 @@ gst_videofilter_get_type (void)
       0,
       gst_videofilter_init,
     };
+
     videofilter_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstVideofilter", &videofilter_info, G_TYPE_FLAG_ABSTRACT);
   }
