@@ -75,7 +75,7 @@ struct wave_header
 
 static GstElementDetails gst_wavenc_details =
 GST_ELEMENT_DETAILS ("WAV encoder",
-    "Codec/Encoder/Audio",
+    "Codec/Muxer/Audio",
     "Encode raw audio into WAV",
     "Iain Holmes <iain@prettypeople.org>");
 
@@ -98,7 +98,7 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
 
 enum
 {
-  PROP_0,
+  PROP_0
 };
 
 static GstElementClass *parent_class = NULL;
@@ -303,23 +303,39 @@ struct _maps
 {
   guint32 id;
   char *name;
-} maps[] = {
+}
+maps[] =
+{
   {
-  GST_RIFF_INFO_IARL, "Location"}, {
-  GST_RIFF_INFO_IART, "Artist"}, {
-  GST_RIFF_INFO_ICMS, "Commissioner"}, {
-  GST_RIFF_INFO_ICMT, "Comment"}, {
-  GST_RIFF_INFO_ICOP, "Copyright"}, {
-  GST_RIFF_INFO_ICRD, "Creation Date"}, {
-  GST_RIFF_INFO_IENG, "Engineer"}, {
-  GST_RIFF_INFO_IGNR, "Genre"}, {
-  GST_RIFF_INFO_IKEY, "Keywords"}, {
-  GST_RIFF_INFO_INAM, "Title"}, /* Name */
+  GST_RIFF_INFO_IARL, "Location"}
+  , {
+  GST_RIFF_INFO_IART, "Artist"}
+  , {
+  GST_RIFF_INFO_ICMS, "Commissioner"}
+  , {
+  GST_RIFF_INFO_ICMT, "Comment"}
+  , {
+  GST_RIFF_INFO_ICOP, "Copyright"}
+  , {
+  GST_RIFF_INFO_ICRD, "Creation Date"}
+  , {
+  GST_RIFF_INFO_IENG, "Engineer"}
+  , {
+  GST_RIFF_INFO_IGNR, "Genre"}
+  , {
+  GST_RIFF_INFO_IKEY, "Keywords"}
+  , {
+  GST_RIFF_INFO_INAM, "Title"}
+  ,                             /* Name */
   {
-  GST_RIFF_INFO_IPRD, "Product"}, {
-  GST_RIFF_INFO_ISBJ, "Subject"}, {
-  GST_RIFF_INFO_ISFT, "Software"}, {
-  GST_RIFF_INFO_ITCH, "Technician"}, {
+  GST_RIFF_INFO_IPRD, "Product"}
+  , {
+  GST_RIFF_INFO_ISBJ, "Subject"}
+  , {
+  GST_RIFF_INFO_ISFT, "Software"}
+  , {
+  GST_RIFF_INFO_ITCH, "Technician"}
+  , {
   0, NULL}
 };
 
@@ -589,13 +605,6 @@ gst_wavenc_chain (GstPad * pad, GstData * _data)
 
   wavenc = GST_WAVENC (gst_pad_get_parent (pad));
 
-  if (!wavenc->setup) {
-    gst_buffer_unref (buf);
-    GST_ELEMENT_ERROR (wavenc, CORE, NEGOTIATION, (NULL),
-        ("encoder not initialised (input is not audio?)"));
-    return;
-  }
-
   if (GST_IS_EVENT (buf)) {
     if (GST_EVENT_TYPE (buf) == GST_EVENT_EOS) {
       wavenc->pad_eos = TRUE;
@@ -618,6 +627,13 @@ gst_wavenc_chain (GstPad * pad, GstData * _data)
     return;
   }
 
+  if (!wavenc->setup) {
+    gst_buffer_unref (buf);
+    GST_ELEMENT_ERROR (wavenc, CORE, NEGOTIATION, (NULL),
+        ("encoder not initialised (input is not audio?)"));
+    return;
+  }
+
   if (GST_PAD_IS_USABLE (wavenc->srcpad)) {
     if (wavenc->flush_header) {
       GstBuffer *outbuf;
