@@ -29,11 +29,24 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
-#include <sys/soundcard.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
 
+#ifdef HAVE_OSS_INCLUDE_IN_SYS
+#include <sys/soundcard.h>
+#else
+
+#ifdef HAVE_OSS_INCLUDE_IN_ROOT
+#include <soundcard.h>
+#else
+
+#include <machine/soundcard.h>
+
+#endif /* HAVE_OSS_INCLUDE_IN_ROOT */
+
+#endif /* HAVE_OSS_INCLUDE_IN_SYS */
+
 #include <gst/propertyprobe/propertyprobe.h>
 
 #include "gstosselement.h"
@@ -44,12 +57,12 @@ enum
   ARG_0,
   ARG_DEVICE,
   ARG_MIXERDEV,
-  ARG_DEVICE_NAME,
+  ARG_DEVICE_NAME
 };
 
 /* elementfactory information */
 static GstElementDetails gst_osselement_details =
-GST_ELEMENT_DETAILS ("Audio Mixer (OSS)",
+GST_ELEMENT_DETAILS ("OSS Mixer",
     "Generic/Audio",
     "OSS-based mixer element",
     "Ronald Bultje <rbultje@ronald.bitfreak.net>");
@@ -194,7 +207,7 @@ gst_osselement_probe (gchar * device_base,
     goto end;
   }
 
-  if (lstat (device, &s) || !S_ISCHR (s.st_mode))
+  if (stat (device, &s) || !S_ISCHR (s.st_mode))
     goto end;
 
   *name = device;
@@ -300,6 +313,7 @@ gst_osselement_class_probe_devices (GstOssElementClass * klass, gboolean check)
           combi = g_new0 (GstOssDeviceCombination, 1);
           combi->dsp = dsp;
           combi->mixer = mixer;
+          combi->dev = mixer ? mixer_dev : dsp_dev;
           device_combinations = device_combination_append (device_combinations,
               combi);
         } else {
@@ -440,7 +454,7 @@ gst_osselement_reset (GstOssElement * oss)
   oss->depth = 16;
   oss->channels = 2;
   oss->rate = 44100;
-  oss->fragment = 6;
+  oss->fragment = 0;
   oss->bps = 0;
 
 /* AFMT_*_BE not available on all OSS includes (e.g. FBSD) */
@@ -585,21 +599,26 @@ gst_osselement_sync_parms (GstOssElement * oss)
   gint target_format;
   gint target_channels;
   gint target_rate;
-  gint fragscale, frag_ln;
+
+  /* gint fragscale, frag_ln; */
 
   if (oss->fd == -1)
     return FALSE;
 
-  if (oss->fragment >> 16)
+  if ((oss->fragment & 0xFFFF) == 0) {
+    frag = 0;
+  } else if (oss->fragment >> 16) {
     frag = oss->fragment;
-  else
+  } else {
     frag = 0x7FFF0000 | oss->fragment;
+  }
 
   GST_INFO
       ("osselement: setting sound card to %dHz %d format %s (%08x fragment)",
       oss->rate, oss->format, (oss->channels == 2) ? "stereo" : "mono", frag);
 
-  ioctl (oss->fd, SNDCTL_DSP_SETFRAGMENT, &frag);
+  if (frag)
+    ioctl (oss->fd, SNDCTL_DSP_SETFRAGMENT, &frag);
   ioctl (oss->fd, SNDCTL_DSP_RESET, 0);
 
   target_format = oss->format;
@@ -618,6 +637,9 @@ gst_osselement_sync_parms (GstOssElement * oss)
     ioctl (oss->fd, SNDCTL_DSP_GETISPACE, &space);
   }
 
+#if 0
+  /* FIXME: make the current fragment info available somehow
+   * the current way overrides preset values and that sucks */
   /* calculate new fragment using a poor man's logarithm function */
   fragscale = 1;
   frag_ln = 0;
@@ -626,6 +648,7 @@ gst_osselement_sync_parms (GstOssElement * oss)
     frag_ln++;
   }
   oss->fragment = space.fragstotal << 16 | frag_ln;
+#endif
 
   GST_INFO ("osselement: set sound card to %dHz, %d format, %s "
       "(%d bytes buffer, %08x fragment)",
@@ -633,7 +656,7 @@ gst_osselement_sync_parms (GstOssElement * oss)
       (oss->channels == 2) ? "stereo" : "mono", space.bytes, oss->fragment);
 
   oss->fragment_time = (GST_SECOND * oss->fragment_size) / oss->bps;
-  GST_INFO ("fragment time %u %" G_GUINT64_FORMAT "\n",
+  GST_INFO ("fragment time %u %" G_GUINT64_FORMAT,
       oss->bps, oss->fragment_time);
 
   if (target_format != oss->format ||
@@ -642,7 +665,7 @@ gst_osselement_sync_parms (GstOssElement * oss)
       g_warning
           ("couldn't set the right number of channels (wanted %d, got %d), enjoy the tone difference",
           target_channels, oss->channels);
-    if (target_rate != oss->rate)
+    if (target_rate < oss->rate - 1 || target_rate > oss->rate + 1)
       g_warning
           ("couldn't set the right sample rate (wanted %d, got %d), enjoy the speed difference",
           target_rate, oss->rate);
@@ -806,6 +829,10 @@ static void
 gst_osselement_close_audio (GstOssElement * oss)
 {
   gst_ossmixer_free_list (oss);
+  if (oss->probed_caps) {
+    gst_caps_free (oss->probed_caps);
+    oss->probed_caps = NULL;
+  }
 
   if (oss->fd < 0)
     return;
@@ -967,3 +994,368 @@ gst_osselement_change_state (GstElement * element)
 
   return GST_STATE_SUCCESS;
 }
+
+
+/* rate probing code */
+
+
+#if 0
+
+#ifdef HAVE_OSS_INCLUDE_IN_SYS
+#include <sys/soundcard.h>
+#else
+
+#ifdef HAVE_OSS_INCLUDE_IN_ROOT
+#include <soundcard.h>
+#else
+
+#include <machine/soundcard.h>
+
+#endif /* HAVE_OSS_INCLUDE_IN_ROOT */
+
+#endif /* HAVE_OSS_INCLUDE_IN_SYS */
+
+#include <unistd.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <glib.h>
+#endif
+
+typedef struct _GstOssProbe GstOssProbe;
+struct _GstOssProbe
+{
+  int fd;
+  int format;
+  int n_channels;
+  GArray *rates;
+  int min;
+  int max;
+};
+
+typedef struct _GstOssRange GstOssRange;
+struct _GstOssRange
+{
+  int min;
+  int max;
+};
+
+static GstStructure *gst_osselement_get_format_structure (unsigned int
+    format_bit);
+static gboolean gst_osselement_rate_probe_check (GstOssProbe * probe);
+static int gst_osselement_rate_check_rate (GstOssProbe * probe, int irate);
+static void gst_osselement_rate_add_range (GQueue * queue, int min, int max);
+static void gst_osselement_rate_add_rate (GArray * array, int rate);
+static int gst_osselement_rate_int_compare (gconstpointer a, gconstpointer b);
+
+void
+gst_osselement_probe_caps (GstOssElement * oss)
+{
+  GstOssProbe *probe;
+  int i;
+  gboolean ret;
+  GstStructure *structure;
+  unsigned int format_bit;
+  unsigned int format_mask;
+  GstCaps *caps;
+
+  if (oss->probed_caps != NULL)
+    return;
+  if (oss->fd == -1)
+    return;
+
+  /* FIXME test make sure we're not currently playing */
+  /* FIXME test both mono and stereo */
+
+  format_mask = AFMT_U8 | AFMT_S16_LE | AFMT_S16_BE | AFMT_S8 |
+      AFMT_U16_LE | AFMT_U16_BE;
+  format_mask &= oss->caps;
+
+  caps = gst_caps_new_empty ();
+
+  /* assume that the most significant bit of format_mask is 0 */
+  for (format_bit = 1; format_bit <= format_mask; format_bit <<= 1) {
+    if (format_bit & format_mask) {
+      GValue rate_value = { 0 };
+
+      probe = g_new0 (GstOssProbe, 1);
+      probe->fd = oss->fd;
+      probe->format = format_bit;
+      probe->n_channels = 2;
+
+      ret = gst_osselement_rate_probe_check (probe);
+      if (probe->min == -1 || probe->max == -1) {
+        g_array_free (probe->rates, TRUE);
+        g_free (probe);
+        continue;
+      }
+
+      if (ret) {
+        GValue value = { 0 };
+
+        g_array_sort (probe->rates, gst_osselement_rate_int_compare);
+
+        g_value_init (&rate_value, GST_TYPE_LIST);
+        g_value_init (&value, G_TYPE_INT);
+
+        for (i = 0; i < probe->rates->len; i++) {
+          g_value_set_int (&value, g_array_index (probe->rates, int, i));
+
+          gst_value_list_append_value (&rate_value, &value);
+        }
+
+        g_value_unset (&value);
+      } else {
+        /* one big range */
+        g_value_init (&rate_value, GST_TYPE_INT_RANGE);
+        gst_value_set_int_range (&rate_value, probe->min, probe->max);
+      }
+
+      g_array_free (probe->rates, TRUE);
+      g_free (probe);
+
+      structure = gst_osselement_get_format_structure (format_bit);
+      gst_structure_set (structure, "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
+      gst_structure_set_value (structure, "rate", &rate_value);
+      g_value_unset (&rate_value);
+
+      gst_caps_append_structure (caps, structure);
+    }
+  }
+
+  if (gst_caps_is_empty (caps)) {
+    GST_ELEMENT_ERROR (oss, RESOURCE, SETTINGS,
+        (_("Your oss device could not be probed correctly")), (NULL));
+    return;
+  }
+  GST_DEBUG ("probed caps: %" GST_PTR_FORMAT, caps);
+  oss->probed_caps = caps;
+}
+
+static GstStructure *
+gst_osselement_get_format_structure (unsigned int format_bit)
+{
+  GstStructure *structure;
+  int endianness;
+  gboolean sign;
+  int width;
+
+  switch (format_bit) {
+    case AFMT_U8:
+      endianness = 0;
+      sign = FALSE;
+      width = 8;
+      break;
+    case AFMT_S16_LE:
+      endianness = G_LITTLE_ENDIAN;
+      sign = TRUE;
+      width = 16;
+      break;
+    case AFMT_S16_BE:
+      endianness = G_BIG_ENDIAN;
+      sign = TRUE;
+      width = 16;
+      break;
+    case AFMT_S8:
+      endianness = 0;
+      sign = TRUE;
+      width = 8;
+      break;
+    case AFMT_U16_LE:
+      endianness = G_LITTLE_ENDIAN;
+      sign = FALSE;
+      width = 16;
+      break;
+    case AFMT_U16_BE:
+      endianness = G_BIG_ENDIAN;
+      sign = FALSE;
+      width = 16;
+      break;
+    default:
+      g_assert_not_reached ();
+      return NULL;
+  }
+
+  structure = gst_structure_new ("audio/x-raw-int",
+      "width", G_TYPE_INT, width,
+      "depth", G_TYPE_INT, width, "signed", G_TYPE_BOOLEAN, sign, NULL);
+
+  if (endianness) {
+    gst_structure_set (structure, "endianness", G_TYPE_INT, endianness, NULL);
+  }
+
+  return structure;
+}
+
+static gboolean
+gst_osselement_rate_probe_check (GstOssProbe * probe)
+{
+  GstOssRange *range;
+  GQueue *ranges;
+  int exact_rates = 0;
+  gboolean checking_exact_rates = TRUE;
+  int n_checks = 0;
+  gboolean result = TRUE;
+
+  ranges = g_queue_new ();
+
+  probe->rates = g_array_new (FALSE, FALSE, sizeof (int));
+
+  probe->min = gst_osselement_rate_check_rate (probe, 1000);
+  n_checks++;
+  probe->max = gst_osselement_rate_check_rate (probe, 100000);
+  /* a little bug workaround */
+  {
+    int max;
+
+    max = gst_osselement_rate_check_rate (probe, 48000);
+    if (max > probe->max) {
+      GST_ERROR
+          ("Driver bug recognized (driver does not round rates correctly).  Please file a bug report.");
+      probe->max = max;
+    }
+  }
+  n_checks++;
+  if (probe->min == -1 || probe->max == -1) {
+    GST_DEBUG ("unexpected check_rate error");
+    return FALSE;
+  }
+  gst_osselement_rate_add_range (ranges, probe->min + 1, probe->max - 1);
+
+  while ((range = g_queue_pop_head (ranges))) {
+    int min1;
+    int max1;
+    int mid;
+    int mid_ret;
+
+    GST_DEBUG ("checking [%d,%d]", range->min, range->max);
+
+    mid = (range->min + range->max) / 2;
+    mid_ret = gst_osselement_rate_check_rate (probe, mid);
+    if (mid_ret == -1) {
+      /* FIXME ioctl returned an error.  do something */
+      GST_DEBUG ("unexpected check_rate error");
+    }
+    n_checks++;
+
+    if (mid == mid_ret && checking_exact_rates) {
+      int max_exact_matches = 20;
+
+      exact_rates++;
+      if (exact_rates > max_exact_matches) {
+        GST_DEBUG ("got %d exact rates, assuming all are exact",
+            max_exact_matches);
+        result = FALSE;
+        g_free (range);
+        break;
+      }
+    } else {
+      checking_exact_rates = FALSE;
+    }
+
+    /* Assume that the rate is arithmetically rounded to the nearest
+     * supported rate. */
+    if (mid == mid_ret) {
+      min1 = mid - 1;
+      max1 = mid + 1;
+    } else {
+      if (mid < mid_ret) {
+        min1 = mid - (mid_ret - mid);
+        max1 = mid_ret + 1;
+      } else {
+        min1 = mid_ret - 1;
+        max1 = mid + (mid - mid_ret);
+      }
+    }
+
+    gst_osselement_rate_add_range (ranges, range->min, min1);
+    gst_osselement_rate_add_range (ranges, max1, range->max);
+
+    g_free (range);
+  }
+
+  while ((range = g_queue_pop_head (ranges))) {
+    g_free (range);
+  }
+  g_queue_free (ranges);
+
+  return result;
+}
+
+static void
+gst_osselement_rate_add_range (GQueue * queue, int min, int max)
+{
+  if (min <= max) {
+    GstOssRange *range = g_new0 (GstOssRange, 1);
+
+    range->min = min;
+    range->max = max;
+
+    g_queue_push_tail (queue, range);
+    /* push_head also works, but has different probing behavior */
+    /*g_queue_push_head (queue, range); */
+  }
+}
+
+static int
+gst_osselement_rate_check_rate (GstOssProbe * probe, int irate)
+{
+  int rate;
+  int format;
+  int n_channels;
+  int ret;
+
+  rate = irate;
+  format = probe->format;
+  n_channels = probe->n_channels;
+
+  GST_LOG ("checking format %d, channels %d, rate %d",
+      format, n_channels, rate);
+  ret = ioctl (probe->fd, SNDCTL_DSP_SETFMT, &format);
+  if (ret < 0)
+    return -1;
+  ret = ioctl (probe->fd, SNDCTL_DSP_CHANNELS, &n_channels);
+  if (ret < 0)
+    return -1;
+  ret = ioctl (probe->fd, SNDCTL_DSP_SPEED, &rate);
+  if (ret < 0)
+    return -1;
+
+  GST_DEBUG ("rate %d -> %d", irate, rate);
+
+  if (rate == irate - 1 || rate == irate + 1) {
+    rate = irate;
+  }
+  gst_osselement_rate_add_rate (probe->rates, rate);
+  return rate;
+}
+
+static void
+gst_osselement_rate_add_rate (GArray * array, int rate)
+{
+  int i;
+  int val;
+
+  for (i = 0; i < array->len; i++) {
+    val = g_array_index (array, int, i);
+
+    if (val == rate)
+      return;
+  }
+  GST_DEBUG ("supported rate: %d", rate);
+  g_array_append_val (array, rate);
+}
+
+static int
+gst_osselement_rate_int_compare (gconstpointer a, gconstpointer b)
+{
+  const int *va = (const int *) a;
+  const int *vb = (const int *) b;
+
+  if (*va < *vb)
+    return -1;
+  if (*va > *vb)
+    return 1;
+  return 0;
+}
