@@ -30,7 +30,20 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+
+#ifdef HAVE_OSS_INCLUDE_IN_SYS
 #include <sys/soundcard.h>
+#else
+
+#ifdef HAVE_OSS_INCLUDE_IN_ROOT
+#include <soundcard.h>
+#else
+
+#include <machine/soundcard.h>
+
+#endif /* HAVE_OSS_INCLUDE_IN_ROOT */
+
+#endif /* HAVE_OSS_INCLUDE_IN_SYS */
 
 #include "gstossmixer.h"
 
@@ -74,37 +87,64 @@ fill_labels (void)
   struct
   {
     gchar *given, *wanted;
-  } cases[] = {
+  }
+  cases[] =
+  {
     /* Note: this list is simply ripped from soundcard.h. For
      * some people, some values might be missing (3D surround,
      * etc.) - feel free to add them. That's the reason why
      * I'm doing this in such a horribly complicated way. */
     {
-    "Vol  ", _("Volume")}, {
-    "Bass ", _("Bass")}, {
-    "Trebl", _("Treble")}, {
-    "Synth", _("Synth")}, {
-    "Pcm  ", _("PCM")}, {
-    "Spkr ", _("Speaker")}, {
-    "Line ", _("Line-in")}, {
-    "Mic  ", _("Microphone")}, {
-    "CD   ", _("CD")}, {
-    "Mix  ", _("Mixer")}, {
-    "Pcm2 ", _("PCM-2")}, {
-    "Rec  ", _("Record")}, {
-    "IGain", _("In-gain")}, {
-    "OGain", _("Out-gain")}, {
-    "Line1", _("Line-1")}, {
-    "Line2", _("Line-2")}, {
-    "Line3", _("Line-3")}, {
-    "Digital1", _("Digital-1")}, {
-    "Digital2", _("Digital-2")}, {
-    "Digital3", _("Digital-3")}, {
-    "PhoneIn", _("Phone-in")}, {
-    "PhoneOut", _("Phone-out")}, {
-    "Video", _("Video")}, {
-    "Radio", _("Radio")}, {
-    "Monitor", _("Monitor")}, {
+    "Vol  ", _("Volume")}
+    , {
+    "Bass ", _("Bass")}
+    , {
+    "Trebl", _("Treble")}
+    , {
+    "Synth", _("Synth")}
+    , {
+    "Pcm  ", _("PCM")}
+    , {
+    "Spkr ", _("Speaker")}
+    , {
+    "Line ", _("Line-in")}
+    , {
+    "Mic  ", _("Microphone")}
+    , {
+    "CD   ", _("CD")}
+    , {
+    "Mix  ", _("Mixer")}
+    , {
+    "Pcm2 ", _("PCM-2")}
+    , {
+    "Rec  ", _("Record")}
+    , {
+    "IGain", _("In-gain")}
+    , {
+    "OGain", _("Out-gain")}
+    , {
+    "Line1", _("Line-1")}
+    , {
+    "Line2", _("Line-2")}
+    , {
+    "Line3", _("Line-3")}
+    , {
+    "Digital1", _("Digital-1")}
+    , {
+    "Digital2", _("Digital-2")}
+    , {
+    "Digital3", _("Digital-3")}
+    , {
+    "PhoneIn", _("Phone-in")}
+    , {
+    "PhoneOut", _("Phone-out")}
+    , {
+    "Video", _("Video")}
+    , {
+    "Radio", _("Radio")}
+    , {
+    "Monitor", _("Monitor")}
+    , {
     NULL, NULL}
   };
 
