@@ -1,4 +1,7 @@
-/* G-Streamer BT8x8/V4L frame grabber plugin
+/* GStreamer
+ *
+ * gstv4lsrc.c: BT8x8/V4L source element
+ *
  * Copyright (C) 2001-2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +27,11 @@
 #include <string.h>
 #include <sys/time.h>
 #include "v4lsrc_calls.h"
+#include <sys/ioctl.h>
+
+/* FIXME: small cheat */
+gboolean gst_v4l_set_window_properties (GstV4lElement * v4lelement);
+gboolean gst_v4l_get_capabilities (GstV4lElement * v4lelement);
 
 /* elementfactory information */
 static GstElementDetails gst_v4lsrc_details =
@@ -32,6 +40,9 @@ GST_ELEMENT_DETAILS ("Video (video4linux/raw) Source",
     "Reads raw frames from a video4linux (BT8x8) device",
     "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
+GST_DEBUG_CATEGORY (v4lsrc_debug);
+#define GST_CAT_DEFAULT v4lsrc_debug
+
 /* V4lSrc signals and args */
 enum
 {
@@ -42,13 +53,17 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_SYNC_MODE GST_V4LSRC_SYNC_MODE_CLOCK
+#define DEFAULT_COPY_MODE FALSE
 /* arguments */
 enum
 {
   ARG_0,
   ARG_NUMBUFS,
   ARG_BUFSIZE,
-  ARG_USE_FIXED_FPS
+  ARG_SYNC_MODE,
+  ARG_COPY_MODE,
+  ARG_AUTOPROBE
 };
 
 GST_FORMATS_FUNCTION (GstPad *, gst_v4lsrc_get_formats,
@@ -56,6 +71,34 @@ GST_FORMATS_FUNCTION (GstPad *, gst_v4lsrc_get_formats,
 GST_QUERY_TYPE_FUNCTION (GstPad *, gst_v4lsrc_get_query_types,
     GST_QUERY_POSITION);
 
+#define GST_TYPE_V4LSRC_SYNC_MODE (gst_v4lsrc_sync_mode_get_type())
+static GType
+gst_v4lsrc_sync_mode_get_type (void)
+{
+  static GType v4lsrc_sync_mode_type = 0;
+  static GEnumValue v4lsrc_sync_mode[] = {
+    {GST_V4LSRC_SYNC_MODE_CLOCK, "0", "Sync to the pipeline clock"},
+    {GST_V4LSRC_SYNC_MODE_PRIVATE_CLOCK, "1", "Sync to a private clock"},
+    {GST_V4LSRC_SYNC_MODE_FIXED_FPS, "2", "Use Fixed fps"},
+    {GST_V4LSRC_SYNC_MODE_NONE, "3", "No Sync"},
+    {0, NULL, NULL},
+  };
+
+  if (!v4lsrc_sync_mode_type) {
+    v4lsrc_sync_mode_type =
+        g_enum_register_static ("GstV4lSrcSyncMode", v4lsrc_sync_mode);
+  }
+  return v4lsrc_sync_mode_type;
+}
+
+/* structure for buffer private data referencing element and frame number */
+typedef struct
+{
+  GstV4lSrc *v4lsrc;
+  int num;
+}
+v4lsrc_private_t;
+
 /* init functions */
 static void gst_v4lsrc_base_init (gpointer g_class);
 static void gst_v4lsrc_class_init (GstV4lSrcClass * klass);
@@ -73,8 +116,9 @@ static gboolean gst_v4lsrc_src_query (GstPad * pad,
     GstQueryType type, GstFormat * format, gint64 * value);
 
 /* buffer functions */
-static GstPadLinkReturn gst_v4lsrc_srcconnect (GstPad * pad,
+static GstPadLinkReturn gst_v4lsrc_src_link (GstPad * pad,
     const GstCaps * caps);
+static GstCaps *gst_v4lsrc_fixate (GstPad * pad, const GstCaps * caps);
 static GstCaps *gst_v4lsrc_getcaps (GstPad * pad);
 static GstData *gst_v4lsrc_get (GstPad * pad);
 
@@ -115,6 +159,7 @@ gst_v4lsrc_get_type (void)
       (GInstanceInitFunc) gst_v4lsrc_init,
       NULL
     };
+
     v4lsrc_type =
         g_type_register_static (GST_TYPE_V4LELEMENT, "GstV4lSrc", &v4lsrc_info,
         0);
@@ -131,7 +176,7 @@ gst_v4lsrc_base_init (gpointer g_class)
   gst_element_class_set_details (gstelement_class, &gst_v4lsrc_details);
 
   src_template = gst_pad_template_new ("src",
-      GST_PAD_SRC, GST_PAD_ALWAYS, NULL);
+      GST_PAD_SRC, GST_PAD_ALWAYS, gst_caps_new_any ());
 
   gst_element_class_add_pad_template (gstelement_class, src_template);
 }
@@ -151,14 +196,21 @@ gst_v4lsrc_class_init (GstV4lSrcClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NUMBUFS,
       g_param_spec_int ("num_buffers", "Num Buffers", "Number of buffers",
-          G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
+          0, G_MAXINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BUFSIZE,
       g_param_spec_int ("buffer_size", "Buffer Size", "Size of buffers",
-          G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USE_FIXED_FPS,
-      g_param_spec_boolean ("use_fixed_fps", "Use Fixed FPS",
-          "Drop/Insert frames to reach a certain FPS (TRUE) "
-          "or adapt FPS to suit the number of frabbed frames",
+          0, G_MAXINT, 0, G_PARAM_READABLE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC_MODE,
+      g_param_spec_enum ("sync_mode", "Sync mode",
+          "Method to use for timestamping captured frames",
+          GST_TYPE_V4LSRC_SYNC_MODE, DEFAULT_SYNC_MODE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_COPY_MODE,
+      g_param_spec_boolean ("copy_mode", "Copy mode",
+          "Don't send out HW buffers, send copy instead", DEFAULT_COPY_MODE,
+          G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_AUTOPROBE,
+      g_param_spec_boolean ("autoprobe", "Autoprobe",
+          "Whether the device should be probed for all possible features",
           TRUE, G_PARAM_READWRITE));
 
   /* signals */
@@ -175,6 +227,7 @@ gst_v4lsrc_class_init (GstV4lSrcClass * klass)
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstV4lSrcClass, frame_insert), NULL,
       NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
+  GST_DEBUG_CATEGORY_INIT (v4lsrc_debug, "v4lsrc", 0, "V4L source element");
   gobject_class->set_property = gst_v4lsrc_set_property;
   gobject_class->get_property = gst_v4lsrc_get_property;
 
@@ -186,7 +239,6 @@ gst_v4lsrc_class_init (GstV4lSrcClass * klass)
   v4lelement_class->close = gst_v4lsrc_close;
 }
 
-
 static void
 gst_v4lsrc_init (GstV4lSrc * v4lsrc)
 {
@@ -201,7 +253,8 @@ gst_v4lsrc_init (GstV4lSrc * v4lsrc)
 
   gst_pad_set_get_function (v4lsrc->srcpad, gst_v4lsrc_get);
   gst_pad_set_getcaps_function (v4lsrc->srcpad, gst_v4lsrc_getcaps);
-  gst_pad_set_link_function (v4lsrc->srcpad, gst_v4lsrc_srcconnect);
+  gst_pad_set_fixate_function (v4lsrc->srcpad, gst_v4lsrc_fixate);
+  gst_pad_set_link_function (v4lsrc->srcpad, gst_v4lsrc_src_link);
   gst_pad_set_convert_function (v4lsrc->srcpad, gst_v4lsrc_src_convert);
   gst_pad_set_formats_function (v4lsrc->srcpad, gst_v4lsrc_get_formats);
   gst_pad_set_query_function (v4lsrc->srcpad, gst_v4lsrc_src_query);
@@ -215,16 +268,21 @@ gst_v4lsrc_init (GstV4lSrc * v4lsrc)
   /* no colourspaces */
   v4lsrc->colourspaces = NULL;
 
-  /* fps */
-  v4lsrc->use_fixed_fps = TRUE;
+  v4lsrc->syncmode = DEFAULT_SYNC_MODE;
+  v4lsrc->copy_mode = DEFAULT_COPY_MODE;
 
   v4lsrc->is_capturing = FALSE;
+  v4lsrc->autoprobe = TRUE;
 }
 
 static void
 gst_v4lsrc_open (GstElement * element, const gchar * device)
 {
   GstV4lSrc *v4lsrc = GST_V4LSRC (element);
+  GstV4lElement *v4l = GST_V4LELEMENT (v4lsrc);
+  gint width = v4l->vcap.minwidth;
+  gint height = v4l->vcap.minheight;
+
   int palette[] = {
     VIDEO_PALETTE_YUV422,
     VIDEO_PALETTE_YUV420P,
@@ -242,7 +300,7 @@ gst_v4lsrc_open (GstElement * element, const gchar * device)
 
   for (i = 0; palette[i] != -1; i++) {
     /* try palette out */
-    if (!gst_v4lsrc_try_palette (v4lsrc, palette[i]))
+    if (!gst_v4lsrc_try_capture (v4lsrc, width, height, palette[i]))
       continue;
     v4lsrc->colourspaces = g_list_append (v4lsrc->colourspaces,
         GINT_TO_POINTER (palette[i]));
@@ -258,13 +316,94 @@ gst_v4lsrc_close (GstElement * element, const gchar * device)
   v4lsrc->colourspaces = NULL;
 }
 
+/* get a list of possible framerates
+ * this is only done for webcams;
+ * other devices return NULL here.
+ */
+static GValue *
+gst_v4lsrc_get_fps_list (GstV4lSrc * v4lsrc)
+{
+  gint fps_index;
+  gfloat fps;
+  struct video_window *vwin = &GST_V4LELEMENT (v4lsrc)->vwin;
+  GstV4lElement *v4lelement = GST_V4LELEMENT (v4lsrc);
+
+  /* check if we have vwin window properties giving a framerate,
+   * as is done for webcams
+   * See http://www.smcc.demon.nl/webcam/api.html
+   * which is used for the Philips and qce-ga drivers */
+  fps_index = (vwin->flags >> 16) & 0x3F;       /* 6 bit index for framerate */
+
+  /* webcams have a non-zero fps_index */
+  if (fps_index == 0) {
+    GST_DEBUG_OBJECT (v4lsrc, "fps_index is 0, no webcam");
+    return NULL;
+  }
+  GST_DEBUG_OBJECT (v4lsrc, "fps_index is %d, so webcam", fps_index);
+
+  {
+    gfloat current_fps;
+    int i;
+    GValue *list = NULL;
+    GValue value = { 0 };
+
+    /* webcam detected, so try all framerates and return a list */
+
+    list = g_new0 (GValue, 1);
+    g_value_init (list, GST_TYPE_LIST);
+
+    /* index of 16 corresponds to 15 fps */
+    current_fps = fps_index * 15.0 / 16;
+    GST_DEBUG_OBJECT (v4lsrc, "device reports fps of %.4f", current_fps);
+    for (i = 0; i < 63; ++i) {
+      /* set bits 16 to 21 to 0 */
+      vwin->flags &= (0x3F00 - 1);
+      /* set bits 16 to 21 to the index */
+      vwin->flags |= i << 16;
+      if (gst_v4l_set_window_properties (v4lelement)) {
+        /* setting it succeeded.  FIXME: get it and check. */
+        fps = i * 15.0 / 16;
+        g_value_init (&value, G_TYPE_DOUBLE);
+        g_value_set_double (&value, fps);
+        gst_value_list_append_value (list, &value);
+        g_value_unset (&value);
+      }
+    }
+    /* FIXME: set back the original fps_index */
+    vwin->flags &= (0x3F00 - 1);
+    vwin->flags |= fps_index << 16;
+    gst_v4l_set_window_properties (v4lelement);
+    return list;
+  }
+  return NULL;
+}
+
 static gfloat
 gst_v4lsrc_get_fps (GstV4lSrc * v4lsrc)
 {
   gint norm;
+  gint fps_index;
   gfloat fps;
+  struct video_window *vwin = &GST_V4LELEMENT (v4lsrc)->vwin;
 
-  if (!v4lsrc->use_fixed_fps && v4lsrc->clock != NULL && v4lsrc->handled > 0) {
+  /* check if we have vwin window properties giving a framerate,
+   * as is done for webcams
+   * See http://www.smcc.demon.nl/webcam/api.html
+   * which is used for the Philips and qce-ga drivers */
+  fps_index = (vwin->flags >> 16) & 0x3F;       /* 6 bit index for framerate */
+
+  /* webcams have a non-zero fps_index */
+  if (fps_index != 0) {
+    gfloat current_fps;
+
+    /* index of 16 corresponds to 15 fps */
+    current_fps = fps_index * 15.0 / 16;
+    GST_LOG_OBJECT (v4lsrc, "device reports fps of %.4f", current_fps);
+    return current_fps;
+  }
+
+  if (!(v4lsrc->syncmode == GST_V4LSRC_SYNC_MODE_FIXED_FPS) &&
+      v4lsrc->clock != NULL && v4lsrc->handled > 0) {
     /* try to get time from clock master and calculate fps */
     GstClockTime time =
         gst_clock_get_time (v4lsrc->clock) - v4lsrc->substract_time;
@@ -444,9 +583,8 @@ gst_v4lsrc_palette_to_caps (int palette)
   return caps;
 }
 
-
 static GstPadLinkReturn
-gst_v4lsrc_srcconnect (GstPad * pad, const GstCaps * vscapslist)
+gst_v4lsrc_src_link (GstPad * pad, const GstCaps * vscapslist)
 {
   GstV4lSrc *v4lsrc;
   guint32 fourcc;
@@ -454,8 +592,10 @@ gst_v4lsrc_srcconnect (GstPad * pad, const GstCaps * vscapslist)
   gdouble fps;
   GstStructure *structure;
   gboolean was_capturing;
+  struct video_window *vwin;
 
   v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
+  vwin = &GST_V4LELEMENT (v4lsrc)->vwin;
   was_capturing = v4lsrc->is_capturing;
 
   /* in case the buffers are active (which means that we already
@@ -482,6 +622,20 @@ gst_v4lsrc_srcconnect (GstPad * pad, const GstCaps * vscapslist)
   gst_structure_get_int (structure, "height", &h);
   gst_structure_get_double (structure, "framerate", &fps);
 
+  /* set framerate if it's not already correct */
+  if (fps != gst_v4lsrc_get_fps (v4lsrc)) {
+    int fps_index = fps / 15.0 * 16;
+
+    GST_DEBUG_OBJECT (v4lsrc, "Trying to set fps index %d", fps_index);
+    /* set bits 16 to 21 to 0 */
+    vwin->flags &= (0x3F00 - 1);
+    /* set bits 16 to 21 to the index */
+    vwin->flags |= fps_index << 16;
+    if (!gst_v4l_set_window_properties (GST_V4LELEMENT (v4lsrc))) {
+      GST_ELEMENT_ERROR (v4lsrc, RESOURCE, SETTINGS, (NULL),
+          ("Could not set framerate of %d fps", fps));
+    }
+  }
   switch (fourcc) {
     case GST_MAKE_FOURCC ('I', '4', '2', '0'):
       palette = VIDEO_PALETTE_YUV420P;
@@ -545,11 +699,46 @@ gst_v4lsrc_srcconnect (GstPad * pad, const GstCaps * vscapslist)
       break;
   }
 
-  if (palette == -1)
+  if (palette == -1) {
+    GST_WARNING_OBJECT (v4lsrc, "palette is -1, refusing link");
     return GST_PAD_LINK_REFUSED;
+  }
 
-  if (!gst_v4lsrc_set_capture (v4lsrc, w, h, palette))
+  GST_DEBUG_OBJECT (v4lsrc, "trying to set_capture %dx%d, palette %d",
+      w, h, palette);
+  if (!gst_v4lsrc_set_capture (v4lsrc, w, h, palette)) {
+    GST_WARNING_OBJECT (v4lsrc, "could not set_capture %dx%d, palette %d",
+        w, h, palette);
     return GST_PAD_LINK_REFUSED;
+  }
+
+  /* first try the negotiated settings using try_capture */
+  if (!gst_v4lsrc_try_capture (v4lsrc, w, h, palette)) {
+    GST_DEBUG_OBJECT (v4lsrc, "failed trying palette %d for w/h", palette);
+    return GST_PAD_LINK_REFUSED;
+  }
+  if (!gst_v4l_get_capabilities (GST_V4LELEMENT (v4lsrc)))
+    GST_DEBUG_OBJECT (v4lsrc, "failed getting capabilities");
+  GST_DEBUG_OBJECT (v4lsrc, "done checking, with w %d h %d palette %d\n",
+      vwin->width, vwin->height, palette);
+  if (vwin->width != v4lsrc->mmap.width) {
+    /* We consider this a device error since it was called with fixed
+     * caps that the device doesn't accept.  Still, there should be
+     * a way to handle this more gracefully, maybe provide a fixate
+     * function */
+    GST_ELEMENT_ERROR (v4lsrc, RESOURCE, SETTINGS, (NULL),
+        ("Tried setting %dx%d but got %dx%d back as suggestion",
+            v4lsrc->mmap.width, v4lsrc->mmap.height, vwin->width,
+            vwin->height));
+    return GST_PAD_LINK_REFUSED;
+  }
+  if (vwin->height != v4lsrc->mmap.height) {
+    GST_ELEMENT_ERROR (v4lsrc, RESOURCE, SETTINGS, (NULL),
+        ("Tried setting %dx%d but got %dx%d back as suggestion",
+            v4lsrc->mmap.width, v4lsrc->mmap.height, vwin->width,
+            vwin->height));
+    return GST_PAD_LINK_REFUSED;
+  }
 
   if (!gst_v4lsrc_capture_init (v4lsrc))
     return GST_PAD_LINK_REFUSED;
@@ -562,6 +751,37 @@ gst_v4lsrc_srcconnect (GstPad * pad, const GstCaps * vscapslist)
   return GST_PAD_LINK_OK;
 }
 
+static GstCaps *
+gst_v4lsrc_fixate (GstPad * pad, const GstCaps * caps)
+{
+  GstStructure *structure;
+  GstCaps *newcaps;
+  GstV4lSrc *v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
+  struct video_capability *vcap = &GST_V4LELEMENT (v4lsrc)->vcap;
+
+  if (gst_caps_get_size (caps) > 1)
+    return NULL;
+
+  if (!vcap) {
+    GST_DEBUG_OBJECT (v4lsrc, "don't have device caps, cannot fixate");
+    return NULL;
+  }
+  newcaps = gst_caps_copy (caps);
+  structure = gst_caps_get_structure (newcaps, 0);
+  GST_DEBUG_OBJECT (v4lsrc, "device reported w: %d-%d, h: %d-%d",
+      vcap->minwidth, vcap->maxwidth, vcap->minheight, vcap->maxheight);
+
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "width",
+          vcap->maxwidth)) {
+    return newcaps;
+  }
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "height",
+          vcap->maxheight)) {
+    return newcaps;
+  }
+  gst_caps_free (newcaps);
+  return NULL;
+}
 
 static GstCaps *
 gst_v4lsrc_getcaps (GstPad * pad)
@@ -569,11 +789,24 @@ gst_v4lsrc_getcaps (GstPad * pad)
   GstCaps *list;
   GstV4lSrc *v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
   struct video_capability *vcap = &GST_V4LELEMENT (v4lsrc)->vcap;
+  gfloat fps = 0.0;
   GList *item;
+  static GValue *fps_list;      /* FIXME: this should be done in a hash table
+                                 * on device name instead */
 
   if (!GST_V4L_IS_OPEN (GST_V4LELEMENT (v4lsrc))) {
     return gst_caps_new_any ();
   }
+  if (!v4lsrc->autoprobe) {
+    /* FIXME: query current caps and return those, with _any appended */
+    return gst_caps_new_any ();
+  }
+
+  /* if not cached from last run, get it */
+  if (!fps_list)
+    fps_list = gst_v4lsrc_get_fps_list (v4lsrc);
+  if (!fps_list)
+    fps = gst_v4lsrc_get_fps (v4lsrc);
 
   list = gst_caps_new_empty ();
   for (item = v4lsrc->colourspaces; item != NULL; item = item->next) {
@@ -582,10 +815,25 @@ gst_v4lsrc_getcaps (GstPad * pad)
     one = gst_v4lsrc_palette_to_caps (GPOINTER_TO_INT (item->data));
     if (!one)
       g_print ("Palette %d gave no caps\n", GPOINTER_TO_INT (item->data));
-    gst_caps_set_simple (one,
-        "width", GST_TYPE_INT_RANGE, vcap->minwidth, vcap->maxwidth,
-        "height", GST_TYPE_INT_RANGE, vcap->minheight, vcap->maxheight,
-        "framerate", G_TYPE_DOUBLE, gst_v4lsrc_get_fps (v4lsrc), NULL);
+    GST_DEBUG_OBJECT (v4lsrc,
+        "Device reports w: %d-%d, h: %d-%d, fps: %f for palette %d",
+        vcap->minwidth, vcap->maxwidth, vcap->minheight, vcap->maxheight, fps,
+        GPOINTER_TO_INT (item->data));
+
+    gst_caps_set_simple (one, "width", GST_TYPE_INT_RANGE, vcap->minwidth,
+        vcap->maxwidth, "height", GST_TYPE_INT_RANGE, vcap->minheight,
+        vcap->maxheight, NULL);
+
+    if (fps_list) {
+      GstStructure *structure = gst_caps_get_structure (one, 0);
+
+      gst_structure_set_value (structure, "framerate", fps_list);
+    } else {
+      GstStructure *structure = gst_caps_get_structure (one, 0);
+
+      gst_structure_set (structure, "framerate", G_TYPE_DOUBLE, fps, NULL);
+    }
+    GST_DEBUG_OBJECT (v4lsrc, "caps: %" GST_PTR_FORMAT, one);
     gst_caps_append (list, one);
   }
 
@@ -600,19 +848,25 @@ gst_v4lsrc_get (GstPad * pad)
   GstBuffer *buf;
   gint num;
   gdouble fps = 0.;
+  v4lsrc_private_t *v4lsrc_private = NULL;
+  GstClockTime now, until;
+  gboolean fixed_fps;
 
   g_return_val_if_fail (pad != NULL, NULL);
 
   v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
+  fps = gst_v4lsrc_get_fps (v4lsrc);
 
-  if (v4lsrc->use_fixed_fps && (fps = gst_v4lsrc_get_fps (v4lsrc)) == 0)
+  fixed_fps = v4lsrc->syncmode == GST_V4LSRC_SYNC_MODE_FIXED_FPS;
+
+  if (fixed_fps && fps == 0.0)
     return NULL;
 
   if (v4lsrc->need_writes > 0) {
     /* use last frame */
     num = v4lsrc->last_frame;
     v4lsrc->need_writes--;
-  } else if (v4lsrc->clock && v4lsrc->use_fixed_fps) {
+  } else if (v4lsrc->clock && fixed_fps) {
     GstClockTime time;
     gboolean have_frame = FALSE;
 
@@ -669,21 +923,100 @@ gst_v4lsrc_get (GstPad * pad)
 
   buf = gst_buffer_new ();
   GST_BUFFER_FREE_DATA_FUNC (buf) = gst_v4lsrc_buffer_free;
-  GST_BUFFER_PRIVATE (buf) = v4lsrc;    /* hack to re-queue buffer on free */
-  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY | GST_BUFFER_DONTFREE);
+  /* to requeue buffers, we need a ref to the element, as well as the frame
+     number we got this buffer from */
+  v4lsrc_private = g_malloc (sizeof (v4lsrc_private_t));
+  v4lsrc_private->v4lsrc = v4lsrc;
+  v4lsrc_private->num = num;
+  GST_BUFFER_PRIVATE (buf) = v4lsrc_private;
+
+  /* don't | the flags, they are integers, not bits!! */
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY);
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_DONTFREE);
   GST_BUFFER_DATA (buf) = gst_v4lsrc_get_buffer (v4lsrc, num);
   GST_BUFFER_MAXSIZE (buf) = v4lsrc->mbuf.size / v4lsrc->mbuf.frames;
   GST_BUFFER_SIZE (buf) = v4lsrc->buffer_size;
-  if (v4lsrc->use_fixed_fps)
-    GST_BUFFER_TIMESTAMP (buf) = v4lsrc->handled * GST_SECOND / fps;
-  else                          /* calculate time based on our own clock */
-    GST_BUFFER_TIMESTAMP (buf) =
-        v4lsrc->timestamp_sync - v4lsrc->substract_time;
+
+  if (v4lsrc->copy_mode) {
+    GstBuffer *copy = gst_buffer_copy (buf);
+
+    gst_buffer_unref (buf);
+    buf = copy;
+  }
+
+  switch (v4lsrc->syncmode) {
+    case GST_V4LSRC_SYNC_MODE_FIXED_FPS:
+      GST_BUFFER_TIMESTAMP (buf) = v4lsrc->handled * GST_SECOND / fps;
+      break;
+    case GST_V4LSRC_SYNC_MODE_PRIVATE_CLOCK:
+      /* calculate time based on our own clock */
+      GST_BUFFER_TIMESTAMP (buf) =
+          v4lsrc->timestamp_sync - v4lsrc->substract_time;
+      break;
+    case GST_V4LSRC_SYNC_MODE_CLOCK:
+      if (v4lsrc->clock) {
+        GstClockTime time = gst_element_get_time (GST_ELEMENT (v4lsrc));
+
+        /* FIXME: figure out a way to add the capture latency here */
+        GST_BUFFER_TIMESTAMP (buf) = time /* + 0.5 * GST_SECOND / fps */ ;
+      } else {
+        GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
+      }
+      break;
+    case GST_V4LSRC_SYNC_MODE_NONE:
+      GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
+      break;
+    default:
+      break;
+  }
+  GST_BUFFER_DURATION (buf) = GST_SECOND / fps;
+
+  GST_LOG_OBJECT (v4lsrc, "outgoing buffer duration: %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
 
   v4lsrc->handled++;
   g_signal_emit (G_OBJECT (v4lsrc),
       gst_v4lsrc_signals[SIGNAL_FRAME_CAPTURE], 0);
 
+  /* this is the current timestamp */
+  now = gst_element_get_time (GST_ELEMENT (v4lsrc));
+
+  until = GST_BUFFER_TIMESTAMP (buf);
+
+  GST_LOG_OBJECT (v4lsrc, "Current time %" GST_TIME_FORMAT
+      ", buffer timestamp %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (now), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+  if (now < until) {
+    GST_LOG_OBJECT (v4lsrc, "waiting until %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (until));
+    if (!gst_element_wait (GST_ELEMENT (v4lsrc), until))
+      g_warning ("waiting from now %" GST_TIME_FORMAT
+          " until %" GST_TIME_FORMAT " failed",
+          GST_TIME_ARGS (now), GST_TIME_ARGS (until));
+    GST_LOG_OBJECT (v4lsrc, "wait done.");
+  }
+  /* check for discont; we do it after grabbing so that we drop the
+   * first frame grabbed, but get an accurate discont event  */
+  if (v4lsrc->need_discont) {
+    GstEvent *event;
+
+    v4lsrc->need_discont = FALSE;
+
+    /* drop the buffer we made */
+    gst_buffer_unref (buf);
+
+    /* get time for discont */
+    now = gst_element_get_time (GST_ELEMENT (v4lsrc));
+    GST_DEBUG_OBJECT (v4lsrc, "sending time discont with %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (now));
+
+    /* store discont internally so we can wait when sending buffers too soon */
+    v4lsrc->last_discont = now;
+
+    /* return event */
+    event = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, now, NULL);
+    return GST_DATA (event);
+  }
   return GST_DATA (buf);
 }
 
@@ -698,9 +1031,19 @@ gst_v4lsrc_set_property (GObject * object,
   v4lsrc = GST_V4LSRC (object);
 
   switch (prop_id) {
-    case ARG_USE_FIXED_FPS:
+    case ARG_SYNC_MODE:
       if (!GST_V4L_IS_ACTIVE (GST_V4LELEMENT (v4lsrc))) {
-        v4lsrc->use_fixed_fps = g_value_get_boolean (value);
+        v4lsrc->syncmode = g_value_get_enum (value);
+      }
+      break;
+
+    case ARG_COPY_MODE:
+      v4lsrc->copy_mode = g_value_get_boolean (value);
+      break;
+
+    case ARG_AUTOPROBE:
+      if (!GST_V4L_IS_ACTIVE (GST_V4LELEMENT (v4lsrc))) {
+        v4lsrc->autoprobe = g_value_get_boolean (value);
       }
       break;
 
@@ -733,8 +1076,16 @@ gst_v4lsrc_get_property (GObject * object,
             v4lsrc->mbuf.size / (v4lsrc->mbuf.frames * 1024));
       break;
 
-    case ARG_USE_FIXED_FPS:
-      g_value_set_boolean (value, v4lsrc->use_fixed_fps);
+    case ARG_SYNC_MODE:
+      g_value_set_enum (value, v4lsrc->syncmode);
+      break;
+
+    case ARG_COPY_MODE:
+      g_value_set_boolean (value, v4lsrc->copy_mode);
+      break;
+
+    case ARG_AUTOPROBE:
+      g_value_set_boolean (value, v4lsrc->autoprobe);
       break;
 
     default:
@@ -760,6 +1111,8 @@ gst_v4lsrc_change_state (GstElement * element)
       break;
     case GST_STATE_READY_TO_PAUSED:
       v4lsrc->handled = 0;
+      v4lsrc->need_discont = TRUE;
+      v4lsrc->last_discont = 0;
       v4lsrc->need_writes = 0;
       v4lsrc->last_frame = 0;
       v4lsrc->substract_time = 0;
@@ -825,24 +1178,27 @@ gst_v4lsrc_buffer_new (GstBufferPool * pool,
 static void
 gst_v4lsrc_buffer_free (GstBuffer * buf)
 {
-  GstV4lSrc *v4lsrc = GST_V4LSRC (GST_BUFFER_PRIVATE (buf));
-  int n;
+  v4lsrc_private_t *v4lsrc_private;
+  GstV4lSrc *v4lsrc;
+  int num;
 
+  v4lsrc_private = GST_BUFFER_PRIVATE (buf);
+  g_assert (v4lsrc_private);
+  v4lsrc = v4lsrc_private->v4lsrc;
+  num = v4lsrc_private->num;
+  g_free (v4lsrc_private);
+  GST_BUFFER_PRIVATE (buf) = 0;
+
+  GST_LOG_OBJECT (v4lsrc, "freeing buffer %p with refcount %d for frame %d",
+      buf, GST_BUFFER_REFCOUNT_VALUE (buf), num);
   if (gst_element_get_state (GST_ELEMENT (v4lsrc)) != GST_STATE_PLAYING)
     return;                     /* we've already cleaned up ourselves */
 
-  for (n = 0; n < v4lsrc->mbuf.frames; n++)
-    if (GST_BUFFER_DATA (buf) == gst_v4lsrc_get_buffer (v4lsrc, n)) {
-      v4lsrc->use_num_times[n]--;
-      if (v4lsrc->use_num_times[n] <= 0) {
-        gst_v4lsrc_requeue_frame (v4lsrc, n);
-      }
-      break;
-    }
-
-  if (n == v4lsrc->mbuf.frames)
-    GST_ELEMENT_ERROR (v4lsrc, RESOURCE, TOO_LAZY, (NULL),
-        ("Couldn\'t find the buffer"));
+  v4lsrc->use_num_times[num]--;
+  if (v4lsrc->use_num_times[num] <= 0) {
+    GST_LOG_OBJECT (v4lsrc, "requeueing frame %d", num);
+    gst_v4lsrc_requeue_frame (v4lsrc, num);
+  }
 }
 
 
