@@ -1,4 +1,7 @@
-/* G-Streamer generic V4L element - generic V4L calls handling
+/* GStreamer
+ *
+ * v4l_calls.c: generic V4L calls
+ *
  * Copyright (C) 2001-2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -42,11 +45,8 @@
 #include "gstv4lmjpegsrc.h"
 #include "gstv4lmjpegsink.h"
 
-#define DEBUG(format, args...) \
-	GST_DEBUG_OBJECT (\
-		GST_ELEMENT(v4lelement), \
-		"V4L: " format, ##args)
-
+GST_DEBUG_CATEGORY_EXTERN (v4l_debug);
+#define GST_CAT_DEFAULT v4l_debug
 
 static const char *picture_name[] = {
   "Hue",
@@ -76,10 +76,10 @@ static const char *norm_name[] = {
  * return value: TRUE on success, FALSE on error
  ******************************************************/
 
-static gboolean
+gboolean
 gst_v4l_get_capabilities (GstV4lElement * v4lelement)
 {
-  DEBUG ("getting capabilities");
+  GST_DEBUG_OBJECT (v4lelement, "getting capabilities");
   GST_V4L_CHECK_OPEN (v4lelement);
 
   if (ioctl (v4lelement->video_fd, VIDIOCGCAP, &(v4lelement->vcap)) < 0) {
@@ -89,9 +89,53 @@ gst_v4l_get_capabilities (GstV4lElement * v4lelement)
     return FALSE;
   }
 
+  if (ioctl (v4lelement->video_fd, VIDIOCGWIN, &(v4lelement->vwin)) < 0) {
+    GST_ELEMENT_ERROR (v4lelement, RESOURCE, SETTINGS, (NULL),
+        ("error getting window properties %s of from device %s",
+            g_strerror (errno), v4lelement->videodev));
+    return FALSE;
+  }
+
   return TRUE;
 }
 
+/******************************************************
+ * gst_v4l_set_window_properties():
+ *   set the device's capturing parameters (vwin)
+ * return value: TRUE on success, FALSE on error
+ ******************************************************/
+
+gboolean
+gst_v4l_set_window_properties (GstV4lElement * v4lelement)
+{
+  struct video_window vwin;
+
+  GST_DEBUG_OBJECT (v4lelement, "setting window flags 0x%x to device %s",
+      v4lelement->vwin.flags, v4lelement->videodev);
+  GST_V4L_CHECK_OPEN (v4lelement);
+
+  if (ioctl (v4lelement->video_fd, VIDIOCSWIN, &(v4lelement->vwin)) < 0) {
+    GST_DEBUG_OBJECT (v4lelement,
+        "could not ioctl window properties 0x%x to device %s",
+        v4lelement->vwin.flags, v4lelement->videodev);
+    return FALSE;
+  }
+
+  /* get it again to make sure we have it correctly */
+  if (ioctl (v4lelement->video_fd, VIDIOCGWIN, &(vwin)) < 0) {
+    GST_ELEMENT_ERROR (v4lelement, RESOURCE, SETTINGS, (NULL),
+        ("error getting window properties %s of from device %s",
+            g_strerror (errno), v4lelement->videodev));
+    return FALSE;
+  }
+  if (vwin.flags != v4lelement->vwin.flags) {
+    GST_DEBUG_OBJECT (v4lelement, "set 0x%x but got 0x%x back",
+        v4lelement->vwin.flags, vwin.flags);
+    return FALSE;
+  }
+
+  return TRUE;
+}
 
 /******************************************************
  * gst_v4l_open():
@@ -104,7 +148,7 @@ gst_v4l_open (GstV4lElement * v4lelement)
 {
   int num;
 
-  DEBUG ("opening device %s", v4lelement->videodev);
+  GST_DEBUG_OBJECT (v4lelement, "opening device %s", v4lelement->videodev);
   GST_V4L_CHECK_NOT_OPEN (v4lelement);
   GST_V4L_CHECK_NOT_ACTIVE (v4lelement);
 
@@ -118,6 +162,11 @@ gst_v4l_open (GstV4lElement * v4lelement)
   /* open the device */
   v4lelement->video_fd = open (v4lelement->videodev, O_RDWR);
   if (!GST_V4L_IS_OPEN (v4lelement)) {
+    if (errno == ENODEV || errno == ENOENT) {
+      GST_ELEMENT_ERROR (v4lelement, RESOURCE, NOT_FOUND,
+          (_("Device \"%s\" does not exist."), v4lelement->videodev), (NULL));
+      return FALSE;
+    }
     GST_ELEMENT_ERROR (v4lelement, RESOURCE, OPEN_READ_WRITE,
         (_("Could not open device \"%s\" for reading and writing."),
             v4lelement->videodev), GST_ERROR_SYSTEM);
@@ -145,7 +194,7 @@ gst_v4l_open (GstV4lElement * v4lelement)
     return FALSE;
   }
 
-  gst_info ("Opened device \'%s\' (\'%s\') successfully\n",
+  GST_INFO_OBJECT (v4lelement, "Opened device \'%s\' (\'%s\') successfully",
       v4lelement->vcap.name, v4lelement->videodev);
 
   /* norms + inputs, for the tuner interface */
@@ -173,7 +222,7 @@ gst_v4l_open (GstV4lElement * v4lelement)
     v4lelement->colors = g_list_append (v4lelement->colors, channel);
   }
 
-  DEBUG ("Setting default norm/input");
+  GST_DEBUG_OBJECT (v4lelement, "Setting default norm/input");
   gst_v4l_set_chan_norm (v4lelement, 0, 0);
 
   return TRUE;
@@ -189,7 +238,7 @@ gst_v4l_open (GstV4lElement * v4lelement)
 gboolean
 gst_v4l_close (GstV4lElement * v4lelement)
 {
-  DEBUG ("closing device");
+  GST_DEBUG_OBJECT (v4lelement, "closing device");
   GST_V4L_CHECK_OPEN (v4lelement);
   GST_V4L_CHECK_NOT_ACTIVE (v4lelement);
 
@@ -214,13 +263,13 @@ gst_v4l_close (GstV4lElement * v4lelement)
 
 /******************************************************
  * gst_v4l_get_num_chans()
- * return value: the numver of video input channels
+ * return value: the number of video input channels
  ******************************************************/
 
 gint
 gst_v4l_get_num_chans (GstV4lElement * v4lelement)
 {
-  DEBUG ("getting number of channels");
+  GST_DEBUG_OBJECT (v4lelement, "getting number of channels");
   GST_V4L_CHECK_OPEN (v4lelement);
 
   return v4lelement->vcap.channels;
@@ -240,7 +289,7 @@ gst_v4l_get_chan_names (GstV4lElement * v4lelement)
   GList *list = NULL;
   gint i;
 
-  DEBUG ("getting channel names");
+  GST_DEBUG_OBJECT (v4lelement, "getting channel names");
 
   if (!GST_V4L_IS_OPEN (v4lelement))
     return NULL;
@@ -272,6 +321,8 @@ gst_v4l_get_chan_names (GstV4lElement * v4lelement)
         if (!strcmp (vtun.name, vchan.name)) {
           v4lchannel->tuner = n;
           channel->flags |= GST_TUNER_CHANNEL_FREQUENCY;
+          channel->freq_multiplicator =
+              62.5 * ((vtun.flags & VIDEO_TUNER_LOW) ? 1 : 1000);
           channel->min_frequency = vtun.rangelow;
           channel->max_frequency = vtun.rangehigh;
           channel->min_signal = 0;
@@ -312,7 +363,7 @@ gst_v4l_get_chan_names (GstV4lElement * v4lelement)
 gboolean
 gst_v4l_get_chan_norm (GstV4lElement * v4lelement, gint * channel, gint * norm)
 {
-  DEBUG ("getting current channel and norm");
+  GST_DEBUG_OBJECT (v4lelement, "getting current channel and norm");
   GST_V4L_CHECK_OPEN (v4lelement);
 
   if (channel)
@@ -334,7 +385,7 @@ gst_v4l_get_chan_norm (GstV4lElement * v4lelement, gint * channel, gint * norm)
 gboolean
 gst_v4l_set_chan_norm (GstV4lElement * v4lelement, gint channel, gint norm)
 {
-  DEBUG ("setting channel = %d, norm = %d (%s)",
+  GST_DEBUG_OBJECT (v4lelement, "setting channel = %d, norm = %d (%s)",
       channel, norm, norm_name[norm]);
   GST_V4L_CHECK_OPEN (v4lelement);
   GST_V4L_CHECK_NOT_ACTIVE (v4lelement);
@@ -369,7 +420,7 @@ gst_v4l_get_signal (GstV4lElement * v4lelement, gint tunernum, guint * signal)
 {
   struct video_tuner tuner;
 
-  DEBUG ("getting tuner signal");
+  GST_DEBUG_OBJECT (v4lelement, "getting tuner signal");
   GST_V4L_CHECK_OPEN (v4lelement);
 
   tuner.tuner = tunernum;
@@ -396,10 +447,13 @@ gst_v4l_get_frequency (GstV4lElement * v4lelement,
     gint tunernum, gulong * frequency)
 {
   struct video_tuner vtun;
+  GstTunerChannel *channel;
 
-  DEBUG ("getting tuner frequency");
+  GST_DEBUG_OBJECT (v4lelement, "getting tuner frequency");
   GST_V4L_CHECK_OPEN (v4lelement);
 
+  channel = gst_tuner_get_channel (GST_TUNER (v4lelement));
+
   /* check that this is the current input */
   vtun.tuner = tunernum;
   if (ioctl (v4lelement->video_fd, VIDIOCGTUNER, &vtun) < 0)
@@ -413,6 +467,8 @@ gst_v4l_get_frequency (GstV4lElement * v4lelement,
     return FALSE;
   }
 
+  *frequency = *frequency * channel->freq_multiplicator;
+
   return TRUE;
 }
 
@@ -428,10 +484,13 @@ gst_v4l_set_frequency (GstV4lElement * v4lelement,
     gint tunernum, gulong frequency)
 {
   struct video_tuner vtun;
+  GstTunerChannel *channel;
 
-  DEBUG ("setting tuner frequency to %lu", frequency);
+  GST_DEBUG_OBJECT (v4lelement, "setting tuner frequency to %lu", frequency);
   GST_V4L_CHECK_OPEN (v4lelement);
 
+  channel = gst_tuner_get_channel (GST_TUNER (v4lelement));
+
   /* check that this is the current input */
   vtun.tuner = tunernum;
   if (ioctl (v4lelement->video_fd, VIDIOCGTUNER, &vtun) < 0)
@@ -439,6 +498,8 @@ gst_v4l_set_frequency (GstV4lElement * v4lelement,
   if (strcmp (vtun.name, v4lelement->vchan.name))
     return FALSE;
 
+  frequency = frequency / channel->freq_multiplicator;
+
   if (ioctl (v4lelement->video_fd, VIDIOCSFREQ, &frequency) < 0) {
     GST_ELEMENT_ERROR (v4lelement, RESOURCE, SETTINGS, (NULL),
         ("Error setting tuner frequency: %s", g_strerror (errno)));
@@ -461,7 +522,8 @@ gst_v4l_get_picture (GstV4lElement * v4lelement,
 {
   struct video_picture vpic;
 
-  DEBUG ("getting picture property type %d (%s)", type, picture_name[type]);
+  GST_DEBUG_OBJECT (v4lelement, "getting picture property type %d (%s)", type,
+      picture_name[type]);
   GST_V4L_CHECK_OPEN (v4lelement);
 
   if (ioctl (v4lelement->video_fd, VIDIOCGPICT, &vpic) < 0) {
@@ -505,7 +567,7 @@ gst_v4l_set_picture (GstV4lElement * v4lelement,
 {
   struct video_picture vpic;
 
-  DEBUG ("setting picture type %d (%s) to value %d",
+  GST_DEBUG_OBJECT (v4lelement, "setting picture type %d (%s) to value %d",
       type, picture_name[type], value);
   GST_V4L_CHECK_OPEN (v4lelement);
 
@@ -556,7 +618,8 @@ gst_v4l_get_audio (GstV4lElement * v4lelement,
 {
   struct video_audio vau;
 
-  DEBUG ("getting audio parameter type %d (%s)", type, audio_name[type]);
+  GST_DEBUG_OBJECT (v4lelement, "getting audio parameter type %d (%s)", type,
+      audio_name[type]);
   GST_V4L_CHECK_OPEN (v4lelement);
 
   vau.audio = audionum;
@@ -598,8 +661,9 @@ gst_v4l_set_audio (GstV4lElement * v4lelement,
 {
   struct video_audio vau;
 
-  DEBUG ("setting audio parameter type %d (%s) to value %d",
-      type, audio_name[type], value);
+  GST_DEBUG_OBJECT (v4lelement,
+      "setting audio parameter type %d (%s) to value %d", type,
+      audio_name[type], value);
   GST_V4L_CHECK_OPEN (v4lelement);
 
   vau.audio = audionum;
